@@ -5,7 +5,13 @@
 #ifdef _MSC_VER
 #include <io.h>
 #else
+
+#if defined(__SVR4) && defined(__sun)
+#include <inttypes.h>
+#else
 #include <stdint.h>
 #endif
 
 #endif
+
+#endif
