@@ -22,12 +22,14 @@ import bzrlib.osutils as osutils
 
 
 class WalkDirsBenchmark(Benchmark):
+    """Benchmarks for `bzrlib.osutils.walkdirs`"""
 
     def test_walkdirs_kernel_like_tree(self):
         """Walking a kernel sized tree is fast!(150ms)."""
         self.make_kernel_like_tree(link_working=True)
         # on roberts machine: this originally took:  157ms/4177ms
         # plain os.walk takes 213ms on this tree
+        # with the pyrex readdir module:              77ms/5423ms
         def dowalk():
             for dirblock in osutils.walkdirs('.'):
                 if dirblock[0][1] == '.bzr':
