@@ -1,8 +1,9 @@
-# Copyright (C) 2006 by Canonical Ltd
+# Copyright (C) 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,10 +23,12 @@ from bzrlib.osutils import pathjoin
 from bzrlib.transform import TreeTransform
 from bzrlib.workingtree import WorkingTree
 
+
 class TransformBenchmark(Benchmark):
+    """Benchmarks for `bzrlib.transform`"""
 
     def test_canonicalize_path(self):
-        """Canonicalizing paths should be fast.""" 
+        """Canonicalizing paths should be fast."""
         wt = self.make_kernel_like_tree(link_working=True)
         paths = []
         for dirpath, dirnames, filenames in os.walk('.'):
