@@ -25,6 +25,9 @@ from bzrlib.trace import note
 
 
 def install_bundle(repository, bundle_reader):
+    custom_install = getattr(bundle_reader, 'install', None)
+    if custom_install is not None:
+        return custom_install(repository)
     pb = bzrlib.ui.ui_factory.nested_progress_bar()
     repository.lock_write()
     try:
@@ -41,7 +44,7 @@ def install_bundle(repository, bundle_reader):
         pb.finished()
 
 
-def merge_bundle(reader, tree, check_clean, merge_type, 
+def merge_bundle(reader, tree, check_clean, merge_type,
                     reprocess, show_base, change_reporter=None):
     """Merge a revision bundle into the current tree."""
     pb = bzrlib.ui.ui_factory.nested_progress_bar()
