@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,51 +43,83 @@ from bzrlib import (
     errors,
     option,
     osutils,
-    registry,
     trace,
     win32utils,
     )
 """)
 
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    deprecated_method,
-    )
+from bzrlib import registry
 # Compatibility
+from bzrlib.hooks import Hooks
 from bzrlib.option import Option
 
 
-plugin_cmds = {}
+class CommandInfo(object):
+    """Information about a command."""
+
+    def __init__(self, aliases):
+        """The list of aliases for the command."""
+        self.aliases = aliases
+
+    @classmethod
+    def from_command(klass, command):
+        """Factory to construct a CommandInfo from a command."""
+        return klass(command.aliases)
+
+
+class CommandRegistry(registry.Registry):
+
+    @staticmethod
+    def _get_name(command_name):
+        if command_name.startswith("cmd_"):
+            return _unsquish_command_name(command_name)
+        else:
+            return command_name
+
+    def register(self, cmd, decorate=False):
+        """Utility function to help register a command
+
+        :param cmd: Command subclass to register
+        :param decorate: If true, allow overriding an existing command
+            of the same name; the old command is returned by this function.
+            Otherwise it is an error to try to override an existing command.
+        """
+        k = cmd.__name__
+        k_unsquished = self._get_name(k)
+        try:
+            previous = self.get(k_unsquished)
+        except KeyError:
+            previous = _builtin_commands().get(k_unsquished)
+        info = CommandInfo.from_command(cmd)
+        try:
+            registry.Registry.register(self, k_unsquished, cmd,
+                                       override_existing=decorate, info=info)
+        except KeyError:
+            trace.log_error('Two plugins defined the same command: %r' % k)
+            trace.log_error('Not loading the one in %r' %
+                            sys.modules[cmd.__module__])
+            trace.log_error('Previously this command was registered from %r' %
+                            sys.modules[previous.__module__])
+        return previous
+
+    def register_lazy(self, command_name, aliases, module_name):
+        """Register a command without loading its module.
+
+        :param command_name: The primary name of the command.
+        :param aliases: A list of aliases for the command.
+        :module_name: The module that the command lives in.
+        """
+        key = self._get_name(command_name)
+        registry.Registry.register_lazy(self, key, module_name, command_name,
+                                        info=CommandInfo(aliases))
+
+
+plugin_cmds = CommandRegistry()
 
 
 def register_command(cmd, decorate=False):
-    """Utility function to help register a command
-
-    :param cmd: Command subclass to register
-    :param decorate: If true, allow overriding an existing command
-        of the same name; the old command is returned by this function.
-        Otherwise it is an error to try to override an existing command.
-    """
     global plugin_cmds
-    k = cmd.__name__
-    if k.startswith("cmd_"):
-        k_unsquished = _unsquish_command_name(k)
-    else:
-        k_unsquished = k
-    if k_unsquished not in plugin_cmds:
-        plugin_cmds[k_unsquished] = cmd
-        ## trace.mutter('registered plugin command %s', k_unsquished)
-        if decorate and k_unsquished in builtin_command_names():
-            return _builtin_commands()[k_unsquished]
-    elif decorate:
-        result = plugin_cmds[k_unsquished]
-        plugin_cmds[k_unsquished] = cmd
-        return result
-    else:
-        trace.log_error('Two plugins defined the same command: %r' % k)
-        trace.log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
-        trace.log_error('Previously this command was registered from %r' %
-                        sys.modules[plugin_cmds[k_unsquished].__module__])
+    return plugin_cmds.register(cmd, decorate)
 
 
 def _squish_command_name(cmd):
@@ -107,12 +139,12 @@ def _builtin_commands():
             real_name = _unsquish_command_name(name)
             r[real_name] = builtins[name]
     return r
-            
+
 
 def builtin_command_names():
     """Return list of builtin command names."""
     return _builtin_commands().keys()
-    
+
 
 def plugin_command_names():
     return plugin_cmds.keys()
@@ -122,10 +154,10 @@ def _get_cmd_dict(plugins_override=True):
     """Return name->class mapping for all commands."""
     d = _builtin_commands()
     if plugins_override:
-        d.update(plugin_cmds)
+        d.update(plugin_cmds.iteritems())
     return d
 
-    
+
 def get_all_cmds(plugins_override=True):
     """Return canonical name and class for all registered commands."""
     for k, v in _get_cmd_dict(plugins_override=plugins_override).iteritems():
@@ -139,7 +171,11 @@ def get_cmd_object(cmd_name, plugins_override=True):
         If true, plugin commands can override builtins.
     """
     try:
-        return _get_cmd_object(cmd_name, plugins_override)
+        cmd = _get_cmd_object(cmd_name, plugins_override)
+        # Allow plugins to extend commands
+        for hook in Command.hooks['extend_command']:
+            hook(cmd)
+        return cmd
     except KeyError:
         raise errors.BzrCommandError('unknown command "%s"' % cmd_name)
 
@@ -154,12 +190,21 @@ def _get_cmd_object(cmd_name, plugins_override=True):
     # In the future, we may actually support Unicode command names.
 
     # first look up this command under the specified name
-    cmds = _get_cmd_dict(plugins_override=plugins_override)
+    if plugins_override:
+        try:
+            return plugin_cmds.get(cmd_name)()
+        except KeyError:
+            pass
+    cmds = _get_cmd_dict(plugins_override=False)
     try:
         return cmds[cmd_name]()
     except KeyError:
         pass
-
+    if plugins_override:
+        for key in plugin_cmds.keys():
+            info = plugin_cmds.get_info(key)
+            if cmd_name in info.aliases:
+                return plugin_cmds.get(key)()
     # look for any command which claims this as an alias
     for real_cmd_name, cmd_class in cmds.iteritems():
         if cmd_name in cmd_class.aliases:
@@ -176,9 +221,8 @@ def _get_cmd_object(cmd_name, plugins_override=True):
         except errors.NoPluginAvailable:
             pass
         else:
-            raise errors.CommandAvailableInPlugin(cmd_name, 
+            raise errors.CommandAvailableInPlugin(cmd_name,
                                                   plugin_metadata, provider)
-
     raise KeyError
 
 
@@ -239,6 +283,7 @@ class Command(object):
             sys.stdout is forced to be a binary stream, and line-endings
             will not mangled.
 
+    :cvar hooks: An instance of CommandHooks.
     """
     aliases = []
     takes_args = []
@@ -246,7 +291,7 @@ class Command(object):
     encoding_type = 'strict'
 
     hidden = False
-    
+
     def __init__(self):
         """Construct an instance of this command."""
         if self.__doc__ == Command.__doc__:
@@ -256,7 +301,7 @@ class Command(object):
 
     def _maybe_expand_globs(self, file_list):
         """Glob expand file_list if the platform does not do that itself.
-        
+
         :return: A possibly empty list of unicode paths.
 
         Introduced in bzrlib 0.18.
@@ -288,7 +333,7 @@ class Command(object):
     def get_help_text(self, additional_see_also=None, plain=True,
                       see_also_as_links=False):
         """Return a text string with help for this command.
-        
+
         :param additional_see_also: Additional help topics to be
             cross-referenced.
         :param plain: if False, raw help (reStructuredText) is
@@ -301,7 +346,7 @@ class Command(object):
             raise NotImplementedError("sorry, no detailed help yet for %r" % self.name())
 
         # Extract the summary (purpose) and sections out from the text
-        purpose,sections = self._get_help_parts(doc)
+        purpose,sections,order = self._get_help_parts(doc)
 
         # If a custom usage section was provided, use it
         if sections.has_key('Usage'):
@@ -339,9 +384,9 @@ class Command(object):
         # Add the custom sections (e.g. Examples). Note that there's no need
         # to indent these as they must be indented already in the source.
         if sections:
-            labels = sorted(sections.keys())
-            for label in labels:
-                result += ':%s:\n%s\n\n' % (label,sections[label])
+            for label in order:
+                if sections.has_key(label):
+                    result += ':%s:\n%s\n\n' % (label,sections[label])
 
         # Add the aliases, source (plug-in) and see also links, if any
         if self.aliases:
@@ -366,7 +411,7 @@ class Command(object):
             result += ':See also: '
             result += ', '.join(see_also) + '\n'
 
-        # If this will be rendered as plan text, convert it
+        # If this will be rendered as plain text, convert it
         if plain:
             import bzrlib.help_topics
             result = bzrlib.help_topics.help_as_plain_text(result)
@@ -376,38 +421,41 @@ class Command(object):
     def _get_help_parts(text):
         """Split help text into a summary and named sections.
 
-        :return: (summary,sections) where summary is the top line and
+        :return: (summary,sections,order) where summary is the top line and
             sections is a dictionary of the rest indexed by section name.
+            order is the order the section appear in the text.
             A section starts with a heading line of the form ":xxx:".
             Indented text on following lines is the section value.
             All text found outside a named section is assigned to the
             default section which is given the key of None.
         """
-        def save_section(sections, label, section):
+        def save_section(sections, order, label, section):
             if len(section) > 0:
                 if sections.has_key(label):
                     sections[label] += '\n' + section
                 else:
+                    order.append(label)
                     sections[label] = section
-            
+
         lines = text.rstrip().splitlines()
         summary = lines.pop(0)
         sections = {}
+        order = []
         label,section = None,''
         for line in lines:
             if line.startswith(':') and line.endswith(':') and len(line) > 2:
-                save_section(sections, label, section)
+                save_section(sections, order, label, section)
                 label,section = line[1:-1],''
             elif (label is not None) and len(line) > 1 and not line[0].isspace():
-                save_section(sections, label, section)
+                save_section(sections, order, label, section)
                 label,section = None,line
             else:
                 if len(section) > 0:
                     section += '\n' + line
                 else:
                     section = line
-        save_section(sections, label, section)
-        return summary, sections
+        save_section(sections, order, label, section)
+        return summary, sections, order
 
     def get_help_topic(self):
         """Return the commands help topic - its name."""
@@ -415,7 +463,7 @@ class Command(object):
 
     def get_see_also(self, additional_terms=None):
         """Return a list of help topics that are related to this command.
-        
+
         The list is derived from the content of the _see_also attribute. Any
         duplicates are removed and the result is in lexical order.
         :param additional_terms: Additional help topics to cross-reference.
@@ -497,7 +545,7 @@ class Command(object):
         self._setup_outf()
 
         return self.run(**all_cmd_args)
-    
+
     def run(self):
         """Actually run the command.
 
@@ -533,9 +581,28 @@ class Command(object):
             return None
 
 
+class CommandHooks(Hooks):
+    """Hooks related to Command object creation/enumeration."""
+
+    def __init__(self):
+        """Create the default hooks.
+
+        These are all empty initially, because by default nothing should get
+        notified.
+        """
+        Hooks.__init__(self)
+        # Introduced in 1.13:
+        # invoked after creating a command object to allow modifications such
+        # as adding or removing options, docs etc. Invoked with the command
+        # object.
+        self['extend_command'] = []
+
+Command.hooks = CommandHooks()
+
+
 def parse_args(command, argv, alias_argv=None):
     """Parse command line.
-    
+
     Arguments and options are parsed at this level before being passed
     down to specific command handlers.  This routine knows, from a
     lookup table, something about the available options, what optargs
@@ -590,7 +657,7 @@ def _match_argform(cmd, takes_args, args):
                                % (cmd, argname.upper()))
             else:
                 argdict[argname] = args.pop(0)
-            
+
     if args:
         raise errors.BzrCommandError("extra argument to command %s: %s"
                                      % (cmd, args[0]))
@@ -605,12 +672,13 @@ def apply_coveraged(dirname, the_callable, *args, **kwargs):
     tracer = trace.Trace(count=1, trace=0)
     sys.settrace(tracer.globaltrace)
 
-    ret = the_callable(*args, **kwargs)
-
-    sys.settrace(None)
-    results = tracer.results()
-    results.write_results(show_missing=1, summary=False,
-                          coverdir=dirname)
+    try:
+        return exception_to_return_code(the_callable, *args, **kwargs)
+    finally:
+        sys.settrace(None)
+        results = tracer.results()
+        results.write_results(show_missing=1, summary=False,
+                              coverdir=dirname)
 
 
 def apply_profiled(the_callable, *args, **kwargs):
@@ -621,7 +689,8 @@ def apply_profiled(the_callable, *args, **kwargs):
     try:
         prof = hotshot.Profile(pfname)
         try:
-            ret = prof.runcall(the_callable, *args, **kwargs) or 0
+            ret = prof.runcall(exception_to_return_code, the_callable, *args,
+                **kwargs) or 0
         finally:
             prof.close()
         stats = hotshot.stats.load(pfname)
@@ -636,9 +705,50 @@ def apply_profiled(the_callable, *args, **kwargs):
         os.remove(pfname)
 
 
+def exception_to_return_code(the_callable, *args, **kwargs):
+    """UI level helper for profiling and coverage.
+
+    This transforms exceptions into a return value of 3. As such its only
+    relevant to the UI layer, and should never be called where catching
+    exceptions may be desirable.
+    """
+    try:
+        return the_callable(*args, **kwargs)
+    except (KeyboardInterrupt, Exception), e:
+        # used to handle AssertionError and KeyboardInterrupt
+        # specially here, but hopefully they're handled ok by the logger now
+        exc_info = sys.exc_info()
+        exitcode = trace.report_exception(exc_info, sys.stderr)
+        if os.environ.get('BZR_PDB'):
+            print '**** entering debugger'
+            tb = exc_info[2]
+            import pdb
+            if sys.version_info[:2] < (2, 6):
+                # XXX: we want to do
+                #    pdb.post_mortem(tb)
+                # but because pdb.post_mortem gives bad results for tracebacks
+                # from inside generators, we do it manually.
+                # (http://bugs.python.org/issue4150, fixed in Python 2.6)
+
+                # Setup pdb on the traceback
+                p = pdb.Pdb()
+                p.reset()
+                p.setup(tb.tb_frame, tb)
+                # Point the debugger at the deepest frame of the stack
+                p.curindex = len(p.stack) - 1
+                p.curframe = p.stack[p.curindex][0]
+                # Start the pdb prompt.
+                p.print_stack_entry(p.stack[p.curindex])
+                p.execRcLines()
+                p.cmdloop()
+            else:
+                pdb.post_mortem(tb)
+        return exitcode
+
+
 def apply_lsprofiled(filename, the_callable, *args, **kwargs):
     from bzrlib.lsprof import profile
-    ret, stats = profile(the_callable, *args, **kwargs)
+    ret, stats = profile(exception_to_return_code, the_callable, *args, **kwargs)
     stats.sort()
     if filename is None:
         stats.pprint()
@@ -675,14 +785,11 @@ def get_alias(cmd, config=None):
 def run_bzr(argv):
     """Execute a command.
 
-    This is similar to main(), but without all the trappings for
-    logging and error handling.  
-    
     argv
        The command-line arguments, without the program name from argv[0]
        These should already be decoded. All library/test code calling
        run_bzr should be passing valid strings (don't need decoding).
-    
+
     Returns a command status or raises an exception.
 
     Special master options: these must come before the command because
@@ -745,6 +852,8 @@ def run_bzr(argv):
             argv_copy.append(a)
         i += 1
 
+    debug.set_debug_flags_from_config()
+
     argv = argv_copy
     if (not argv):
         from bzrlib.builtins import cmd_help
@@ -755,7 +864,7 @@ def run_bzr(argv):
         from bzrlib.builtins import cmd_version
         cmd_version().run_argv_aliases([])
         return 0
-        
+
     if not opt_no_plugins:
         from bzrlib.plugin import load_plugins
         load_plugins()
@@ -768,7 +877,8 @@ def run_bzr(argv):
     if not opt_no_aliases:
         alias_argv = get_alias(argv[0])
         if alias_argv:
-            alias_argv = [a.decode(bzrlib.user_encoding) for a in alias_argv]
+            user_encoding = osutils.get_user_encoding()
+            alias_argv = [a.decode(user_encoding) for a in alias_argv]
             argv[0] = alias_argv.pop(0)
 
     cmd = argv.pop(0)
@@ -781,6 +891,10 @@ def run_bzr(argv):
     run_argv = [argv, alias_argv]
 
     try:
+        # We can be called recursively (tests for example), but we don't want
+        # the verbosity level to propagate.
+        saved_verbosity_level = option._verbosity_level
+        option._verbosity_level = 0
         if opt_lsprof:
             if opt_coverage_dir:
                 trace.warning(
@@ -796,20 +910,14 @@ def run_bzr(argv):
         else:
             ret = run(*run_argv)
         if 'memory' in debug.debug_flags:
-            try:
-                status_file = file('/proc/%s/status' % os.getpid(), 'rb')
-            except IOError:
-                pass
-            else:
-                status = status_file.read()
-                status_file.close()
-                trace.note("Process status after command:")
-                for line in status.splitlines():
-                    trace.note(line)
+            trace.debug_memory('Process status after command:', short=False)
         return ret or 0
     finally:
-        # reset, in case we may do other commands later within the same process
-        option._verbosity_level = 0
+        # reset, in case we may do other commands later within the same
+        # process. Commands that want to execute sub-commands must propagate
+        # --verbose in their own way.
+        option._verbosity_level = saved_verbosity_level
+
 
 def display_command(func):
     """Decorator that suppresses pipe/interrupt errors."""
@@ -833,15 +941,16 @@ def display_command(func):
 
 def main(argv):
     import bzrlib.ui
-    from bzrlib.ui.text import TextUIFactory
-    bzrlib.ui.ui_factory = TextUIFactory()
-     
+    bzrlib.ui.ui_factory = bzrlib.ui.make_ui_for_terminal(
+        sys.stdin, sys.stdout, sys.stderr)
+
     # Is this a final release version? If so, we should suppress warnings
     if bzrlib.version_info[3] == 'final':
         from bzrlib import symbol_versioning
         symbol_versioning.suppress_deprecation_warnings(override=False)
     try:
-        argv = [a.decode(bzrlib.user_encoding) for a in argv[1:]]
+        user_encoding = osutils.get_user_encoding()
+        argv = [a.decode(user_encoding) for a in argv[1:]]
     except UnicodeDecodeError:
         raise errors.BzrError(("Parameter '%r' is unsupported by the current "
                                                             "encoding." % a))
@@ -851,19 +960,12 @@ def main(argv):
 
 
 def run_bzr_catch_errors(argv):
-    # Note: The except clause logic below should be kept in sync with the
-    # profile() routine in lsprof.py.
-    try:
-        return run_bzr(argv)
-    except (KeyboardInterrupt, Exception), e:
-        # used to handle AssertionError and KeyboardInterrupt
-        # specially here, but hopefully they're handled ok by the logger now
-        exitcode = trace.report_exception(sys.exc_info(), sys.stderr)
-        if os.environ.get('BZR_PDB'):
-            print '**** entering debugger'
-            import pdb
-            pdb.post_mortem(sys.exc_traceback)
-        return exitcode
+    """Run a bzr command with parameters as described by argv.
+
+    This function assumed that that UI layer is setup, that symbol deprecations
+    are already applied, and that unicode decoding has already been performed on argv.
+    """
+    return exception_to_return_code(run_bzr, argv)
 
 
 def run_bzr_catch_user_errors(argv):
@@ -911,8 +1013,8 @@ class Provider(object):
 
     def plugin_for_command(self, cmd_name):
         '''Takes a command and returns the information for that plugin
-        
-        :return: A dictionary with all the available information 
+
+        :return: A dictionary with all the available information
         for the requested plugin
         '''
         raise NotImplementedError
