@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007 Canonical Ltd
+# Copyright (C) 2005, 2007, 2008 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #            and others
 #
@@ -37,7 +37,7 @@ setting.
 [/home/robertc/source]
 recurse=False|True(default)
 email= as above
-check_signatures= as above 
+check_signatures= as above
 create_signatures= as above.
 
 explanation of options
@@ -45,9 +45,9 @@ explanation of options
 editor - this option sets the pop up editor to use during commits.
 email - this option sets the user id bzr will use when committing.
 check_signatures - this option controls whether bzr will require good gpg
-                   signatures, ignore them, or check them if they are 
+                   signatures, ignore them, or check them if they are
                    present.
-create_signatures - this option controls whether bzr will always create 
+create_signatures - this option controls whether bzr will always create
                     gpg signatures, never create them, or create them if the
                     branch is configured to require them.
 log_format - this option sets the default log format.  Possible values are
@@ -78,6 +78,7 @@ from bzrlib import (
     errors,
     mail_client,
     osutils,
+    registry,
     symbol_versioning,
     trace,
     ui,
@@ -121,20 +122,25 @@ STORE_LOCATION_APPENDPATH = POLICY_APPENDPATH
 STORE_BRANCH = 3
 STORE_GLOBAL = 4
 
+_ConfigObj = None
+def ConfigObj(*args, **kwargs):
+    global _ConfigObj
+    if _ConfigObj is None:
+        class ConfigObj(configobj.ConfigObj):
 
-class ConfigObj(configobj.ConfigObj):
+            def get_bool(self, section, key):
+                return self[section].as_bool(key)
 
-    def get_bool(self, section, key):
-        return self[section].as_bool(key)
-
-    def get_value(self, section, name):
-        # Try [] for the old DEFAULT section.
-        if section == "DEFAULT":
-            try:
-                return self[name]
-            except KeyError:
-                pass
-        return self[section][name]
+            def get_value(self, section, name):
+                # Try [] for the old DEFAULT section.
+                if section == "DEFAULT":
+                    try:
+                        return self[name]
+                    except KeyError:
+                        pass
+                return self[section][name]
+        _ConfigObj = ConfigObj
+    return _ConfigObj(*args, **kwargs)
 
 
 class Config(object):
@@ -147,21 +153,9 @@ class Config(object):
     def get_mail_client(self):
         """Get a mail client to use"""
         selected_client = self.get_user_option('mail_client')
+        _registry = mail_client.mail_client_registry
         try:
-            mail_client_class = {
-                None: mail_client.DefaultMail,
-                # Specific clients
-                'emacsclient': mail_client.EmacsMail,
-                'evolution': mail_client.Evolution,
-                'kmail': mail_client.KMail,
-                'mutt': mail_client.Mutt,
-                'thunderbird': mail_client.Thunderbird,
-                # Generic options
-                'default': mail_client.DefaultMail,
-                'editor': mail_client.Editor,
-                'mapi': mail_client.MAPIClient,
-                'xdg-email': mail_client.XDGEmail,
-            }[selected_client]
+            mail_client_class = _registry.get(selected_client)
         except KeyError:
             raise errors.UnknownMailClient(selected_client)
         return mail_client_class(self)
@@ -222,21 +216,21 @@ class Config(object):
 
     def username(self):
         """Return email-style username.
-    
+
         Something similar to 'Martin Pool <mbp@sourcefrog.net>'
-        
+
         $BZR_EMAIL can be set to override this (as well as the
         deprecated $BZREMAIL), then
         the concrete policy type is checked, and finally
         $EMAIL is examined.
         If none is found, a reasonable default is (hopefully)
         created.
-    
+
         TODO: Check it's reasonably well-formed.
         """
         v = os.environ.get('BZR_EMAIL')
         if v:
-            return v.decode(bzrlib.user_encoding)
+            return v.decode(osutils.get_user_encoding())
 
         v = self._get_user_id()
         if v:
@@ -244,7 +238,7 @@ class Config(object):
 
         v = os.environ.get('EMAIL')
         if v:
-            return v.decode(bzrlib.user_encoding)
+            return v.decode(osutils.get_user_encoding())
 
         name, email = _auto_user_id()
         if name:
@@ -391,7 +385,7 @@ class IniBasedConfig(Config):
         super(IniBasedConfig, self).__init__()
         self._get_filename = get_filename
         self._parser = None
-        
+
     def _post_commit(self):
         """See Config.post_commit."""
         return self._get_user_option('post_commit')
@@ -420,7 +414,7 @@ class IniBasedConfig(Config):
 
     def _get_alias(self, value):
         try:
-            return self._get_parser().get_value("ALIASES", 
+            return self._get_parser().get_value("ALIASES",
                                                 value)
         except KeyError:
             pass
@@ -648,7 +642,7 @@ class BranchConfig(Config):
 
     def _get_safe_value(self, option_name):
         """This variant of get_best_value never returns untrusted values.
-        
+
         It does not return values from the branch data, because the branch may
         not be controlled by the user.
 
@@ -663,17 +657,17 @@ class BranchConfig(Config):
 
     def _get_user_id(self):
         """Return the full user id for the branch.
-    
+
         e.g. "John Hacker <jhacker@example.com>"
         This is looked up in the email controlfile for the branch.
         """
         try:
             return (self.branch._transport.get_bytes("email")
-                    .decode(bzrlib.user_encoding)
+                    .decode(osutils.get_user_encoding())
                     .rstrip("\r\n"))
         except errors.NoSuchFile, e:
             pass
-        
+
         return self._get_best_value('_get_user_id')
 
     def _get_signature_checking(self):
@@ -719,14 +713,14 @@ class BranchConfig(Config):
     def _gpg_signing_command(self):
         """See Config.gpg_signing_command."""
         return self._get_safe_value('_gpg_signing_command')
-        
+
     def __init__(self, branch):
         super(BranchConfig, self).__init__()
         self._location_config = None
         self._branch_data_config = None
         self._global_config = None
         self.branch = branch
-        self.option_sources = (self._get_location_config, 
+        self.option_sources = (self._get_location_config,
                                self._get_branch_data_config,
                                self._get_global_config)
 
@@ -774,7 +768,7 @@ def config_dir():
     """Return per-user configuration directory.
 
     By default this is ~/.bazaar/
-    
+
     TODO: Global option --config-dir to override this.
     """
     base = os.environ.get('BZR_HOME', None)
@@ -846,7 +840,11 @@ def _auto_user_id():
     try:
         import pwd
         uid = os.getuid()
-        w = pwd.getpwuid(uid)
+        try:
+            w = pwd.getpwuid(uid)
+        except KeyError:
+            raise errors.BzrCommandError('Unable to determine your name.  '
+                'Please use "bzr whoami" to set it.')
 
         # we try utf-8 first, because on many variants (like Linux),
         # /etc/passwd "should" be in utf-8, and because it's unlikely to give
@@ -857,8 +855,8 @@ def _auto_user_id():
             encoding = 'utf-8'
         except UnicodeError:
             try:
-                gecos = w.pw_gecos.decode(bzrlib.user_encoding)
-                encoding = bzrlib.user_encoding
+                encoding = osutils.get_user_encoding()
+                gecos = w.pw_gecos.decode(encoding)
             except UnicodeError:
                 raise errors.BzrCommandError('Unable to determine your name.  '
                    'Use "bzr whoami" to set it.')
@@ -879,10 +877,11 @@ def _auto_user_id():
     except ImportError:
         import getpass
         try:
-            realname = username = getpass.getuser().decode(bzrlib.user_encoding)
+            user_encoding = osutils.get_user_encoding()
+            realname = username = getpass.getuser().decode(user_encoding)
         except UnicodeDecodeError:
             raise errors.BzrError("Can't decode username as %s." % \
-                    bzrlib.user_encoding)
+                    user_encoding)
 
     return realname, (username + '@' + socket.gethostname())
 
@@ -899,7 +898,7 @@ def parse_username(username):
 def extract_email_address(e):
     """Return just the address part of an email string.
 
-    That is just the user@domain part, nothing else. 
+    That is just the user@domain part, nothing else.
     This part is required to contain only ascii characters.
     If it can't be extracted, raises an error.
 
@@ -919,7 +918,7 @@ class TreeConfig(IniBasedConfig):
 
     def __init__(self, branch):
         # XXX: Really this should be asking the branch for its configuration
-        # data, rather than relying on a Transport, so that it can work 
+        # data, rather than relying on a Transport, so that it can work
         # more cleanly with a RemoteBranch that has no transport.
         self._config = TransportConfig(branch._transport, 'branch.conf')
         self.branch = branch
@@ -1070,6 +1069,47 @@ class AuthenticationConfig(object):
 
         return credentials
 
+    def set_credentials(self, name, host, user, scheme=None, password=None,
+                        port=None, path=None, verify_certificates=None):
+        """Set authentication credentials for a host.
+
+        Any existing credentials with matching scheme, host, port and path
+        will be deleted, regardless of name.
+
+        :param name: An arbitrary name to describe this set of credentials.
+        :param host: Name of the host that accepts these credentials.
+        :param user: The username portion of these credentials.
+        :param scheme: The URL scheme (e.g. ssh, http) the credentials apply
+            to.
+        :param password: Password portion of these credentials.
+        :param port: The IP port on the host that these credentials apply to.
+        :param path: A filesystem path on the host that these credentials
+            apply to.
+        :param verify_certificates: On https, verify server certificates if
+            True.
+        """
+        values = {'host': host, 'user': user}
+        if password is not None:
+            values['password'] = password
+        if scheme is not None:
+            values['scheme'] = scheme
+        if port is not None:
+            values['port'] = '%d' % port
+        if path is not None:
+            values['path'] = path
+        if verify_certificates is not None:
+            values['verify_certificates'] = str(verify_certificates)
+        config = self._get_config()
+        for_deletion = []
+        for section, existing_values in config.items():
+            for key in ('scheme', 'host', 'port', 'path'):
+                if existing_values.get(key) != values.get(key):
+                    break
+            else:
+                del config[section]
+        config.update({name: values})
+        self._save()
+
     def get_user(self, scheme, host, port=None,
                  realm=None, path=None, prompt=None):
         """Get a user from authentication file.
@@ -1137,9 +1177,57 @@ class AuthenticationConfig(object):
         return password
 
     def decode_password(self, credentials, encoding):
+        try:
+            cs = credential_store_registry.get_credential_store(encoding)
+        except KeyError:
+            raise ValueError('%r is not a known password_encoding' % encoding)
+        credentials['password'] = cs.decode_password(credentials)
         return credentials
 
 
+class CredentialStoreRegistry(registry.Registry):
+    """A class that registers credential stores.
+
+    A credential store provides access to credentials via the password_encoding
+    field in authentication.conf sections.
+
+    Except for stores provided by bzr itself,most stores are expected to be
+    provided by plugins that will therefore use
+    register_lazy(password_encoding, module_name, member_name, help=help,
+    info=info) to install themselves.
+    """
+
+    def get_credential_store(self, encoding=None):
+        cs = self.get(encoding)
+        if callable(cs):
+            cs = cs()
+        return cs
+
+
+credential_store_registry = CredentialStoreRegistry()
+
+
+class CredentialStore(object):
+    """An abstract class to implement storage for credentials"""
+
+    def decode_password(self, credentials):
+        """Returns a password for the provided credentials in clear text."""
+        raise NotImplementedError(self.decode_password)
+
+
+class PlainTextCredentialStore(CredentialStore):
+    """Plain text credential store for the authentication.conf file."""
+
+    def decode_password(self, credentials):
+        """See CredentialStore.decode_password."""
+        return credentials['password']
+
+
+credential_store_registry.register('plain', PlainTextCredentialStore,
+                                   help=PlainTextCredentialStore.__doc__)
+credential_store_registry.default_key = 'plain'
+
+
 class BzrDirConfig(object):
 
     def __init__(self, transport):
