@@ -21,7 +21,12 @@ to true URLs.  Examples include lp:urls and per-user location aliases.
 """
 
 from bzrlib import errors, registry
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from bzrlib.branch import Branch
+from bzrlib import urlutils
+""")
+
 
 class DirectoryServiceRegistry(registry.Registry):
     """This object maintains and uses a list of directory services.
@@ -72,14 +77,22 @@ class AliasDirectory(object):
             'push': branch.get_push_location,
             'this': lambda: branch.base
         }
+        parts = url.split('/', 1)
+        if len(parts) == 2:
+            name, extra = parts
+        else:
+            (name,) = parts
+            extra = None
         try:
-            method = lookups[url[1:]]
+            method = lookups[name[1:]]
         except KeyError:
             raise errors.InvalidLocationAlias(url)
         else:
             result = method()
         if result is None:
             raise errors.UnsetLocationAlias(url)
+        if extra is not None:
+            result = urlutils.join(result, extra)
         return result
 
 directories.register(':', AliasDirectory,
