@@ -32,7 +32,7 @@ def register_exporter(format, extensions, func, override=False):
     """Register an exporter.
 
     :param format: This is the name of the format, such as 'tgz' or 'zip'
-    :param extensions: Extensions which should be used in the case that a 
+    :param extensions: Extensions which should be used in the case that a
                        format was not explicitly specified.
     :type extensions: List
     :param func: The function. It will be called with (tree, dest, root)
@@ -55,14 +55,14 @@ def register_lazy_exporter(scheme, extensions, module, funcname):
 
     When requesting a specific type of export, load the respective path.
     """
-    def _loader(tree, dest, root):
+    def _loader(tree, dest, root, subdir):
         mod = __import__(module, globals(), locals(), [funcname])
         func = getattr(mod, funcname)
-        return func(tree, dest, root)
+        return func(tree, dest, root, subdir)
     register_exporter(scheme, extensions, _loader)
 
 
-def export(tree, dest, format=None, root=None):
+def export(tree, dest, format=None, root=None, subdir=None):
     """Export the given Tree to the specific destination.
 
     :param tree: A Tree (such as RevisionTree) to export
@@ -70,12 +70,15 @@ def export(tree, dest, format=None, root=None):
     :param format: The format (dir, zip, etc), if None, it will check the
                    extension on dest, looking for a match
     :param root: The root location inside the format.
-                 It is common practise to have zipfiles and tarballs 
+                 It is common practise to have zipfiles and tarballs
                  extract into a subdirectory, rather than into the
                  current working directory.
                  If root is None, the default root will be
                  selected as the destination without its
                  extension.
+    :param subdir: A starting directory within the tree. None means to export
+        the entire tree, and anything else should specify the relative path to
+        a directory to start exporting from.
     """
     global _exporters, _exporter_extensions
 
@@ -92,7 +95,11 @@ def export(tree, dest, format=None, root=None):
 
     if format not in _exporters:
         raise errors.NoSuchExportFormat(format)
-    return _exporters[format](tree, dest, root)
+    tree.lock_read()
+    try:
+        return _exporters[format](tree, dest, root, subdir)
+    finally:
+        tree.unlock()
 
 
 def get_root_name(dest):
@@ -125,6 +132,34 @@ def get_root_name(dest):
     return dest
 
 
+def _export_iter_entries(tree, subdir):
+    """Iter the entries for tree suitable for exporting.
+
+    :param tree: A tree object.
+    :param subdir: None or the path of a directory to start exporting from.
+    """
+    inv = tree.inventory
+    if subdir is None:
+        subdir_id = None
+    else:
+        subdir_id = inv.path2id(subdir)
+    entries = inv.iter_entries(subdir_id)
+    if subdir is None:
+        entries.next() # skip root
+    for entry in entries:
+        # The .bzr* namespace is reserved for "magic" files like
+        # .bzrignore and .bzrrules - do not export these
+        if entry[0].startswith(".bzr"):
+            continue
+        if subdir is None:
+            if not tree.has_filename(entry[0]):
+                continue
+        else:
+            if not tree.has_filename(os.path.join(subdir, entry[0])):
+                continue
+        yield entry
+
+
 register_lazy_exporter(None, [], 'bzrlib.export.dir_exporter', 'dir_exporter')
 register_lazy_exporter('dir', [], 'bzrlib.export.dir_exporter', 'dir_exporter')
 register_lazy_exporter('tar', ['.tar'], 'bzrlib.export.tar_exporter', 'tar_exporter')
