@@ -43,6 +43,23 @@ from bzrlib.symbol_versioning import (
 # somewhat redundant with what's done in LockDir; the main difference is that
 # LockableFiles permits reentrancy.
 
+class _LockWarner(object):
+    """Hold a counter for a lock and warn if GCed while the count is >= 1.
+
+    This is separate from LockableFiles because putting a __del__ on
+    LockableFiles can result in uncollectable cycles.
+    """
+
+    def __init__(self, repr):
+        self.lock_count = 0
+        self.repr = repr
+
+    def __del__(self):
+        if self.lock_count >= 1:
+            # There should have been a try/finally to unlock this.
+            warnings.warn("%r was gc'd while locked" % self.repr)
+
+
 class LockableFiles(object):
     """Object representing a set of related files locked within the same scope.
 
@@ -57,28 +74,28 @@ class LockableFiles(object):
     support this.)
 
     Instances of this class are often called control_files.
-    
+
     This object builds on top of a Transport, which is used to actually write
     the files to disk, and an OSLock or LockDir, which controls how access to
     the files is controlled.  The particular type of locking used is set when
     the object is constructed.  In older formats OSLocks are used everywhere.
-    in newer formats a LockDir is used for Repositories and Branches, and 
+    in newer formats a LockDir is used for Repositories and Branches, and
     OSLocks for the local filesystem.
 
-    This class is now deprecated; code should move to using the Transport 
-    directly for file operations and using the lock or CountedLock for 
+    This class is now deprecated; code should move to using the Transport
+    directly for file operations and using the lock or CountedLock for
     locking.
     """
 
     # _lock_mode: None, or 'r' or 'w'
 
     # _lock_count: If _lock_mode is true, a positive count of the number of
-    # times the lock has been taken *by this process*.   
-    
+    # times the lock has been taken *by this process*.
+
     def __init__(self, transport, lock_name, lock_class):
         """Create a LockableFiles group
 
-        :param transport: Transport pointing to the directory holding the 
+        :param transport: Transport pointing to the directory holding the
             control files and lock.
         :param lock_name: Name of the lock guarding these files.
         :param lock_class: Class of lock strategy to use: typically
@@ -88,7 +105,7 @@ class LockableFiles(object):
         self.lock_name = lock_name
         self._transaction = None
         self._lock_mode = None
-        self._lock_count = 0
+        self._lock_warner = _LockWarner(repr(self))
         self._find_modes()
         esc_name = self._escape(lock_name)
         self._lock = lock_class(transport, esc_name,
@@ -109,12 +126,6 @@ class LockableFiles(object):
     def __str__(self):
         return 'LockableFiles(%s, %s)' % (self.lock_name, self._transport.base)
 
-    def __del__(self):
-        if self.is_locked():
-            # do not automatically unlock; there should have been a
-            # try/finally to unlock this.
-            warnings.warn("%r was gc'd while locked" % self)
-
     def break_lock(self):
         """Break the lock of this lockable files group if it is held.
 
@@ -123,6 +134,7 @@ class LockableFiles(object):
         self._lock.break_lock()
 
     def _escape(self, file_or_path):
+        """DEPRECATED: Do not use outside this class"""
         if not isinstance(file_or_path, basestring):
             file_or_path = '/'.join(file_or_path)
         if file_or_path == '':
@@ -131,7 +143,7 @@ class LockableFiles(object):
 
     def _find_modes(self):
         """Determine the appropriate modes for files and directories.
-        
+
         :deprecated: Replaced by BzrDir._find_modes.
         """
         try:
@@ -151,7 +163,7 @@ class LockableFiles(object):
     @deprecated_method(deprecated_in((1, 6, 0)))
     def controlfilename(self, file_or_path):
         """Return location relative to branch.
-        
+
         :deprecated: Use Transport methods instead.
         """
         return self._transport.abspath(self._escape(file_or_path))
@@ -160,7 +172,7 @@ class LockableFiles(object):
     @deprecated_method(deprecated_in((1, 5, 0)))
     def get(self, relpath):
         """Get a file as a bytestream.
-        
+
         :deprecated: Use a Transport instead of LockableFiles.
         """
         relpath = self._escape(relpath)
@@ -170,7 +182,7 @@ class LockableFiles(object):
     @deprecated_method(deprecated_in((1, 5, 0)))
     def get_utf8(self, relpath):
         """Get a file as a unicode stream.
-        
+
         :deprecated: Use a Transport instead of LockableFiles.
         """
         relpath = self._escape(relpath)
@@ -181,7 +193,7 @@ class LockableFiles(object):
     @deprecated_method(deprecated_in((1, 6, 0)))
     def put(self, path, file):
         """Write a file.
-        
+
         :param path: The path to put the file, relative to the .bzr control
                      directory
         :param file: A file-like or string object whose contents should be copied.
@@ -233,7 +245,7 @@ class LockableFiles(object):
 
     def lock_write(self, token=None):
         """Lock this group of files for writing.
-        
+
         :param token: if this is already locked, then lock_write will fail
             unless the token matches the existing lock.
         :returns: a token if this instance supports tokens, otherwise None.
@@ -252,13 +264,13 @@ class LockableFiles(object):
             if self._lock_mode != 'w' or not self.get_transaction().writeable():
                 raise errors.ReadOnlyError(self)
             self._lock.validate_token(token)
-            self._lock_count += 1
+            self._lock_warner.lock_count += 1
             return self._token_from_lock
         else:
             token_from_lock = self._lock.lock_write(token=token)
             #traceback.print_stack()
             self._lock_mode = 'w'
-            self._lock_count = 1
+            self._lock_warner.lock_count = 1
             self._set_transaction(transactions.WriteTransaction())
             self._token_from_lock = token_from_lock
             return token_from_lock
@@ -267,36 +279,40 @@ class LockableFiles(object):
         if self._lock_mode:
             if self._lock_mode not in ('r', 'w'):
                 raise ValueError("invalid lock mode %r" % (self._lock_mode,))
-            self._lock_count += 1
+            self._lock_warner.lock_count += 1
         else:
             self._lock.lock_read()
             #traceback.print_stack()
             self._lock_mode = 'r'
-            self._lock_count = 1
+            self._lock_warner.lock_count = 1
             self._set_transaction(transactions.ReadOnlyTransaction())
             # 5K may be excessive, but hey, its a knob.
             self.get_transaction().set_cache_size(5000)
-                        
+
     def unlock(self):
         if not self._lock_mode:
             raise errors.LockNotHeld(self)
-        if self._lock_count > 1:
-            self._lock_count -= 1
+        if self._lock_warner.lock_count > 1:
+            self._lock_warner.lock_count -= 1
         else:
             #traceback.print_stack()
             self._finish_transaction()
             try:
                 self._lock.unlock()
             finally:
-                self._lock_mode = self._lock_count = None
+                self._lock_mode = self._lock_warner.lock_count = None
+
+    @property
+    def _lock_count(self):
+        return self._lock_warner.lock_count
 
     def is_locked(self):
         """Return true if this LockableFiles group is locked"""
-        return self._lock_count >= 1
+        return self._lock_warner.lock_count >= 1
 
     def get_physical_lock_status(self):
         """Return physical lock status.
-        
+
         Returns true if a lock is held on the transport. If no lock is held, or
         the underlying locking mechanism does not support querying lock
         status, false is returned.
@@ -384,4 +400,4 @@ class TransportLock(object):
     def validate_token(self, token):
         if token is not None:
             raise errors.TokenLockingNotSupported(self)
-        
+
