@@ -19,6 +19,7 @@ import os
 import subprocess
 import sys
 import tempfile
+import urllib
 
 import bzrlib
 from bzrlib import (
@@ -27,8 +28,11 @@ from bzrlib import (
     msgeditor,
     osutils,
     urlutils,
+    registry
     )
 
+mail_client_registry = registry.Registry()
+
 
 class MailClient(object):
     """A mail client that can send messages with attachements."""
@@ -109,6 +113,8 @@ class Editor(MailClient):
                                         body,
                                         attachment,
                                         attachment_mime_subtype=mime_subtype)
+mail_client_registry.register('editor', Editor,
+                              help=Editor.__doc__)
 
 
 class ExternalMailClient(MailClient):
@@ -130,7 +136,7 @@ class ExternalMailClient(MailClient):
         """
         if basename is None:
             basename = 'attachment'
-        pathname = tempfile.mkdtemp(prefix='bzr-mail-')
+        pathname = osutils.mkdtemp(prefix='bzr-mail-')
         attach_path = osutils.pathjoin(pathname, basename + extension)
         outfile = open(attach_path, 'wb')
         try:
@@ -186,7 +192,7 @@ class ExternalMailClient(MailClient):
         :return:    encoded string if u is unicode, u itself otherwise.
         """
         if isinstance(u, unicode):
-            return u.encode(bzrlib.user_encoding, 'replace')
+            return u.encode(osutils.get_user_encoding(), 'replace')
         return u
 
     def _encode_path(self, path, kind):
@@ -200,7 +206,7 @@ class ExternalMailClient(MailClient):
         """
         if isinstance(path, unicode):
             try:
-                return path.encode(bzrlib.user_encoding)
+                return path.encode(osutils.get_user_encoding())
             except UnicodeEncodeError:
                 raise errors.UnableEncodePath(path, kind)
         return path
@@ -222,6 +228,8 @@ class Evolution(ExternalMailClient):
                         sorted(message_options.iteritems())]
         return ['mailto:%s?%s' % (self._encode_safe(to or ''),
             '&'.join(options_list))]
+mail_client_registry.register('evolution', Evolution,
+                              help=Evolution.__doc__)
 
 
 class Mutt(ExternalMailClient):
@@ -240,6 +248,8 @@ class Mutt(ExternalMailClient):
         if to is not None:
             message_options.append(self._encode_safe(to))
         return message_options
+mail_client_registry.register('mutt', Mutt,
+                              help=Mutt.__doc__)
 
 
 class Thunderbird(ExternalMailClient):
@@ -253,7 +263,8 @@ class Thunderbird(ExternalMailClient):
     """
 
     _client_commands = ['thunderbird', 'mozilla-thunderbird', 'icedove',
-        '/Applications/Mozilla/Thunderbird.app/Contents/MacOS/thunderbird-bin']
+        '/Applications/Mozilla/Thunderbird.app/Contents/MacOS/thunderbird-bin',
+        '/Applications/Thunderbird.app/Contents/MacOS/thunderbird-bin']
 
     def _get_compose_commandline(self, to, subject, attach_path):
         """See ExternalMailClient._get_compose_commandline"""
@@ -268,6 +279,8 @@ class Thunderbird(ExternalMailClient):
         options_list = ["%s='%s'" % (k, v) for k, v in
                         sorted(message_options.iteritems())]
         return ['-compose', ','.join(options_list)]
+mail_client_registry.register('thunderbird', Thunderbird,
+                              help=Thunderbird.__doc__)
 
 
 class KMail(ExternalMailClient):
@@ -286,6 +299,34 @@ class KMail(ExternalMailClient):
         if to is not None:
             message_options.extend([self._encode_safe(to)])
         return message_options
+mail_client_registry.register('kmail', KMail,
+                              help=KMail.__doc__)
+
+
+class Claws(ExternalMailClient):
+    """Claws mail client."""
+
+    _client_commands = ['claws-mail']
+
+    def _get_compose_commandline(self, to, subject, attach_path):
+        """See ExternalMailClient._get_compose_commandline"""
+        compose_url = ['mailto:']
+        if to is not None:
+            compose_url.append(self._encode_safe(to))
+        compose_url.append('?')
+        if subject is not None:
+            # Don't use urllib.quote_plus because Claws doesn't seem
+            # to recognise spaces encoded as "+".
+            compose_url.append(
+                'subject=%s' % urllib.quote(self._encode_safe(subject)))
+        # Collect command-line options.
+        message_options = ['--compose', ''.join(compose_url)]
+        if attach_path is not None:
+            message_options.extend(
+                ['--attach', self._encode_path(attach_path, 'attachment')])
+        return message_options
+mail_client_registry.register('claws', Claws,
+                              help=Claws.__doc__)
 
 
 class XDGEmail(ExternalMailClient):
@@ -304,6 +345,8 @@ class XDGEmail(ExternalMailClient):
             commandline.extend(['--attach',
                 self._encode_path(attach_path, 'attachment')])
         return commandline
+mail_client_registry.register('xdg-email', XDGEmail,
+                              help=XDGEmail.__doc__)
 
 
 class EmacsMail(ExternalMailClient):
@@ -348,7 +391,9 @@ class EmacsMail(ExternalMailClient):
             (if (functionp 'etach-attach)
               (etach-attach file)
               (mail-attach-file file))))
-         ((or (eq agent 'message-user-agent)(eq agent 'gnus-user-agent))
+         ((or (eq agent 'message-user-agent)
+              (eq agent 'gnus-user-agent)
+              (eq agent 'mh-e-user-agent))
           (progn
             (mml-attach-file file "text/x-patch" "BZR merge" "inline")))
          ((eq agent 'mew-user-agent)
@@ -405,6 +450,8 @@ class EmacsMail(ExternalMailClient):
             commandline.append(rmform)
 
         return commandline
+mail_client_registry.register('emacsclient', EmacsMail,
+                              help=EmacsMail.__doc__)
 
 
 class MAPIClient(ExternalMailClient):
@@ -423,6 +470,8 @@ class MAPIClient(ExternalMailClient):
             if e.code != simplemapi.MAPI_USER_ABORT:
                 raise errors.MailClientNotFound(['MAPI supported mail client'
                                                  ' (error %d)' % (e.code,)])
+mail_client_registry.register('mapi', MAPIClient,
+                              help=MAPIClient.__doc__)
 
 
 class DefaultMail(MailClient):
@@ -455,3 +504,6 @@ class DefaultMail(MailClient):
         except errors.MailClientNotFound:
             return Editor(self.config).compose_merge_request(to, subject,
                           directive, basename=basename)
+mail_client_registry.register('default', DefaultMail,
+                              help=DefaultMail.__doc__)
+mail_client_registry.default_key = 'default'
