@@ -28,6 +28,7 @@ from bzrlib import (
     add,
     bzrdir,
     hooks,
+    symbol_versioning,
     )
 from bzrlib.osutils import dirname
 from bzrlib.revisiontree import RevisionTree
@@ -41,7 +42,6 @@ from bzrlib import (
     )
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.osutils import splitpath
-from bzrlib.symbol_versioning import DEPRECATED_PARAMETER
 
 
 def needs_tree_write_lock(unbound):
@@ -69,12 +69,18 @@ class MutableTree(tree.Tree):
     entirely in memory.
 
     For now, we are not treating MutableTree as an interface to provide
-    conformance tests for - rather we are testing MemoryTree specifically, and 
+    conformance tests for - rather we are testing MemoryTree specifically, and
     interface testing implementations of WorkingTree.
 
     A mutable tree always has an associated Branch and BzrDir object - the
     branch and bzrdir attributes.
     """
+    def __init__(self, *args, **kw):
+        super(MutableTree, self).__init__(*args, **kw)
+        # Is this tree on a case-insensitive or case-preserving file-system?
+        # Sub-classes may initialize to False if they detect they are being
+        # used on media which doesn't differentiate the case of names.
+        self.case_sensitive = True
 
     @needs_tree_write_lock
     def add(self, files, ids=None, kinds=None):
@@ -124,7 +130,7 @@ class MutableTree(tree.Tree):
             if self.is_control_filename(f):
                 raise errors.ForbiddenControlFileError(filename=f)
             fp = splitpath(f)
-        # fill out file kinds for all files [not needed when we stop 
+        # fill out file kinds for all files [not needed when we stop
         # caring about the instantaneous file kind within a uncommmitted tree
         #
         self._gather_kinds(files, kinds)
@@ -181,26 +187,65 @@ class MutableTree(tree.Tree):
         from bzrlib import commit
         if revprops is None:
             revprops = {}
+        possible_master_transports=[]
         if not 'branch-nick' in revprops:
-            revprops['branch-nick'] = self.branch.nick
+            revprops['branch-nick'] = self.branch._get_nick(
+                kwargs.get('local', False),
+                possible_master_transports)
+        authors = kwargs.pop('authors', None)
         author = kwargs.pop('author', None)
-        if author is not None:
-            if 'author' in revprops:
+        if authors is not None:
+            if author is not None:
+                raise AssertionError('Specifying both author and authors '
+                        'is not allowed. Specify just authors instead')
+            if 'author' in revprops or 'authors' in revprops:
                 # XXX: maybe we should just accept one of them?
                 raise AssertionError('author property given twice')
-            revprops['author'] = author
+            if authors:
+                for individual in authors:
+                    if '\n' in individual:
+                        raise AssertionError('\\n is not a valid character '
+                                'in an author identity')
+                revprops['authors'] = '\n'.join(authors)
+        if author is not None:
+            symbol_versioning.warn('The parameter author was deprecated'
+                   ' in version 1.13. Use authors instead',
+                   DeprecationWarning)
+            if 'author' in revprops or 'authors' in revprops:
+                # XXX: maybe we should just accept one of them?
+                raise AssertionError('author property given twice')
+            if '\n' in author:
+                raise AssertionError('\\n is not a valid character '
+                        'in an author identity')
+            revprops['authors'] = author
         # args for wt.commit start at message from the Commit.commit method,
         args = (message, ) + args
         for hook in MutableTree.hooks['start_commit']:
             hook(self)
         committed_id = commit.Commit().commit(working_tree=self,
-            revprops=revprops, *args, **kwargs)
+            revprops=revprops,
+            possible_master_transports=possible_master_transports,
+            *args, **kwargs)
         return committed_id
 
     def _gather_kinds(self, files, kinds):
         """Helper function for add - sets the entries of kinds."""
         raise NotImplementedError(self._gather_kinds)
 
+    def get_file_with_stat(self, file_id, path=None):
+        """Get a file handle and stat object for file_id.
+
+        The default implementation returns (self.get_file, None) for backwards
+        compatibility.
+
+        :param file_id: The file id to read.
+        :param path: The path of the file, if it is known.
+        :return: A tuple (file_handle, stat_value_or_None). If the tree has
+            no stat facility, or need for a stat cache feedback during commit,
+            it may return None for the second element of the tuple.
+        """
+        return (self.get_file(file_id, path), None)
+
     @needs_read_lock
     def last_revision(self):
         """Return the revision id of the last commit performed in this tree.
@@ -208,7 +253,7 @@ class MutableTree(tree.Tree):
         In early tree formats the result of last_revision is the same as the
         branch last_revision, but that is no longer the case for modern tree
         formats.
-        
+
         last_revision returns the left most parent id, or None if there are no
         parents.
 
@@ -231,7 +276,7 @@ class MutableTree(tree.Tree):
     def lock_write(self):
         """Lock the tree and its branch. This allows mutating calls to be made.
 
-        Some mutating methods will take out implicit write locks, but in 
+        Some mutating methods will take out implicit write locks, but in
         general you should always obtain a write lock before calling mutating
         methods on a tree.
         """
@@ -247,10 +292,33 @@ class MutableTree(tree.Tree):
         """
         raise NotImplementedError(self.mkdir)
 
+    def _observed_sha1(self, file_id, path, (sha1, stat_value)):
+        """Tell the tree we have observed a paths sha1.
+
+        The intent of this function is to allow trees that have a hashcache to
+        update the hashcache during commit. If the observed file is too new
+        (based on the stat_value) to be safely hash-cached the tree will ignore
+        it.
+
+        The default implementation does nothing.
+
+        :param file_id: The file id
+        :param path: The file path
+        :param sha1: The sha 1 that was observed.
+        :param stat_value: A stat result for the file the sha1 was read from.
+        :return: None
+        """
+
+    def _fix_case_of_inventory_path(self, path):
+        """If our tree isn't case sensitive, return the canonical path"""
+        if not self.case_sensitive:
+            path = self.get_canonical_inventory_path(path)
+        return path
+
     @needs_write_lock
     def put_file_bytes_non_atomic(self, file_id, bytes):
         """Update the content of a file in the tree.
-        
+
         Note that the file is written in-place rather than being
         written to a temporary location and renamed. As a consequence,
         readers can potentially see the file half-written.
@@ -270,7 +338,7 @@ class MutableTree(tree.Tree):
     def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
         """Set the parents of the working tree.
 
-        :param parents_list: A list of (revision_id, tree) tuples. 
+        :param parents_list: A list of (revision_id, tree) tuples.
             If tree is None, then that element is treated as an unreachable
             parent tree - i.e. a ghost.
         """
@@ -284,7 +352,7 @@ class MutableTree(tree.Tree):
         For the specific behaviour see the help for cmd_add().
 
         :param action: A reporter to be called with the inventory, parent_ie,
-            path and kind of the path being added. It may return a file_id if 
+            path and kind of the path being added. It may return a file_id if
             a specific one should be used.
         :param save: Save the inventory after completing the adds. If False
             this provides dry-run functionality by doing the add and not saving
@@ -298,7 +366,7 @@ class MutableTree(tree.Tree):
         from bzrlib.inventory import InventoryEntry
         if action is None:
             action = add.AddAction()
-        
+
         if not file_list:
             # no paths supplied: add the entire tree.
             file_list = [u'.']
@@ -309,16 +377,17 @@ class MutableTree(tree.Tree):
         dirs_to_add = []
         user_dirs = set()
 
-        # validate user file paths and convert all paths to tree 
+        # validate user file paths and convert all paths to tree
         # relative : it's cheaper to make a tree relative path an abspath
-        # than to convert an abspath to tree relative.
-        for filepath in file_list:
-            rf = _FastPath(self.relpath(filepath))
+        # than to convert an abspath to tree relative, and it's cheaper to
+        # perform the canonicalization in bulk.
+        for filepath in osutils.canonical_relpaths(self.basedir, file_list):
+            rf = _FastPath(filepath)
             # validate user parameters. Our recursive code avoids adding new files
-            # that need such validation 
+            # that need such validation
             if self.is_control_filename(rf.raw_path):
                 raise errors.ForbiddenControlFileError(filename=rf.raw_path)
-            
+
             abspath = self.abspath(rf.raw_path)
             kind = osutils.file_kind(abspath)
             if kind == 'directory':
@@ -329,7 +398,7 @@ class MutableTree(tree.Tree):
                     raise errors.BadFileKindError(filename=abspath, kind=kind)
             # ensure the named path is added, so that ignore rules in the later directory
             # walk dont skip it.
-            # we dont have a parent ie known yet.: use the relatively slower inventory 
+            # we dont have a parent ie known yet.: use the relatively slower inventory
             # probing method
             versioned = inv.has_filename(rf.raw_path)
             if versioned:
@@ -372,9 +441,10 @@ class MutableTree(tree.Tree):
             if parent_ie is not None:
                 versioned = directory.base_path in parent_ie.children
             else:
-                # without the parent ie, use the relatively slower inventory 
+                # without the parent ie, use the relatively slower inventory
                 # probing method
-                versioned = inv.has_filename(directory.raw_path)
+                versioned = inv.has_filename(
+                        self._fix_case_of_inventory_path(directory.raw_path))
 
             if kind == 'directory':
                 try:
@@ -395,7 +465,7 @@ class MutableTree(tree.Tree):
                 # mutter("%r is already versioned", abspath)
             elif sub_tree:
                 # XXX: This is wrong; people *might* reasonably be trying to add
-                # subtrees as subtrees.  This should probably only be done in formats 
+                # subtrees as subtrees.  This should probably only be done in formats
                 # which can represent subtrees, and even then perhaps only when
                 # the user asked to add subtrees.  At the moment you can add them
                 # specially through 'join --reference', which is perhaps
@@ -411,21 +481,22 @@ class MutableTree(tree.Tree):
                     # must be present:
                     this_ie = parent_ie.children[directory.base_path]
                 else:
-                    # without the parent ie, use the relatively slower inventory 
+                    # without the parent ie, use the relatively slower inventory
                     # probing method
-                    this_id = inv.path2id(directory.raw_path)
+                    this_id = inv.path2id(
+                            self._fix_case_of_inventory_path(directory.raw_path))
                     if this_id is None:
                         this_ie = None
                     else:
                         this_ie = inv[this_id]
 
                 for subf in sorted(os.listdir(abspath)):
-                    # here we could use TreeDirectory rather than 
+                    # here we could use TreeDirectory rather than
                     # string concatenation.
                     subp = osutils.pathjoin(directory.raw_path, subf)
-                    # TODO: is_control_filename is very slow. Make it faster. 
-                    # TreeDirectory.is_control_filename could also make this 
-                    # faster - its impossible for a non root dir to have a 
+                    # TODO: is_control_filename is very slow. Make it faster.
+                    # TreeDirectory.is_control_filename could also make this
+                    # faster - its impossible for a non root dir to have a
                     # control file.
                     if self.is_control_filename(subp):
                         mutter("skip control directory %r", subp)
@@ -489,7 +560,7 @@ class MutableTree(tree.Tree):
 
 
 class MutableTreeHooks(hooks.Hooks):
-    """A dictionary mapping a hook name to a list of callables for mutabletree 
+    """A dictionary mapping a hook name to a list of callables for mutabletree
     hooks.
     """
 
@@ -545,7 +616,7 @@ def _add_one_and_parent(tree, inv, parent_ie, path, kind, action):
         added = []
     else:
         # slower but does not need parent_ie
-        if inv.has_filename(path.raw_path):
+        if inv.has_filename(tree._fix_case_of_inventory_path(path.raw_path)):
             return []
         # its really not there : add the parent
         # note that the dirname use leads to some extra str copying etc but as
