@@ -1,5 +1,4 @@
-# Copyright (C) 2005 Aaron Bentley <aaron.bentley@utoronto.ca>
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,36 +18,33 @@
 """Progress indicators.
 
 The usual way to use this is via bzrlib.ui.ui_factory.nested_progress_bar which
-will maintain a ProgressBarStack for you.
-
-For direct use, the factory ProgressBar will return an auto-detected progress
-bar that should match your terminal type. You can manually create a
-ProgressBarStack too if you need multiple levels of cooperating progress bars.
-Note that bzrlib's internal functions use the ui module, so if you are using
-bzrlib it really is best to use bzrlib.ui.ui_factory.
+will manage a conceptual stack of nested activities.
 """
 
-# TODO: Optionally show elapsed time instead/as well as ETA; nicer
-# when the rate is unpredictable
 
 import sys
 import time
 import os
+import warnings
+
 
-from bzrlib.lazy_import import lazy_import
-lazy_import(globals(), """
 from bzrlib import (
     errors,
+    osutils,
+    trace,
+    ui,
     )
-""")
-
 from bzrlib.trace import mutter
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    deprecated_method,
+    )
 
 
 def _supports_progress(f):
     """Detect if we can use pretty progress bars on the output stream f.
 
-    If this returns true we expect that a human may be looking at that 
+    If this returns true we expect that a human may be looking at that
     output, and that we can repaint a line to update it.
     """
     isatty = getattr(f, 'isatty', None)
@@ -62,7 +58,79 @@ def _supports_progress(f):
     return True
 
 
-_progress_bar_types = {}
+class ProgressTask(object):
+    """Model component of a progress indicator.
+
+    Most code that needs to indicate progress should update one of these,
+    and it will in turn update the display, if one is present.
+
+    Code updating the task may also set fields as hints about how to display
+    it: show_pct, show_spinner, show_eta, show_count, show_bar.  UIs
+    will not necessarily respect all these fields.
+    """
+
+    def __init__(self, parent_task=None, ui_factory=None):
+        self._parent_task = parent_task
+        self._last_update = 0
+        self.total_cnt = None
+        self.current_cnt = None
+        self.msg = ''
+        self.ui_factory = ui_factory
+        self.show_pct = False
+        self.show_spinner = True
+        self.show_eta = False,
+        self.show_count = True
+        self.show_bar = True
+
+    def __repr__(self):
+        return '%s(%r/%r, msg=%r)' % (
+            self.__class__.__name__,
+            self.current_cnt,
+            self.total_cnt,
+            self.msg)
+
+    def update(self, msg, current_cnt=None, total_cnt=None):
+        self.msg = msg
+        self.current_cnt = current_cnt
+        if total_cnt:
+            self.total_cnt = total_cnt
+        self.ui_factory._progress_updated(self)
+
+    def tick(self):
+        self.update(self.msg)
+
+    def finished(self):
+        self.ui_factory._progress_finished(self)
+
+    def make_sub_task(self):
+        return ProgressTask(self, self.ui_factory)
+
+    def _overall_completion_fraction(self, child_fraction=0.0):
+        """Return fractional completion of this task and its parents
+
+        Returns None if no completion can be computed."""
+        if self.current_cnt is not None and self.total_cnt:
+            own_fraction = (float(self.current_cnt) + child_fraction) / self.total_cnt
+        else:
+            own_fraction = None
+        if self._parent_task is None:
+            return own_fraction
+        else:
+            if own_fraction is None:
+                own_fraction = 0.0
+            return self._parent_task._overall_completion_fraction(own_fraction)
+
+    def note(self, fmt_string, *args):
+        """Record a note without disrupting the progress bar."""
+        # XXX: shouldn't be here; put it in mutter or the ui instead
+        if args:
+            self.ui_factory.note(fmt_string % args)
+        else:
+            self.ui_factory.note(fmt_string)
+
+    def clear(self):
+        # XXX: shouldn't be here; put it in mutter or the ui instead
+        self.ui_factory.clear_term()
 
 
 def ProgressBar(to_file=None, **kwargs):
@@ -87,10 +155,15 @@ def ProgressBar(to_file=None, **kwargs):
                                                 _progress_bar_types.keys())
         return _progress_bar_types[requested_bar_type](to_file=to_file, **kwargs)
 
- 
-class ProgressBarStack(object):
-    """A stack of progress bars."""
 
+class ProgressBarStack(object):
+    """A stack of progress bars.
+
+    This class is deprecated: instead, ask the ui factory for a new progress
+    task and finish it when it's done.
+    """
+
+    @deprecated_method(deprecated_in((1, 12, 0)))
     def __init__(self,
                  to_file=None,
                  show_pct=False,
@@ -147,10 +220,11 @@ class ProgressBarStack(object):
     def return_pb(self, bar):
         """Return bar after its been used."""
         if bar is not self._stack[-1]:
-            raise errors.MissingProgressBarFinish()
-        self._stack.pop()
+            warnings.warn("%r is not currently active" % (bar,))
+        else:
+            self._stack.pop()
+
 
- 
 class _BaseProgressBar(object):
 
     def __init__(self,
@@ -206,6 +280,7 @@ class DummyProgress(_BaseProgressBar):
 
     This can be used as the default argument for methods that
     take an optional progress indicator."""
+
     def tick(self):
         pass
 
@@ -217,7 +292,7 @@ class DummyProgress(_BaseProgressBar):
 
     def clear(self):
         pass
-        
+
     def note(self, fmt_string, *args, **kwargs):
         """See _BaseProgressBar.note()."""
 
@@ -225,20 +300,16 @@ class DummyProgress(_BaseProgressBar):
         return DummyProgress(**kwargs)
 
 
-_progress_bar_types['dummy'] = DummyProgress
-_progress_bar_types['none'] = DummyProgress
-
-
 class DotsProgressBar(_BaseProgressBar):
 
     def __init__(self, **kwargs):
         _BaseProgressBar.__init__(self, **kwargs)
         self.last_msg = None
         self.need_nl = False
-        
+
     def tick(self):
         self.update()
-        
+
     def update(self, msg=None, current_cnt=None, total_cnt=None):
         if msg and msg != self.last_msg:
             if self.need_nl:
@@ -247,19 +318,18 @@ class DotsProgressBar(_BaseProgressBar):
             self.last_msg = msg
         self.need_nl = True
         self.to_file.write('.')
-        
+
     def clear(self):
         if self.need_nl:
             self.to_file.write('\n')
         self.need_nl = False
-        
+
     def child_update(self, message, current, total):
         self.tick()
 
 
-_progress_bar_types['dots'] = DotsProgressBar
 
-    
+
 class TTYProgressBar(_BaseProgressBar):
     """Progress bar display object.
 
@@ -292,7 +362,7 @@ class TTYProgressBar(_BaseProgressBar):
         self._max_last_updates = 10
         self.child_fraction = 0
         self._have_output = False
-    
+
     def throttle(self, old_msg):
         """Return True if the bar was updated too recently"""
         # time.time consistently takes 40/4000 ms = 0.01 ms.
@@ -312,7 +382,7 @@ class TTYProgressBar(_BaseProgressBar):
         self.last_updates = self.last_updates[-self._max_last_updates:]
         self.last_update = now
         return False
-        
+
     def tick(self):
         self.update(self.last_msg, self.last_cnt, self.last_total,
                     self.child_fraction)
@@ -329,8 +399,9 @@ class TTYProgressBar(_BaseProgressBar):
         self.tick()
 
     def update(self, msg, current_cnt=None, total_cnt=None,
-               child_fraction=0):
-        """Update and redraw progress bar."""
+            child_fraction=0):
+        """Update and redraw progress bar.
+        """
         if msg is None:
             msg = self.last_msg
 
@@ -339,11 +410,11 @@ class TTYProgressBar(_BaseProgressBar):
 
         if current_cnt < 0:
             current_cnt = 0
-            
+
         if current_cnt > total_cnt:
             total_cnt = current_cnt
-        
-        ## # optional corner case optimisation 
+
+        ## # optional corner case optimisation
         ## # currently does not seem to fire so costs more than saved.
         ## # trivial optimal case:
         ## # NB if callers are doing a clear and restore with
@@ -356,6 +427,9 @@ class TTYProgressBar(_BaseProgressBar):
         ##     self.child_fraction == child_fraction):
         ##     return
 
+        if msg is None:
+            msg = ''
+
         old_msg = self.last_msg
         # save these for the tick() function
         self.last_msg = msg
@@ -363,7 +437,7 @@ class TTYProgressBar(_BaseProgressBar):
         self.last_total = total_cnt
         self.child_fraction = child_fraction
 
-        # each function call takes 20ms/4000 = 0.005 ms, 
+        # each function call takes 20ms/4000 = 0.005 ms,
         # but multiple that by 4000 calls -> starts to cost.
         # so anything to make this function call faster
         # will improve base 'diff' time by up to 0.1 seconds.
@@ -371,14 +445,14 @@ class TTYProgressBar(_BaseProgressBar):
             return
 
         if self.show_eta and self.start_time and self.last_total:
-            eta = get_eta(self.start_time, self.last_cnt + self.child_fraction, 
+            eta = get_eta(self.start_time, self.last_cnt + self.child_fraction,
                     self.last_total, last_updates = self.last_updates)
             eta_str = " " + str_tdelta(eta)
         else:
             eta_str = ""
 
         if self.show_spinner:
-            spin_str = self.SPIN_CHARS[self.spin_pos % 4] + ' '            
+            spin_str = self.SPIN_CHARS[self.spin_pos % 4] + ' '
         else:
             spin_str = ''
 
@@ -401,7 +475,7 @@ class TTYProgressBar(_BaseProgressBar):
             # make both fields the same size
             t = '%i' % (self.last_total)
             c = '%*i' % (len(t), self.last_cnt)
-            count_str = ' ' + c + '/' + t 
+            count_str = ' ' + c + '/' + t
 
         if self.show_bar:
             # progress bar, if present, soaks up all remaining space
@@ -410,7 +484,7 @@ class TTYProgressBar(_BaseProgressBar):
 
             if self.last_total:
                 # number of markers highlighted in bar
-                markers = int(round(float(cols) * 
+                markers = int(round(float(cols) *
                               (self.last_cnt + self.child_fraction) / self.last_total))
                 bar_str = '[' + ('=' * markers).ljust(cols) + '] '
             elif False:
@@ -418,26 +492,26 @@ class TTYProgressBar(_BaseProgressBar):
                 # so just show an expanded spinning thingy
                 m = self.spin_pos % cols
                 ms = (' ' * m + '*').ljust(cols)
-                
+
                 bar_str = '[' + ms + '] '
             else:
                 bar_str = ''
         else:
             bar_str = ''
 
-        m = spin_str + bar_str + self.last_msg + count_str + pct_str + eta_str
+        m = spin_str + bar_str + self.last_msg + count_str \
+            + pct_str + eta_str
         self.to_file.write('\r%-*.*s' % (self.width - 1, self.width - 1, m))
         self._have_output = True
         #self.to_file.flush()
-            
+
     def clear(self):
         if self._have_output:
             self.to_file.write('\r%s\r' % (' ' * (self.width - 1)))
         self._have_output = False
-        #self.to_file.flush()        
+        #self.to_file.flush()
 
 
-_progress_bar_types['tty'] = TTYProgressBar
 
 
 class ChildProgress(_BaseProgressBar):
@@ -527,7 +601,7 @@ def get_eta(start_time, current, total, enough_samples=3, last_updates=None, n_r
 
     if elapsed < 2.0:                   # not enough time to estimate
         return None
-    
+
     total_duration = float(elapsed) * float(total) / float(current)
 
     if last_updates and len(last_updates) >= n_recent:
@@ -557,3 +631,10 @@ class ProgressPhase(object):
         else:
             self.cur_phase += 1
         self.pb.update(self.message, self.cur_phase, self.total)
+
+
+_progress_bar_types = {}
+_progress_bar_types['dummy'] = DummyProgress
+_progress_bar_types['none'] = DummyProgress
+_progress_bar_types['tty'] = TTYProgressBar
+_progress_bar_types['dots'] = DotsProgressBar
