@@ -62,6 +62,7 @@ class Reconfigure(object):
         self._create_tree = False
         self._create_repository = False
         self._destroy_repository = False
+        self._repository_trees = None
 
     @staticmethod
     def to_branch(bzrdir):
@@ -140,6 +141,21 @@ class Reconfigure(object):
             raise errors.AlreadyStandalone(bzrdir)
         return reconfiguration
 
+    @classmethod
+    def set_repository_trees(klass, bzrdir, with_trees):
+        """Adjust a repository's working tree presence default"""
+        reconfiguration = klass(bzrdir)
+        if not reconfiguration.repository.is_shared():
+            raise errors.ReconfigurationNotSupported(reconfiguration.bzrdir)
+        if with_trees and reconfiguration.repository.make_working_trees():
+            raise errors.AlreadyWithTrees(bzrdir)
+        elif (not with_trees
+              and not reconfiguration.repository.make_working_trees()):
+            raise errors.AlreadyWithNoTrees(bzrdir)
+        else:
+            reconfiguration._repository_trees = with_trees
+        return reconfiguration
+
     def _plan_changes(self, want_tree, want_branch, want_bound,
                       want_reference):
         """Determine which changes are needed to assume the configuration"""
@@ -302,3 +318,5 @@ class Reconfigure(object):
             local_branch.bind(branch.Branch.open(bind_location))
         if self._destroy_repository:
             self.bzrdir.destroy_repository()
+        if self._repository_trees is not None:
+            repo.set_make_working_trees(self._repository_trees)
