@@ -24,6 +24,7 @@ over SSH), and pass them to and from the protocol logic.  See the overview in
 bzrlib/transport/smart/__init__.py.
 """
 
+import errno
 import os
 import socket
 import sys
@@ -31,13 +32,18 @@ import urllib
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import atexit
+import weakref
 from bzrlib import (
+    debug,
     errors,
     osutils,
     symbol_versioning,
+    trace,
+    ui,
     urlutils,
     )
-from bzrlib.smart import protocol
+from bzrlib.smart import client, protocol
 from bzrlib.transport import ssh
 """)
 
@@ -82,10 +88,10 @@ def _get_protocol_factory_for_bytes(bytes):
 
 def _get_line(read_bytes_func):
     """Read bytes using read_bytes_func until a newline byte.
-    
+
     This isn't particularly efficient, so should only be used when the
     expected size of the line is quite short.
-    
+
     :returns: a tuple of two strs: (line, excess)
     """
     newline_pos = -1
@@ -107,7 +113,7 @@ class SmartMedium(object):
 
     def __init__(self):
         self._push_back_buffer = None
-        
+
     def _push_back(self, bytes):
         """Return unused bytes to the medium, because they belong to the next
         request(s).
@@ -147,7 +153,7 @@ class SmartMedium(object):
 
     def _get_line(self):
         """Read bytes from this request's response until a newline byte.
-        
+
         This isn't particularly efficient, so should only be used when the
         expected size of the line is quite short.
 
@@ -156,7 +162,19 @@ class SmartMedium(object):
         line, excess = _get_line(self.read_bytes)
         self._push_back(excess)
         return line
- 
+
+    def _report_activity(self, bytes, direction):
+        """Notify that this medium has activity.
+
+        Implementations should call this from all methods that actually do IO.
+        Be careful that it's not called twice, if one method is implemented on
+        top of another.
+
+        :param bytes: Number of bytes read or written.
+        :param direction: 'read' or 'write' or None.
+        """
+        ui.ui_factory.report_transport_activity(self, bytes, direction)
+
 
 class SmartServerStreamMedium(SmartMedium):
     """Handles smart commands coming over a stream.
@@ -167,7 +185,7 @@ class SmartServerStreamMedium(SmartMedium):
     One instance is created for each connected client; it can serve multiple
     requests in the lifetime of the connection.
 
-    The server passes requests through to an underlying backing transport, 
+    The server passes requests through to an underlying backing transport,
     which will typically be a LocalTransport looking at the server's filesystem.
 
     :ivar _push_back_buffer: a str of bytes that have been read from the stream
@@ -218,7 +236,7 @@ class SmartServerStreamMedium(SmartMedium):
 
     def _serve_one_request(self, protocol):
         """Read one request from input, process, send back a response.
-        
+
         :param protocol: a SmartServerRequestProtocol.
         """
         try:
@@ -263,13 +281,15 @@ class SmartServerSocketStreamMedium(SmartServerStreamMedium):
                 self.finished = True
                 return
             protocol.accept_bytes(bytes)
-        
+
         self._push_back(protocol.unused_data)
 
     def _read_bytes(self, desired_count):
         # We ignore the desired_count because on sockets it's more efficient to
         # read large chunks (of _MAX_READ_SIZE bytes) at a time.
-        return self.socket.recv(_MAX_READ_SIZE)
+        bytes = osutils.until_no_eintr(self.socket.recv, _MAX_READ_SIZE)
+        self._report_activity(len(bytes), 'read')
+        return bytes
 
     def terminate_due_to_error(self):
         # TODO: This should log to a server log file, but no such thing
@@ -278,7 +298,7 @@ class SmartServerSocketStreamMedium(SmartServerStreamMedium):
         self.finished = True
 
     def _write_out(self, bytes):
-        osutils.send_all(self.socket, bytes)
+        osutils.send_all(self.socket, bytes, self._report_activity)
 
 
 class SmartServerPipeStreamMedium(SmartServerStreamMedium):
@@ -345,8 +365,8 @@ class SmartClientMediumRequest(object):
     request.finished_reading()
 
     It is up to the individual SmartClientMedium whether multiple concurrent
-    requests can exist. See SmartClientMedium.get_request to obtain instances 
-    of SmartClientMediumRequest, and the concrete Medium you are using for 
+    requests can exist. See SmartClientMedium.get_request to obtain instances
+    of SmartClientMediumRequest, and the concrete Medium you are using for
     details on concurrency and pipelining.
     """
 
@@ -398,7 +418,7 @@ class SmartClientMediumRequest(object):
     def _finished_reading(self):
         """Helper for finished_reading.
 
-        finished_reading checks the state of the request to determine if 
+        finished_reading checks the state of the request to determine if
         finished_reading is allowed, and if it is hands off to _finished_reading
         to perform the action.
         """
@@ -418,7 +438,7 @@ class SmartClientMediumRequest(object):
     def _finished_writing(self):
         """Helper for finished_writing.
 
-        finished_writing checks the state of the request to determine if 
+        finished_writing checks the state of the request to determine if
         finished_writing is allowed, and if it is hands off to _finished_writing
         to perform the action.
         """
@@ -444,7 +464,7 @@ class SmartClientMediumRequest(object):
         read_bytes checks the state of the request to determing if bytes
         should be read. After that it hands off to _read_bytes to do the
         actual read.
-        
+
         By default this forwards to self._medium.read_bytes because we are
         operating on the medium's stream.
         """
@@ -455,19 +475,69 @@ class SmartClientMediumRequest(object):
         if not line.endswith('\n'):
             # end of file encountered reading from server
             raise errors.ConnectionReset(
-                "please check connectivity and permissions",
-                "(and try -Dhpss if further diagnosis is required)")
+                "please check connectivity and permissions")
         return line
 
     def _read_line(self):
         """Helper for SmartClientMediumRequest.read_line.
-        
+
         By default this forwards to self._medium._get_line because we are
         operating on the medium's stream.
         """
         return self._medium._get_line()
 
 
+class _DebugCounter(object):
+    """An object that counts the HPSS calls made to each client medium.
+
+    When a medium is garbage-collected, or failing that when atexit functions
+    are run, the total number of calls made on that medium are reported via
+    trace.note.
+    """
+
+    def __init__(self):
+        self.counts = weakref.WeakKeyDictionary()
+        client._SmartClient.hooks.install_named_hook(
+            'call', self.increment_call_count, 'hpss call counter')
+        atexit.register(self.flush_all)
+
+    def track(self, medium):
+        """Start tracking calls made to a medium.
+
+        This only keeps a weakref to the medium, so shouldn't affect the
+        medium's lifetime.
+        """
+        medium_repr = repr(medium)
+        # Add this medium to the WeakKeyDictionary
+        self.counts[medium] = [0, medium_repr]
+        # Weakref callbacks are fired in reverse order of their association
+        # with the referenced object.  So we add a weakref *after* adding to
+        # the WeakKeyDict so that we can report the value from it before the
+        # entry is removed by the WeakKeyDict's own callback.
+        ref = weakref.ref(medium, self.done)
+
+    def increment_call_count(self, params):
+        # Increment the count in the WeakKeyDictionary
+        value = self.counts[params.medium]
+        value[0] += 1
+
+    def done(self, ref):
+        value = self.counts[ref]
+        count, medium_repr = value
+        # In case this callback is invoked for the same ref twice (by the
+        # weakref callback and by the atexit function), set the call count back
+        # to 0 so this item won't be reported twice.
+        value[0] = 0
+        if count != 0:
+            trace.note('HPSS calls: %d %s', count, medium_repr)
+
+    def flush_all(self):
+        for ref in list(self.counts.keys()):
+            self.done(ref)
+
+_debug_counter = None
+
+
 class SmartClientMedium(SmartMedium):
     """Smart client is a medium for sending smart protocol requests over."""
 
@@ -482,6 +552,12 @@ class SmartClientMedium(SmartMedium):
         # _remote_version_is_before tracks the bzr version the remote side
         # can be based on what we've seen so far.
         self._remote_version_is_before = None
+        # Install debug hook function if debug flag is set.
+        if 'hpss' in debug.debug_flags:
+            global _debug_counter
+            if _debug_counter is None:
+                _debug_counter = _DebugCounter()
+            _debug_counter.track(self)
 
     def _is_remote_before(self, version_tuple):
         """Is it possible the remote side supports RPCs for a given version?
@@ -512,6 +588,10 @@ class SmartClientMedium(SmartMedium):
         """
         if (self._remote_version_is_before is not None and
             version_tuple > self._remote_version_is_before):
+            # We have been told that the remote side is older than some version
+            # which is newer than a previously supplied older-than version.
+            # This indicates that some smart verb call is not guarded
+            # appropriately (it should simply not have been tried).
             raise AssertionError(
                 "_remember_remote_is_before(%r) called, but "
                 "_remember_remote_is_before(%r) was called previously."
@@ -555,13 +635,13 @@ class SmartClientMedium(SmartMedium):
 
     def disconnect(self):
         """If this medium maintains a persistent connection, close it.
-        
+
         The default implementation does nothing.
         """
-        
+
     def remote_path_from_transport(self, transport):
         """Convert transport into a path suitable for using in a request.
-        
+
         Note that the resulting remote path doesn't encode the host name or
         anything but path, so it is only safe to use it in requests sent over
         the medium from the matching transport.
@@ -595,7 +675,7 @@ class SmartClientStreamMedium(SmartClientMedium):
 
     def _flush(self):
         """Flush the output stream.
-        
+
         This method is used by the SmartClientStreamMediumRequest to ensure that
         all data for a request is sent, to avoid long timeouts or deadlocks.
         """
@@ -612,7 +692,7 @@ class SmartClientStreamMedium(SmartClientMedium):
 
 class SmartSimplePipesClientMedium(SmartClientStreamMedium):
     """A client medium using simple pipes.
-    
+
     This client does not manage the pipes: it assumes they will always be open.
     """
 
@@ -624,6 +704,7 @@ class SmartSimplePipesClientMedium(SmartClientStreamMedium):
     def _accept_bytes(self, bytes):
         """See SmartClientStreamMedium.accept_bytes."""
         self._writeable_pipe.write(bytes)
+        self._report_activity(len(bytes), 'write')
 
     def _flush(self):
         """See SmartClientStreamMedium._flush()."""
@@ -631,16 +712,18 @@ class SmartSimplePipesClientMedium(SmartClientStreamMedium):
 
     def _read_bytes(self, count):
         """See SmartClientStreamMedium._read_bytes."""
-        return self._readable_pipe.read(count)
+        bytes = self._readable_pipe.read(count)
+        self._report_activity(len(bytes), 'read')
+        return bytes
 
 
 class SmartSSHClientMedium(SmartClientStreamMedium):
     """A client medium using SSH."""
-    
+
     def __init__(self, host, port=None, username=None, password=None,
             base=None, vendor=None, bzr_remote_path=None):
         """Creates a client that will connect on the first use.
-        
+
         :param vendor: An optional override for the ssh vendor to use. See
             bzrlib.transport.ssh for details on ssh vendors.
         """
@@ -665,6 +748,7 @@ class SmartSSHClientMedium(SmartClientStreamMedium):
         """See SmartClientStreamMedium.accept_bytes."""
         self._ensure_connection()
         self._write_to.write(bytes)
+        self._report_activity(len(bytes), 'write')
 
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""
@@ -700,17 +784,19 @@ class SmartSSHClientMedium(SmartClientStreamMedium):
         if not self._connected:
             raise errors.MediumNotConnected(self)
         bytes_to_read = min(count, _MAX_READ_SIZE)
-        return self._read_from.read(bytes_to_read)
+        bytes = self._read_from.read(bytes_to_read)
+        self._report_activity(len(bytes), 'read')
+        return bytes
 
 
 # Port 4155 is the default port for bzr://, registered with IANA.
-BZR_DEFAULT_INTERFACE = '0.0.0.0'
+BZR_DEFAULT_INTERFACE = None
 BZR_DEFAULT_PORT = 4155
 
 
 class SmartTCPClientMedium(SmartClientStreamMedium):
     """A client medium using TCP."""
-    
+
     def __init__(self, host, port, base):
         """Creates a client that will connect on the first use."""
         SmartClientStreamMedium.__init__(self, base)
@@ -722,7 +808,7 @@ class SmartTCPClientMedium(SmartClientStreamMedium):
     def _accept_bytes(self, bytes):
         """See SmartClientMedium.accept_bytes."""
         self._ensure_connection()
-        osutils.send_all(self._socket, bytes)
+        osutils.send_all(self._socket, bytes, self._report_activity)
 
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""
@@ -736,15 +822,31 @@ class SmartTCPClientMedium(SmartClientStreamMedium):
         """Connect this medium if not already connected."""
         if self._connected:
             return
-        self._socket = socket.socket()
-        self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         if self._port is None:
             port = BZR_DEFAULT_PORT
         else:
             port = int(self._port)
         try:
-            self._socket.connect((self._host, port))
-        except socket.error, err:
+            sockaddrs = socket.getaddrinfo(self._host, port, socket.AF_UNSPEC,
+                socket.SOCK_STREAM, 0, 0)
+        except socket.gaierror, (err_num, err_msg):
+            raise errors.ConnectionError("failed to lookup %s:%d: %s" %
+                    (self._host, port, err_msg))
+        # Initialize err in case there are no addresses returned:
+        err = socket.error("no address found for %s" % self._host)
+        for (family, socktype, proto, canonname, sockaddr) in sockaddrs:
+            try:
+                self._socket = socket.socket(family, socktype, proto)
+                self._socket.setsockopt(socket.IPPROTO_TCP,
+                                        socket.TCP_NODELAY, 1)
+                self._socket.connect(sockaddr)
+            except socket.error, err:
+                if self._socket is not None:
+                    self._socket.close()
+                self._socket = None
+                continue
+            break
+        if self._socket is None:
             # socket errors either have a (string) or (errno, string) as their
             # args.
             if type(err.args) is str:
@@ -757,8 +859,8 @@ class SmartTCPClientMedium(SmartClientStreamMedium):
 
     def _flush(self):
         """See SmartClientStreamMedium._flush().
-        
-        For TCP we do no flushing. We may want to turn off TCP_NODELAY and 
+
+        For TCP we do no flushing. We may want to turn off TCP_NODELAY and
         add a means to do a flush, but that can be done in the future.
         """
 
@@ -768,7 +870,17 @@ class SmartTCPClientMedium(SmartClientStreamMedium):
             raise errors.MediumNotConnected(self)
         # We ignore the desired_count because on sockets it's more efficient to
         # read large chunks (of _MAX_READ_SIZE bytes) at a time.
-        return self._socket.recv(_MAX_READ_SIZE)
+        try:
+            bytes = osutils.until_no_eintr(self._socket.recv, _MAX_READ_SIZE)
+        except socket.error, e:
+            if len(e.args) and e.args[0] == errno.ECONNRESET:
+                # Callers expect an empty string in that case
+                return ''
+            else:
+                raise
+        else:
+            self._report_activity(len(bytes), 'read')
+            return bytes
 
 
 class SmartClientStreamMediumRequest(SmartClientMediumRequest):
@@ -787,7 +899,7 @@ class SmartClientStreamMediumRequest(SmartClientMediumRequest):
 
     def _accept_bytes(self, bytes):
         """See SmartClientMediumRequest._accept_bytes.
-        
+
         This forwards to self._medium._accept_bytes because we are operating
         on the mediums stream.
         """
@@ -796,13 +908,13 @@ class SmartClientStreamMediumRequest(SmartClientMediumRequest):
     def _finished_reading(self):
         """See SmartClientMediumRequest._finished_reading.
 
-        This clears the _current_request on self._medium to allow a new 
+        This clears the _current_request on self._medium to allow a new
         request to be created.
         """
         if self._medium._current_request is not self:
             raise AssertionError()
         self._medium._current_request = None
-        
+
     def _finished_writing(self):
         """See SmartClientMediumRequest._finished_writing.
 
