@@ -29,7 +29,7 @@ from bzrlib import debug
 from bzrlib import errors
 from bzrlib.smart import message, request
 from bzrlib.trace import log_exception_quietly, mutter
-from bzrlib.util.bencode import bdecode, bencode
+from bzrlib.util.bencode import bdecode_as_tuple, bencode
 
 
 # Protocol version strings.  These are sent as prefixes of bzr requests and
@@ -109,11 +109,11 @@ class SmartProtocolBase(object):
         for start, length in offsets:
             txt.append('%d,%d' % (start, length))
         return '\n'.join(txt)
-        
+
 
 class SmartServerRequestProtocolOne(SmartProtocolBase):
     """Server-side encoding and decoding logic for smart version 1."""
-    
+
     def __init__(self, backing_transport, write_func, root_client_path='/'):
         self._backing_transport = backing_transport
         self._root_client_path = root_client_path
@@ -127,7 +127,7 @@ class SmartServerRequestProtocolOne(SmartProtocolBase):
 
     def accept_bytes(self, bytes):
         """Take bytes, and advance the internal state machine appropriately.
-        
+
         :param bytes: must be a byte string
         """
         if not isinstance(bytes, str):
@@ -169,7 +169,7 @@ class SmartServerRequestProtocolOne(SmartProtocolBase):
 
         if self._has_dispatched:
             if self._finished:
-                # nothing to do.XXX: this routine should be a single state 
+                # nothing to do.XXX: this routine should be a single state
                 # machine too.
                 self.unused_data += self.in_buffer
                 self.in_buffer = ''
@@ -211,7 +211,7 @@ class SmartServerRequestProtocolOne(SmartProtocolBase):
 
     def _write_protocol_version(self):
         """Write any prefixes this protocol requires.
-        
+
         Version one doesn't send protocol versions.
         """
 
@@ -234,7 +234,7 @@ class SmartServerRequestProtocolOne(SmartProtocolBase):
 
 class SmartServerRequestProtocolTwo(SmartServerRequestProtocolOne):
     r"""Version two of the server side of the smart protocol.
-   
+
     This prefixes responses with the value of RESPONSE_VERSION_TWO.
     """
 
@@ -250,7 +250,7 @@ class SmartServerRequestProtocolTwo(SmartServerRequestProtocolOne):
 
     def _write_protocol_version(self):
         r"""Write any prefixes this protocol requires.
-        
+
         Version two sends the value of RESPONSE_VERSION_TWO.
         """
         self._write_func(self.response_marker)
@@ -412,7 +412,7 @@ class ChunkedBodyDecoder(_StatefulDecoder):
         self.chunks = collections.deque()
         self.error = False
         self.error_in_progress = None
-    
+
     def next_read_size(self):
         # Note: the shortest possible chunk is 2 bytes: '0\n', and the
         # end-of-body marker is 4 bytes: 'END\n'.
@@ -456,7 +456,6 @@ class ChunkedBodyDecoder(_StatefulDecoder):
 
     def _finished(self):
         self.unused_data = self._get_in_buffer()
-        # self._in_buffer = None
         self._in_buffer_list = []
         self._in_buffer_len = 0
         self.state_accept = self._state_accept_reading_unused
@@ -507,7 +506,7 @@ class ChunkedBodyDecoder(_StatefulDecoder):
                 self.chunks.append(self.chunk_in_progress)
             self.chunk_in_progress = None
             self.state_accept = self._state_accept_expecting_length
-        
+
     def _state_accept_reading_unused(self):
         self.unused_data += self._get_in_buffer()
         self._in_buffer_list = []
@@ -515,14 +514,14 @@ class ChunkedBodyDecoder(_StatefulDecoder):
 
 class LengthPrefixedBodyDecoder(_StatefulDecoder):
     """Decodes the length-prefixed bulk data."""
-    
+
     def __init__(self):
         _StatefulDecoder.__init__(self)
         self.state_accept = self._state_accept_expecting_length
         self.state_read = self._state_read_no_data
         self._body = ''
         self._trailer_buffer = ''
-    
+
     def next_read_size(self):
         if self.bytes_left is not None:
             # Ideally we want to read all the remainder of the body and the
@@ -538,7 +537,7 @@ class LengthPrefixedBodyDecoder(_StatefulDecoder):
         else:
             # Reading excess data.  Either way, 1 byte at a time is fine.
             return 1
-        
+
     def read_pending_data(self):
         """Return any pending data that has been decoded."""
         return self.state_read()
@@ -565,7 +564,7 @@ class LengthPrefixedBodyDecoder(_StatefulDecoder):
                 self._body = self._body[:self.bytes_left]
             self.bytes_left = None
             self.state_accept = self._state_accept_reading_trailer
-        
+
     def _state_accept_reading_trailer(self):
         self._trailer_buffer += self._get_in_buffer()
         self._set_in_buffer(None)
@@ -575,7 +574,7 @@ class LengthPrefixedBodyDecoder(_StatefulDecoder):
             self.unused_data = self._trailer_buffer[len('done\n'):]
             self.state_accept = self._state_accept_reading_unused
             self.finished_reading = True
-    
+
     def _state_accept_reading_unused(self):
         self.unused_data += self._get_in_buffer()
         self._set_in_buffer(None)
@@ -657,6 +656,14 @@ class SmartClientRequestProtocolOne(SmartProtocolBase, Requester,
             mutter('              %d bytes in readv request', len(readv_bytes))
         self._last_verb = args[0]
 
+    def call_with_body_stream(self, args, stream):
+        # Protocols v1 and v2 don't support body streams.  So it's safe to
+        # assume that a v1/v2 server doesn't support whatever method we're
+        # trying to call with a body stream.
+        self._request.finished_writing()
+        self._request.finished_reading()
+        raise errors.UnknownSmartMethod(args[0])
+
     def cancel_read_body(self):
         """After expecting a body, a response code may indicate one otherwise.
 
@@ -722,7 +729,7 @@ class SmartClientRequestProtocolOne(SmartProtocolBase, Requester,
     def _response_is_unknown_method(self, result_tuple):
         """Raise UnexpectedSmartServerResponse if the response is an 'unknonwn
         method' response to the request.
-        
+
         :param response: The response from a smart client call_expecting_body
             call.
         :param verb: The verb used in that call.
@@ -735,11 +742,11 @@ class SmartClientRequestProtocolOne(SmartProtocolBase, Requester,
             # The response will have no body, so we've finished reading.
             self._request.finished_reading()
             raise errors.UnknownSmartMethod(self._last_verb)
-        
+
     def read_body_bytes(self, count=-1):
         """Read bytes from the body, decoding into a byte stream.
-        
-        We read all bytes at once to ensure we've checked the trailer for 
+
+        We read all bytes at once to ensure we've checked the trailer for
         errors, and then feed the buffer back as read_body_bytes is called.
         """
         if self._body_buffer is not None:
@@ -783,14 +790,14 @@ class SmartClientRequestProtocolOne(SmartProtocolBase, Requester,
 
     def _write_protocol_version(self):
         """Write any prefixes this protocol requires.
-        
+
         Version one doesn't send protocol versions.
         """
 
 
 class SmartClientRequestProtocolTwo(SmartClientRequestProtocolOne):
     """Version two of the client side of the smart protocol.
-    
+
     This prefixes the request with the value of REQUEST_VERSION_TWO.
     """
 
@@ -824,7 +831,7 @@ class SmartClientRequestProtocolTwo(SmartClientRequestProtocolOne):
 
     def _write_protocol_version(self):
         """Write any prefixes this protocol requires.
-        
+
         Version two sends the value of REQUEST_VERSION_TWO.
         """
         self._request.accept_bytes(self.request_marker)
@@ -932,7 +939,7 @@ class ProtocolThreeDecoder(_StatefulDecoder):
     def _extract_prefixed_bencoded_data(self):
         prefixed_bytes = self._extract_length_prefixed_bytes()
         try:
-            decoded = bdecode(prefixed_bytes)
+            decoded = bdecode_as_tuple(prefixed_bytes)
         except ValueError:
             raise errors.SmartProtocolError(
                 'Bytes %r not bencoded' % (prefixed_bytes,))
@@ -978,7 +985,7 @@ class ProtocolThreeDecoder(_StatefulDecoder):
             self.message_handler.headers_received(decoded)
         except:
             raise errors.SmartMessageHandlerError(sys.exc_info())
-    
+
     def _state_accept_expecting_message_part(self):
         message_part_kind = self._extract_single_byte()
         if message_part_kind == 'o':
@@ -1053,16 +1060,18 @@ class _ProtocolThreeEncoder(object):
     response_marker = request_marker = MESSAGE_VERSION_THREE
 
     def __init__(self, write_func):
-        self._buf = ''
+        self._buf = []
         self._real_write_func = write_func
 
     def _write_func(self, bytes):
-        self._buf += bytes
+        self._buf.append(bytes)
+        if len(self._buf) > 100:
+            self.flush()
 
     def flush(self):
         if self._buf:
-            self._real_write_func(self._buf)
-            self._buf = ''
+            self._real_write_func(''.join(self._buf))
+            del self._buf[:]
 
     def _serialise_offsets(self, offsets):
         """Serialise a readv offset list."""
@@ -1070,7 +1079,7 @@ class _ProtocolThreeEncoder(object):
         for start, length in offsets:
             txt.append('%d,%d' % (start, length))
         return '\n'.join(txt)
-        
+
     def _write_protocol_version(self):
         self._write_func(MESSAGE_VERSION_THREE)
 
@@ -1101,6 +1110,9 @@ class _ProtocolThreeEncoder(object):
         self._write_func(struct.pack('!L', len(bytes)))
         self._write_func(bytes)
 
+    def _write_chunked_body_start(self):
+        self._write_func('oC')
+
     def _write_error_status(self):
         self._write_func('oE')
 
@@ -1148,11 +1160,57 @@ class ProtocolThreeResponder(_ProtocolThreeEncoder):
         if response.body is not None:
             self._write_prefixed_body(response.body)
         elif response.body_stream is not None:
-            for chunk in response.body_stream:
-                self._write_prefixed_body(chunk)
-                self.flush()
+            for exc_info, chunk in _iter_with_errors(response.body_stream):
+                if exc_info is not None:
+                    self._write_error_status()
+                    error_struct = request._translate_error(exc_info[1])
+                    self._write_structure(error_struct)
+                    break
+                else:
+                    if isinstance(chunk, request.FailedSmartServerResponse):
+                        self._write_error_status()
+                        self._write_structure(chunk.args)
+                        break
+                    self._write_prefixed_body(chunk)
         self._write_end()
-        
+
+
+def _iter_with_errors(iterable):
+    """Handle errors from iterable.next().
+
+    Use like::
+
+        for exc_info, value in _iter_with_errors(iterable):
+            ...
+
+    This is a safer alternative to::
+
+        try:
+            for value in iterable:
+               ...
+        except:
+            ...
+
+    Because the latter will catch errors from the for-loop body, not just
+    iterable.next()
+
+    If an error occurs, exc_info will be a exc_info tuple, and the generator
+    will terminate.  Otherwise exc_info will be None, and value will be the
+    value from iterable.next().  Note that KeyboardInterrupt and SystemExit
+    will not be itercepted.
+    """
+    iterator = iter(iterable)
+    while True:
+        try:
+            yield None, iterator.next()
+        except StopIteration:
+            return
+        except (KeyboardInterrupt, SystemExit):
+            raise
+        except Exception:
+            yield sys.exc_info(), None
+            return
+
 
 class ProtocolThreeRequester(_ProtocolThreeEncoder, Requester):
 
@@ -1163,7 +1221,7 @@ class ProtocolThreeRequester(_ProtocolThreeEncoder, Requester):
 
     def set_headers(self, headers):
         self._headers = headers.copy()
-        
+
     def call(self, *args):
         if 'hpss' in debug.debug_flags:
             mutter('hpss call:   %s', repr(args)[1:-1])
@@ -1218,3 +1276,33 @@ class ProtocolThreeRequester(_ProtocolThreeEncoder, Requester):
         self._write_end()
         self._medium_request.finished_writing()
 
+    def call_with_body_stream(self, args, stream):
+        if 'hpss' in debug.debug_flags:
+            mutter('hpss call w/body stream: %r', args)
+            path = getattr(self._medium_request._medium, '_path', None)
+            if path is not None:
+                mutter('                  (to %s)', path)
+            self._request_start_time = time.time()
+        self._write_protocol_version()
+        self._write_headers(self._headers)
+        self._write_structure(args)
+        # TODO: notice if the server has sent an early error reply before we
+        #       have finished sending the stream.  We would notice at the end
+        #       anyway, but if the medium can deliver it early then it's good
+        #       to short-circuit the whole request...
+        for exc_info, part in _iter_with_errors(stream):
+            if exc_info is not None:
+                # Iterating the stream failed.  Cleanly abort the request.
+                self._write_error_status()
+                # Currently the client unconditionally sends ('error',) as the
+                # error args.
+                self._write_structure(('error',))
+                self._write_end()
+                self._medium_request.finished_writing()
+                raise exc_info[0], exc_info[1], exc_info[2]
+            else:
+                self._write_prefixed_body(part)
+                self.flush()
+        self._write_end()
+        self._medium_request.finished_writing()
+
