@@ -25,25 +25,13 @@ unique ID.
 """
 
 import os
-from cStringIO import StringIO
-import urllib
-from zlib import adler32
 
-import bzrlib
 from bzrlib import (
     errors,
-    osutils,
-    symbol_versioning,
-    urlutils,
     versionedfile,
     )
-from bzrlib.errors import BzrError, UnlistableStore, TransportNotPossible
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    )
+from bzrlib.errors import BzrError, UnlistableStore
 from bzrlib.trace import mutter
-from bzrlib.transport import Transport
-from bzrlib.transport.local import LocalTransport
 
 ######################################################################
 # stores
@@ -54,7 +42,7 @@ class StoreError(Exception):
 
 class Store(object):
     """This class represents the abstract storage layout for saving information.
-    
+
     Files can be added, but not modified once they are in.  Typically
     the hash is used as the name, or something else known to be unique,
     such as a UUID.
@@ -65,7 +53,7 @@ class Store(object):
 
     def get(self, fileid, suffix=None):
         """Returns a file reading from a particular entry.
-        
+
         If suffix is present, retrieve the named suffix for fileid.
         """
         raise NotImplementedError
@@ -83,8 +71,8 @@ class Store(object):
 
     def has_id(self, fileid, suffix=None):
         """Return True or false for the presence of fileid in the store.
-        
-        suffix, if present, is a per file suffix, i.e. for digital signature 
+
+        suffix, if present, is a per file suffix, i.e. for digital signature
         data."""
         raise NotImplementedError
 
@@ -148,10 +136,10 @@ class Store(object):
 
     def _copy_one(self, fileid, suffix, other, pb):
         """Most generic copy-one object routine.
-        
+
         Subclasses can override this to provide an optimised
         copy between their own instances. Such overriden routines
-        should call this if they have no optimised facility for a 
+        should call this if they have no optimised facility for a
         specific 'other'.
         """
         mutter('Store._copy_one: %r', fileid)
@@ -170,7 +158,7 @@ class TransportStore(Store):
         mutter("add store entry %r", fileid)
         names = self._id_to_names(fileid, suffix)
         if self._transport.has_any(names):
-            raise BzrError("store %r already contains id %r" 
+            raise BzrError("store %r already contains id %r"
                            % (self._transport.base, fileid))
 
         # Most of the time, just adding the file will work
@@ -211,7 +199,7 @@ class TransportStore(Store):
 
     def _get_name(self, fileid, suffix=None):
         """A special check, which returns the name of an existing file.
-        
+
         This is similar in spirit to 'has_id', but it is designed
         to return information about which file the store has.
         """
@@ -223,7 +211,7 @@ class TransportStore(Store):
     def _get(self, filename):
         """Return an vanilla file stream for clients to read from.
 
-        This is the body of a template method on 'get', and should be 
+        This is the body of a template method on 'get', and should be
         implemented by subclasses.
         """
         raise NotImplementedError
@@ -329,5 +317,5 @@ class TransportStore(Store):
         for relpath in self._transport.iter_files_recursive():
             count += 1
             total += self._transport.stat(relpath).st_size
-                
+
         return count, total
