@@ -24,7 +24,7 @@ from bzrlib.tests.test_win32utils import NeedsGlobExpansionFeature
 
 
 class TestAdd(ExternalBase):
-        
+
     def test_add_reports(self):
         """add command prints the names of added files."""
         tree = self.make_branch_and_tree('.')
@@ -35,10 +35,10 @@ class TestAdd(ExternalBase):
         results = sorted(out.rstrip('\n').split('\n'))
         self.assertEquals(['If you wish to add some of these files, please'\
                            ' add them by name.',
-                           'added .bzrignore',
-                           'added dir',
-                           'added dir/sub.txt',
-                           'added top.txt',
+                           'adding .bzrignore',
+                           'adding dir',
+                           'adding dir/sub.txt',
+                           'adding top.txt',
                            'ignored 1 file(s).'],
                           results)
         out = self.run_bzr('add -v')[0]
@@ -50,7 +50,7 @@ class TestAdd(ExternalBase):
 
     def test_add_quiet_is(self):
         """add -q does not print the names of added files."""
-        tree = self.make_branch_and_tree('.') 
+        tree = self.make_branch_and_tree('.')
         self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
         out = self.run_bzr('add -q')[0]
         # the ordering is not defined at the moment
@@ -100,18 +100,18 @@ class TestAdd(ExternalBase):
         eq = self.assertEqual
         ass = self.assertTrue
         chdir = os.chdir
-        
+
         t = self.make_branch_and_tree('.')
         b = t.branch
         self.build_tree(['src/', 'README'])
-        
+
         eq(sorted(t.unknowns()),
            ['README', 'src'])
-        
+
         self.run_bzr('add src')
-        
+
         self.build_tree(['src/foo.c'])
-        
+
         # add with no arguments in a subdirectory gets only files below that
         # subdirectory
         chdir('src')
@@ -122,7 +122,7 @@ class TestAdd(ExternalBase):
         versioned = [path for path, entry in t.iter_entries_by_dir()]
         self.assertEquals(versioned,
             ['', 'src', 'src/foo.c'])
-                
+
         # add from the parent directory should pick up all file names
         chdir('..')
         self.run_bzr('add')
@@ -146,9 +146,9 @@ class TestAdd(ExternalBase):
         os.chdir('new')
         out, err = self.run_bzr('add --file-ids-from ../base')
         self.assertEqual('', err)
-        self.assertEqualDiff('added a w/ file id from a\n'
-                             'added b w/ file id from b\n'
-                             'added b/c w/ file id from b/c\n',
+        self.assertEqualDiff('adding a w/ file id from a\n'
+                             'adding b w/ file id from b\n'
+                             'adding b/c w/ file id from b/c\n',
                              out)
         new_tree = new_tree.bzrdir.open_workingtree()
         self.assertEqual(base_tree.path2id('a'), new_tree.path2id('a'))
@@ -167,8 +167,8 @@ class TestAdd(ExternalBase):
         os.chdir('new')
         out, err = self.run_bzr('add --file-ids-from ../base/b')
         self.assertEqual('', err)
-        self.assertEqualDiff('added c w/ file id from b/c\n'
-                             'added d w/ file id from b/d\n',
+        self.assertEqualDiff('adding c w/ file id from b/c\n'
+                             'adding d w/ file id from b/d\n',
                              out)
 
         new_tree = new_tree.bzrdir.open_workingtree()
