@@ -20,46 +20,52 @@
 """
 
 import os
+import sys
 
 from bzrlib.tests.blackbox import TestCaseWithTransport
 
 class TestCat(TestCaseWithTransport):
 
     def test_cat(self):
-
-        def bzr(*args, **kwargs):
-            return self.run_bzr_subprocess(*args, **kwargs)[0]
-
-        os.mkdir('branch')
+        tree = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/a', 'foo\n')])
+        tree.add('a')
         os.chdir('branch')
-        bzr('init')
-        open('a', 'wb').write('foo\n')
-        bzr('add', 'a')
-
         # 'bzr cat' without an option should cat the last revision
-        bzr('cat', 'a', retcode=3)
+        self.run_bzr(['cat', 'a'], retcode=3)
 
-        bzr('commit', '-m', '1')
-        open('a', 'wb').write('baz\n')
+        tree.commit(message='1')
+        self.build_tree_contents([('a', 'baz\n')])
 
-        self.assertEquals(bzr('cat', 'a'), 'foo\n')
+        # We use run_bzr_subprocess rather than run_bzr here so that we can
+        # test mangling of line-endings on Windows.
+        self.assertEquals(self.run_bzr_subprocess(['cat', 'a'])[0], 'foo\n')
 
-        bzr('commit', '-m', '2')
-        self.assertEquals(bzr('cat', 'a'), 'baz\n')
-        self.assertEquals(bzr('cat', 'a', '-r', '1'), 'foo\n')
-        self.assertEquals(bzr('cat', 'a', '-r', '-1'), 'baz\n')
+        tree.commit(message='2')
+        self.assertEquals(self.run_bzr_subprocess(['cat', 'a'])[0], 'baz\n')
+        self.assertEquals(self.run_bzr_subprocess(
+            ['cat', 'a', '-r', '1'])[0],
+            'foo\n')
+        self.assertEquals(self.run_bzr_subprocess(
+            ['cat', 'a', '-r', '-1'])[0],
+            'baz\n')
 
-        rev_id = bzr('revision-history').strip().split('\n')[-1]
+        rev_id = tree.branch.last_revision()
+
+        self.assertEquals(self.run_bzr_subprocess(
+            ['cat', 'a', '-r', 'revid:%s' % rev_id])[0],
+            'baz\n')
 
-        self.assertEquals(bzr('cat', 'a', '-r', 'revid:%s' % rev_id), 'baz\n')
-        
         os.chdir('..')
-        
-        self.assertEquals(bzr('cat', 'branch/a', '-r', 'revno:1:branch'),
-                          'foo\n')
-        bzr('cat', 'a', retcode=3)
-        bzr('cat', 'a', '-r', 'revno:1:branch-that-does-not-exist', retcode=3)
-        
+
+        self.assertEquals(self.run_bzr_subprocess(
+            ['cat', 'branch/a', '-r', 'revno:1:branch'])[0],
+            'foo\n')
+        self.run_bzr(['cat', 'a'], retcode=3)
+        self.run_bzr(
+                ['cat', 'a', '-r', 'revno:1:branch-that-does-not-exist'],
+                retcode=3)
+
     def test_cat_different_id(self):
         """'cat' works with old and new files"""
         tree = self.make_branch_and_tree('.')
@@ -73,7 +79,7 @@ class TestCat(TestCaseWithTransport):
         try:
             tree.add(['a-rev-tree', 'c-rev', 'd-rev'])
             tree.commit('add test files')
-            # remove currently uses self._write_inventory - 
+            # remove currently uses self._write_inventory -
             # work around that for now.
             tree.flush()
             tree.remove(['d-rev'])
@@ -89,15 +95,16 @@ class TestCat(TestCaseWithTransport):
                            'cat b-tree --name-from-revision')
 
         # get to the old file automatically
-        out, err = self.run_bzr('cat d-rev')
+        out, err = self.run_bzr_subprocess('cat d-rev')
         self.assertEqual('bar\n', out)
         self.assertEqual('', err)
 
-        out, err = self.run_bzr('cat a-rev-tree --name-from-revision')
+        out, err = \
+                self.run_bzr_subprocess('cat a-rev-tree --name-from-revision')
         self.assertEqual('foo\n', out)
         self.assertEqual('', err)
 
-        out, err = self.run_bzr('cat a-rev-tree')
+        out, err = self.run_bzr_subprocess('cat a-rev-tree')
         self.assertEqual('baz\n', out)
         self.assertEqual('', err)
 
@@ -108,7 +115,7 @@ class TestCat(TestCaseWithTransport):
         wt.commit('Making sure there is a basis_tree available')
 
         url = self.get_readonly_url() + '/README'
-        out, err = self.run_bzr(['cat', url])
+        out, err = self.run_bzr_subprocess(['cat', url])
         self.assertEqual('contents of README\n', out)
 
     def test_cat_no_working_tree(self):
@@ -119,6 +126,12 @@ class TestCat(TestCaseWithTransport):
         wt.branch.bzrdir.destroy_workingtree()
 
         url = self.get_readonly_url() + '/README'
-        out, err = self.run_bzr(['cat', url])
+        out, err = self.run_bzr_subprocess(['cat', url])
         self.assertEqual('contents of README\n', out)
-        
+
+    def test_cat_nonexistent_branch(self):
+        if sys.platform == "win32":
+            location = "C:/i/do/not/exist"
+        else:
+            location = "/i/do/not/exist"
+        self.run_bzr_error(['^bzr: ERROR: Not a branch'], ['cat', location])
