@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -61,6 +61,7 @@ Location:
 """, out)
         self.assertEqual('', err)
 
+        # Standalone branch - verbose mode
         out, err = self.run_bzr('info standalone -v')
         self.assertEqualDiff(
 """Standalone tree (format: weave)
@@ -83,6 +84,37 @@ In the working tree:
          0 ignored
          0 versioned subdirectories
 
+Branch history:
+         0 revisions
+
+Repository:
+         0 revisions
+""", out)
+        self.assertEqual('', err)
+
+        # Standalone branch - really verbose mode
+        out, err = self.run_bzr('info standalone -vv')
+        self.assertEqualDiff(
+"""Standalone tree (format: weave)
+Location:
+  branch root: standalone
+
+Format:
+       control: All-in-one format 6
+  working tree: Working tree format 2
+        branch: Branch format 4
+    repository: Weave repository format 6
+
+In the working tree:
+         0 unchanged
+         0 modified
+         1 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
 Branch history:
          0 revisions
          0 committers
@@ -139,7 +171,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -186,7 +217,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -230,7 +260,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -248,6 +277,7 @@ Repository:
         out, err = self.run_bzr('info -v lightcheckout')
         self.assertEqualDiff(
 """Lightweight checkout (format: 1.6 or 1.6.1-rich-root \
+or 1.9 or 1.9-rich-root \
 or dirstate or dirstate-tags or \
 pack-0.92 or rich-root or rich-root-pack)
 Location:
@@ -272,7 +302,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -318,7 +347,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -360,7 +388,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -400,7 +427,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -416,6 +442,7 @@ Repository:
         out, err = self.run_bzr('info lightcheckout --verbose')
         self.assertEqualDiff(
 """Lightweight checkout (format: 1.6 or 1.6.1-rich-root or \
+1.9 or 1.9-rich-root or \
 dirstate or dirstate-tags or \
 pack-0.92 or rich-root or rich-root-pack)
 Location:
@@ -442,7 +469,6 @@ In the working tree:
 
 Branch history:
          2 revisions
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -470,7 +496,6 @@ Format:
 
 Branch history:
          0 revisions
-         0 committers
 
 Repository:
          0 revisions
@@ -521,7 +546,6 @@ Format:
 
 Branch history:
          0 revisions
-         0 committers
 
 Repository:
          0 revisions
@@ -533,7 +557,7 @@ Repository:
         # Create lightweight checkout
         transport.mkdir('tree')
         transport.mkdir('tree/lightcheckout')
-        tree2 = branch1.create_checkout('tree/lightcheckout', 
+        tree2 = branch1.create_checkout('tree/lightcheckout',
             lightweight=True)
         branch2 = tree2.branch
         self.assertCheckoutStatusOutput('-v tree/lightcheckout', tree2,
@@ -553,6 +577,7 @@ Repository:
         out, err = self.run_bzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
 """Lightweight checkout (format: 1.6 or 1.6.1-rich-root or \
+1.9 or 1.9-rich-root or \
 dirstate or dirstate-tags or \
 pack-0.92 or rich-root or rich-root-pack)
 Location:
@@ -578,7 +603,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -619,7 +643,6 @@ In the working tree:
 
 Branch history:
          0 revisions
-         0 committers
 
 Repository:
          0 revisions
@@ -657,7 +680,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -677,6 +699,7 @@ Repository:
         out, err = self.run_bzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
 """Lightweight checkout (format: 1.6 or 1.6.1-rich-root or \
+1.9 or 1.9-rich-root or \
 dirstate or dirstate-tags or \
 pack-0.92 or rich-root or rich-root-pack)
 Location:
@@ -704,7 +727,6 @@ In the working tree:
 
 Branch history:
          2 revisions
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -732,7 +754,6 @@ Format:
 
 Branch history:
          2 revisions
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -819,7 +840,6 @@ In the working tree:
 
 Branch history:
          0 revisions
-         0 committers
 
 Repository:
          0 revisions
@@ -860,7 +880,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -902,7 +921,6 @@ In the working tree:
 
 Branch history:
          0 revisions
-         0 committers
 
 Repository:
          1 revision
@@ -942,7 +960,6 @@ In the working tree:
 
 Branch history:
          1 revision
-         1 committer
          0 days old
    first revision: %s
   latest revision: %s
@@ -974,7 +991,7 @@ Repository:
        ),
        out)
         self.assertEqual('', err)
-    
+
     def test_info_shared_repository_with_tree_in_root(self):
         format = bzrdir.format_registry.make_bzrdir('knit')
         transport = self.get_transport()
@@ -1029,7 +1046,6 @@ In the working tree:
 
 Branch history:
          0 revisions
-         0 committers
 
 Repository:
          0 revisions
@@ -1053,7 +1069,7 @@ Repository:
         allow us, the test writers, to document what *should* be present in
         the output. Removing this separation would remove the value of the
         tests.
-        
+
         :param path: the path to the light checkout.
         :param lco_tree: the tree object for the light checkout.
         :param shared_repo: A shared repository is in use, expect that in
@@ -1067,7 +1083,7 @@ Repository:
             actually locked then this parameter is overridden. This is because
             pack repositories do not have any public API for obtaining an
             exclusive repository wide lock.
-        :param verbose: If true, expect verbose output
+        :param verbose: verbosity level: 2 or higher to show committers
         """
         def friendly_location(url):
             path = urlutils.unescape_for_display(url, 'ascii')
@@ -1093,6 +1109,7 @@ Repository:
             (False, False): 'Checkout',
             }[(shared_repo is not None, light_checkout)]
         format = {True: '1.6 or 1.6.1-rich-root'
+                        ' or 1.9 or 1.9-rich-root'
                         ' or dirstate or dirstate-tags or pack-0.92'
                         ' or rich-root or rich-root-pack',
                   False: 'dirstate'}[light_checkout]
@@ -1138,12 +1155,12 @@ Repository:
         else:
             branch_data = ("   checkout of branch: %s\n" %
                 lco_tree.branch.bzrdir.root_transport.base)
-        
-        if verbose:
+
+        if verbose >= 2:
             verbose_info = '         0 committers\n'
         else:
             verbose_info = ''
-            
+
         self.assertEqualDiff(
 """%s (format: %s)
 Location:
@@ -1331,7 +1348,6 @@ In the working tree:
 
 Branch history:
          0 revisions
-         0 committers
 
 Repository:
          0 revisions
@@ -1364,7 +1380,6 @@ In the working tree:
 
 Branch history:
          0 revisions
-         0 committers
 
 Repository:
          0 revisions
@@ -1376,13 +1391,13 @@ Repository:
     def test_info_stacked(self):
         # We have a mainline
         trunk_tree = self.make_branch_and_tree('mainline',
-            format='development1')
+            format='1.6')
         trunk_tree.commit('mainline')
         # and a branch from it which is stacked
         new_dir = trunk_tree.bzrdir.sprout('newbranch', stacked=True)
         out, err = self.run_bzr('info newbranch')
         self.assertEqual(
-"""Standalone tree (format: development1)
+"""Standalone tree (format: 1.6)
 Location:
   branch root: newbranch
 
