@@ -22,6 +22,7 @@ import re
 
 from bzrlib import (
     branch as _mod_branch,
+    urlutils,
     )
 from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.tests import TestSkipped
@@ -44,7 +45,8 @@ class TestInit(ExternalBase):
         # --format=weave should be accepted to allow interoperation with
         # old releases when desired.
         out, err = self.run_bzr('init --format=weave')
-        self.assertEqual('', out)
+        self.assertEqual("""Created a standalone tree (format: weave)\n""",
+            out)
         self.assertEqual('', err)
 
     def test_init_at_repository_root(self):
@@ -57,36 +59,49 @@ class TestInit(ExternalBase):
         repo = newdir.create_repository(shared=True)
         repo.set_make_working_trees(False)
         out, err = self.run_bzr('init repo')
-        self.assertEqual('', out)
+        self.assertEqual("""Created a repository tree (format: pack-0.92)
+Using shared repository: %s
+""" % urlutils.local_path_from_url(
+            repo.bzrdir.root_transport.external_url()), out)
+        self.assertEndsWith(out, "bzrlib.tests.blackbox.test_init.TestInit."
+            "test_init_at_repository_root/work/repo/\n")
         self.assertEqual('', err)
         newdir.open_branch()
         newdir.open_workingtree()
-        
+
     def test_init_branch(self):
         out, err = self.run_bzr('init')
-        self.assertEqual('', out)
+        self.assertEqual("""Created a standalone tree (format: pack-0.92)\n""",
+            out)
         self.assertEqual('', err)
 
         # Can it handle subdirectories of branches too ?
         out, err = self.run_bzr('init subdir1')
-        self.assertEqual('', out)
+        self.assertEqual("""Created a standalone tree (format: pack-0.92)\n""",
+            out)
         self.assertEqual('', err)
         WorkingTree.open('subdir1')
-        
+
         self.run_bzr_error(['Parent directory of subdir2/nothere does not exist'],
                             'init subdir2/nothere')
         out, err = self.run_bzr('init subdir2/nothere', retcode=3)
         self.assertEqual('', out)
-        
+
         os.mkdir('subdir2')
         out, err = self.run_bzr('init subdir2')
-        self.assertEqual('', out)
+        self.assertEqual("""Created a standalone tree (format: pack-0.92)\n""",
+            out)
         self.assertEqual('', err)
         # init an existing branch.
         out, err = self.run_bzr('init subdir2', retcode=3)
         self.assertEqual('', out)
         self.failUnless(err.startswith('bzr: ERROR: Already a branch:'))
 
+    def test_init_branch_quiet(self):
+        out, err = self.run_bzr('init -q')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+
     def test_init_existing_branch(self):
         self.run_bzr('init')
         out, err = self.run_bzr('init', retcode=3)
@@ -118,7 +133,7 @@ class TestInit(ExternalBase):
         except UnicodeError:
             raise TestSkipped("Unable to create Unicode filename")
         # try to init unicode dir
-        self.run_bzr(['init', u'mu-\xb5'])
+        self.run_bzr(['init', '-q', u'mu-\xb5'])
 
     def create_simple_tree(self):
         tree = self.make_branch_and_tree('tree')
@@ -142,9 +157,10 @@ class TestSFTPInit(TestCaseWithSFTPServer):
     def test_init(self):
         # init on a remote url should succeed.
         out, err = self.run_bzr(['init', self.get_url()])
-        self.assertEqual('', out)
+        self.assertEqual(out,
+            """Created a standalone branch (format: pack-0.92)\n""")
         self.assertEqual('', err)
-    
+
     def test_init_existing_branch(self):
         # when there is already a branch present, make mention
         self.make_branch('.')
