@@ -20,22 +20,31 @@ import sys
 import os
 
 from bzrlib import osutils, urlutils
-import bzrlib
-from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.tests import (
+    TestCaseWithTransport,
+    TestSkipped,
+    multiply_tests,
+    )
+from bzrlib.tests.EncodingAdapter import encoding_scenarios
 from bzrlib.trace import mutter, note
 
 
+def load_tests(standard_tests, module, loader):
+    return multiply_tests(standard_tests, encoding_scenarios,
+        loader.suiteClass())
+
+
 class TestNonAscii(TestCaseWithTransport):
     """Test that bzr handles files/committers/etc which are non-ascii."""
 
     def setUp(self):
         super(TestNonAscii, self).setUp()
         self._orig_email = os.environ.get('BZR_EMAIL', None)
-        self._orig_encoding = bzrlib.user_encoding
+        self._orig_encoding = osutils._cached_user_encoding
 
-        bzrlib.user_encoding = self.encoding
+        osutils._cached_user_encoding = self.encoding
         email = self.info['committer'] + ' <joe@foo.com>'
-        os.environ['BZR_EMAIL'] = email.encode(bzrlib.user_encoding)
+        os.environ['BZR_EMAIL'] = email.encode(osutils.get_user_encoding())
         self.create_base()
 
     def tearDown(self):
@@ -44,7 +53,7 @@ class TestNonAscii(TestCaseWithTransport):
         else:
             if os.environ.get('BZR_EMAIL', None) is not None:
                 del os.environ['BZR_EMAIL']
-        bzrlib.user_encoding = self._orig_encoding
+        osutils._cached_user_encoding = self._orig_encoding
         super(TestNonAscii, self).tearDown()
 
     def run_bzr_decode(self, args, encoding=None, fail=False, retcode=None,
@@ -53,11 +62,11 @@ class TestNonAscii(TestCaseWithTransport):
 
         Returns a string containing the stdout output from bzr.
 
-        :param fail: If true, the operation is expected to fail with 
+        :param fail: If true, the operation is expected to fail with
             a UnicodeError.
         """
         if encoding is None:
-            encoding = bzrlib.user_encoding
+            encoding = osutils.get_user_encoding()
         try:
             out = self.run_bzr(args, output_encoding=encoding, encoding=encoding,
                 retcode=retcode, working_dir=working_dir)[0]
@@ -235,7 +244,7 @@ class TestNonAscii(TestCaseWithTransport):
         txt = self.run_bzr_decode('pull', working_dir=dirname2)
 
         expected = osutils.pathjoin(osutils.getcwd(), dirname1)
-        self.assertEqual(u'Using saved location: %s/\n'
+        self.assertEqual(u'Using saved parent location: %s/\n'
                 'No revisions to pull.\n' % (expected,), txt)
 
         self.build_tree_contents(
@@ -299,7 +308,7 @@ class TestNonAscii(TestCaseWithTransport):
         fname = self.info['filename']
         txt = self.run_bzr_decode(['file-id', fname])
 
-        # TODO: jam 20060106 We don't support non-ascii file ids yet, 
+        # TODO: jam 20060106 We don't support non-ascii file ids yet,
         #       so there is nothing which would fail in ascii encoding
         #       This *should* be retcode=3
         txt = self.run_bzr_decode(['file-id', fname], encoding='ascii')
@@ -319,23 +328,23 @@ class TestNonAscii(TestCaseWithTransport):
 
         txt = self.run_bzr_decode(['file-path', path])
 
-        # TODO: jam 20060106 We don't support non-ascii file ids yet, 
+        # TODO: jam 20060106 We don't support non-ascii file ids yet,
         #       so there is nothing which would fail in ascii encoding
         #       This *should* be retcode=3
         txt = self.run_bzr_decode(['file-path', path], encoding='ascii')
 
     def test_revision_history(self):
-        # TODO: jam 20060106 We don't support non-ascii revision ids yet, 
+        # TODO: jam 20060106 We don't support non-ascii revision ids yet,
         #       so there is nothing which would fail in ascii encoding
         txt = self.run_bzr_decode('revision-history')
 
     def test_ancestry(self):
-        # TODO: jam 20060106 We don't support non-ascii revision ids yet, 
+        # TODO: jam 20060106 We don't support non-ascii revision ids yet,
         #       so there is nothing which would fail in ascii encoding
         txt = self.run_bzr_decode('ancestry')
 
     def test_diff(self):
-        # TODO: jam 20060106 diff is a difficult one to test, because it 
+        # TODO: jam 20060106 diff is a difficult one to test, because it
         #       shouldn't encode the file contents, but it needs some sort
         #       of encoding for the paths, etc which are displayed.
         self.build_tree_contents([(self.info['filename'], 'newline\n')])
@@ -414,7 +423,7 @@ class TestNonAscii(TestCaseWithTransport):
         self.wt.commit(u'Renamed %s => %s' % (fname, fname2))
 
         txt = self.run_bzr_decode(['touching-revisions', fname2])
-        expected_txt = (u'     3 added %s\n' 
+        expected_txt = (u'     3 added %s\n'
                         u'     4 renamed %s => %s\n'
                         % (fname, fname, fname2))
         self.assertEqual(expected_txt, txt)
@@ -437,7 +446,7 @@ class TestNonAscii(TestCaseWithTransport):
         fname = self.info['filename'] + '2'
         self.build_tree_contents([(fname, 'unknown\n')])
 
-        # TODO: jam 20060112 bzr unknowns is the only one which 
+        # TODO: jam 20060112 bzr unknowns is the only one which
         #       quotes paths do we really want it to?
         #       awilkins 20080521 added and modified do it now as well
         txt = self.run_bzr_decode('unknowns')
