@@ -19,7 +19,6 @@
 """Black-box tests for running bzr outside of a working tree."""
 
 import os
-import tempfile
 
 from bzrlib import (
     osutils,
@@ -32,8 +31,9 @@ class TestOutsideWT(tests.ChrootedTestCase):
     """Test that bzr gives proper errors outside of a working tree."""
 
     def test_cwd_log(self):
-        tmp_dir = tempfile.mkdtemp()
+        tmp_dir = osutils.mkdtemp()
         self.addCleanup(lambda: osutils.rmtree(tmp_dir))
+        self.addCleanup(lambda: os.chdir('..'))
         os.chdir(tmp_dir)
         out, err = self.run_bzr('log', retcode=3)
         self.assertEqual(u'bzr: ERROR: Not a branch: "%s/".\n'
@@ -47,8 +47,9 @@ class TestOutsideWT(tests.ChrootedTestCase):
                          u' "%s".\n' % url, err)
 
     def test_diff_outside_tree(self):
-        tmp_dir = tempfile.mkdtemp()
+        tmp_dir = osutils.mkdtemp()
         self.addCleanup(lambda: osutils.rmtree(tmp_dir))
+        self.addCleanup(lambda: os.chdir('..'))
         os.chdir(tmp_dir)
         self.run_bzr('init branch1')
         self.run_bzr(['commit', '-m', 'nothing',
