@@ -38,7 +38,7 @@ class TestRemoveTree(ExternalBase):
         os.chdir('branch1')
         self.run_bzr('remove-tree')
         self.failIfExists('foo')
-    
+
     def test_remove_tree_original_branch_explicit(self):
         self.run_bzr('remove-tree branch1')
         self.failIfExists('branch1/foo')
@@ -49,7 +49,7 @@ class TestRemoveTree(ExternalBase):
         os.chdir('branch2')
         self.run_bzr('remove-tree')
         self.failIfExists('foo')
-    
+
     def test_remove_tree_sprouted_branch_explicit(self):
         self.tree.bzrdir.sprout('branch2')
         self.failUnlessExists('branch2/foo')
@@ -64,7 +64,7 @@ class TestRemoveTree(ExternalBase):
         self.failIfExists('foo')
         os.chdir('..')
         self.failUnlessExists('branch1/foo')
-    
+
     def test_remove_tree_checkout_explicit(self):
         self.tree.branch.create_checkout('branch2', lightweight=False)
         self.failUnlessExists('branch2/foo')
@@ -84,7 +84,7 @@ class TestRemoveTree(ExternalBase):
         self.failUnlessExists('foo')
         os.chdir('..')
         self.failUnlessExists('branch1/foo')
-    
+
     def test_remove_tree_lightweight_checkout_explicit(self):
         self.tree.branch.create_checkout('branch2', lightweight=True)
         self.failUnlessExists('branch2/foo')
@@ -109,3 +109,16 @@ class TestRemoveTree(ExternalBase):
     def test_remove_tree_remote_path(self):
         # TODO: I can't think of a way to implement this...
         pass
+
+    def test_remove_tree_uncommitted_changes(self):
+        self.build_tree(['branch1/bar'])
+        self.tree.add('bar')
+        output = self.run_bzr_error(["Working tree .* has uncommitted changes"],
+                                    'remove-tree branch1', retcode=3)
+
+    def test_remove_tree_uncommitted_changes_force(self):
+        self.build_tree(['branch1/bar'])
+        self.tree.add('bar')
+        self.run_bzr('remove-tree branch1 --force')
+        self.failIfExists('branch1/foo')
+        self.failUnlessExists('branch1/bar')
