@@ -25,26 +25,39 @@ from bzrlib.tests import TestCaseInTempDir
 class TestSharedRepo(TestCaseInTempDir):
 
     def test_make_repository(self):
-        out, err = self.run_bzr("init-repository", "a")
+        out, err = self.run_bzr("init-repository a")
+        self.assertEqual(out,
+"""Shared repository with trees (format: pack-0.92)
+Location:
+  shared repository: a
+""")
+        self.assertEqual(err, "")
+        dir = BzrDir.open('a')
+        self.assertIs(dir.open_repository().is_shared(), True)
+        self.assertRaises(errors.NotBranchError, dir.open_branch)
+        self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)
+
+    def test_make_repository_quiet(self):
+        out, err = self.run_bzr("init-repository a -q")
         self.assertEqual(out, "")
         self.assertEqual(err, "")
         dir = BzrDir.open('a')
         self.assertIs(dir.open_repository().is_shared(), True)
         self.assertRaises(errors.NotBranchError, dir.open_branch)
-        self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)        
+        self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)
 
     def test_init_repo_existing_dir(self):
         """Make repo in existing directory.
-        
+
         (Malone #38331)
         """
-        out, err = self.run_bzr("init-repository", ".")
+        out, err = self.run_bzr("init-repository .")
         dir = BzrDir.open('.')
         self.assertTrue(dir.open_repository())
 
     def test_init(self):
-        self.run_bzr("init-repo", "a")
-        self.run_bzr("init", "--format=default", "a/b")
+        self.run_bzr("init-repo a")
+        self.run_bzr("init --format=default a/b")
         dir = BzrDir.open('a')
         self.assertIs(dir.open_repository().is_shared(), True)
         self.assertRaises(errors.NotBranchError, dir.open_branch)
@@ -55,22 +68,22 @@ class TestSharedRepo(TestCaseInTempDir):
         wt = bdir.open_workingtree()
 
     def test_branch(self):
-        self.run_bzr("init-repo", "a")
-        self.run_bzr("init", "--format=default", "a/b")
-        self.run_bzr('branch', 'a/b', 'a/c')
+        self.run_bzr("init-repo a")
+        self.run_bzr("init --format=default a/b")
+        self.run_bzr('branch a/b a/c')
         cdir = BzrDir.open('a/c')
         cdir.open_branch()
         self.assertRaises(errors.NoRepositoryPresent, cdir.open_repository)
         cdir.open_workingtree()
 
     def test_branch_tree(self):
-        self.run_bzr("init-repo", "--trees", "a")
-        self.run_bzr("init", "--format=default", "b")
+        self.run_bzr("init-repo --trees a")
+        self.run_bzr("init --format=default b")
         file('b/hello', 'wt').write('bar')
-        self.run_bzr("add", "b/hello")
-        self.run_bzr("commit", "-m", "bar", "b/hello")
+        self.run_bzr("add b/hello")
+        self.run_bzr("commit -m bar b/hello")
 
-        self.run_bzr('branch', 'b', 'a/c')
+        self.run_bzr('branch b a/c')
         cdir = BzrDir.open('a/c')
         cdir.open_branch()
         self.assertRaises(errors.NoRepositoryPresent, cdir.open_repository)
@@ -79,19 +92,32 @@ class TestSharedRepo(TestCaseInTempDir):
 
     def test_trees_default(self):
         # 0.15 switched to trees by default
-        self.run_bzr("init-repo", "repo")
+        self.run_bzr("init-repo repo")
         repo = BzrDir.open("repo").open_repository()
         self.assertEqual(True, repo.make_working_trees())
 
     def test_trees_argument(self):
         # Supplying the --trees argument should be harmless,
         # as it was previously non-default we need to get it right.
-        self.run_bzr("init-repo", "--trees", "trees")
+        self.run_bzr("init-repo --trees trees")
         repo = BzrDir.open("trees").open_repository()
         self.assertEqual(True, repo.make_working_trees())
 
     def test_no_trees_argument(self):
         # --no-trees should make it so that there is no working tree
-        self.run_bzr("init-repo", "--no-trees", "notrees")
+        self.run_bzr("init-repo --no-trees notrees")
         repo = BzrDir.open("notrees").open_repository()
         self.assertEqual(False, repo.make_working_trees())
+
+    def test_init_repo_smart_acceptance(self):
+        # The amount of hpss calls made on init-repo to a smart server should
+        # be fixed.
+        self.setup_smart_server_with_call_log()
+        self.run_bzr(['init-repo', self.get_url('repo')])
+        rpc_count = len(self.hpss_calls)
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertEqual(16, rpc_count)
