@@ -17,7 +17,7 @@
 """Tests of status command.
 
 Most of these depend on the particular formatting used.
-As such they really are blackbox tests even though some of the 
+As such they really are blackbox tests even though some of the
 tests are not using self.capture. If we add tests for the programmatic
 interface later, they will be non blackbox tests.
 """
@@ -32,6 +32,7 @@ from bzrlib import (
     bzrdir,
     conflicts,
     errors,
+    osutils,
     )
 import bzrlib.branch
 from bzrlib.osutils import pathjoin
@@ -42,24 +43,25 @@ from bzrlib.workingtree import WorkingTree
 
 
 class BranchStatus(TestCaseWithTransport):
-    
+
     def assertStatus(self, expected_lines, working_tree,
-        revision=None, short=False, pending=True):
+        revision=None, short=False, pending=True, verbose=False):
         """Run status in working_tree and look for output.
-        
+
         :param expected_lines: The lines to look for.
         :param working_tree: The tree to run status in.
         """
         output_string = self.status_string(working_tree, revision, short,
-                pending)
+                pending, verbose)
         self.assertEqual(expected_lines, output_string.splitlines(True))
-    
-    def status_string(self, wt, revision=None, short=False, pending=True):
+
+    def status_string(self, wt, revision=None, short=False, pending=True,
+        verbose=False):
         # use a real file rather than StringIO because it doesn't handle
         # Unicode very well.
         tof = codecs.getwriter('utf-8')(TemporaryFile())
         show_tree_status(wt, to_file=tof, revision=revision, short=short,
-                show_pending=pending)
+                show_pending=pending, verbose=verbose)
         tof.seek(0)
         return tof.read().decode('utf-8')
 
@@ -94,10 +96,18 @@ class BranchStatus(TestCaseWithTransport):
                 'unknown:\n',
                 '  bye.c\n',
                 '  hello.c\n',
-                'pending merges:\n',
+                'pending merge tips: (use -v to see all merge revisions)\n',
                 '  (ghost) pending@pending-0-0\n',
             ],
             wt)
+        self.assertStatus([
+                'unknown:\n',
+                '  bye.c\n',
+                '  hello.c\n',
+                'pending merges:\n',
+                '  (ghost) pending@pending-0-0\n',
+            ],
+            wt, verbose=True)
         self.assertStatus([
                 '?   bye.c\n',
                 '?   hello.c\n',
@@ -137,7 +147,7 @@ class BranchStatus(TestCaseWithTransport):
         self.build_tree(['more.c'])
         wt.add('more.c')
         wt.commit('Another test message')
-        
+
         revs.append(RevisionSpec.from_string('1'))
         self.assertStatus([
                 'added:\n',
@@ -158,7 +168,7 @@ class BranchStatus(TestCaseWithTransport):
         wt2 = b_2_dir.open_workingtree()
         wt.commit(u"\N{TIBETAN DIGIT TWO} Empty commit 2")
         wt2.merge_from_branch(wt.branch)
-        message = self.status_string(wt2)
+        message = self.status_string(wt2, verbose=True)
         self.assertStartsWith(message, "pending merges:\n")
         self.assertEndsWith(message, "Empty commit 2\n")
         wt2.commit("merged")
@@ -166,7 +176,7 @@ class BranchStatus(TestCaseWithTransport):
         wt.commit("Empty commit 3 " +
                    "blah blah blah blah " * 100)
         wt2.merge_from_branch(wt.branch)
-        message = self.status_string(wt2)
+        message = self.status_string(wt2, verbose=True)
         self.assertStartsWith(message, "pending merges:\n")
         self.assert_("Empty commit 3" in message)
         self.assertEndsWith(message, "...\n")
@@ -196,7 +206,7 @@ class BranchStatus(TestCaseWithTransport):
         wt.add('directory')
         wt.add('test.c')
         wt.commit('testing')
-        
+
         self.assertStatus([
                 'unknown:\n',
                 '  bye.c\n',
@@ -215,9 +225,9 @@ class BranchStatus(TestCaseWithTransport):
         tof = StringIO()
         self.assertRaises(errors.PathsDoNotExist,
                           show_tree_status,
-                          wt, specific_files=['bye.c','test.c','absent.c'], 
+                          wt, specific_files=['bye.c','test.c','absent.c'],
                           to_file=tof)
-        
+
         tof = StringIO()
         show_tree_status(wt, specific_files=['directory'], to_file=tof)
         tof.seek(0)
@@ -274,12 +284,176 @@ class BranchStatus(TestCaseWithTransport):
         self.assertEqualDiff('conflicts:\n  Contents conflict in dir2/file1\n',
                              tof.getvalue())
 
+    def _prepare_nonexistent(self):
+        wt = self.make_branch_and_tree('.')
+        self.assertStatus([], wt)
+        self.build_tree(['FILE_A', 'FILE_B', 'FILE_C', 'FILE_D', 'FILE_E', ])
+        wt.add('FILE_A')
+        wt.add('FILE_B')
+        wt.add('FILE_C')
+        wt.add('FILE_D')
+        wt.add('FILE_E')
+        wt.commit('Create five empty files.')
+        open('FILE_B', 'w').write('Modification to file FILE_B.')
+        open('FILE_C', 'w').write('Modification to file FILE_C.')
+        unlink('FILE_E')  # FILE_E will be versioned but missing
+        open('FILE_Q', 'w').write('FILE_Q is added but not committed.')
+        wt.add('FILE_Q')  # FILE_Q will be added but not committed
+        open('UNVERSIONED_BUT_EXISTING', 'w')
+        return wt
+
     def test_status_nonexistent_file(self):
         # files that don't exist in either the basis tree or working tree
         # should give an error
-        wt = self.make_branch_and_tree('.')
-        out, err = self.run_bzr('status does-not-exist', retcode=3)
-        self.assertContainsRe(err, r'do not exist.*does-not-exist')
+        wt = self._prepare_nonexistent()
+        self.assertStatus([
+            'removed:\n',
+            '  FILE_E\n',
+            'added:\n',
+            '  FILE_Q\n',
+            'modified:\n',
+            '  FILE_B\n',
+            '  FILE_C\n',
+            'unknown:\n',
+            '  UNVERSIONED_BUT_EXISTING\n',
+            ],
+            wt)
+        self.assertStatus([
+            ' M  FILE_B\n',
+            ' M  FILE_C\n',
+            ' D  FILE_E\n',
+            '+N  FILE_Q\n',
+            '?   UNVERSIONED_BUT_EXISTING\n',
+            ],
+            wt, short=True)
+
+        # Okay, everything's looking good with the existent files.
+        # Let's see what happens when we throw in non-existent files.
+
+        # bzr st [--short] NONEXISTENT '
+        expected = [
+          'nonexistent:\n',
+          '  NONEXISTENT\n',
+          ]
+        out, err = self.run_bzr('status NONEXISTENT', retcode=3)
+        self.assertEqual(expected, out.splitlines(True))
+        self.assertContainsRe(err,
+                              r'.*ERROR: Path\(s\) do not exist: '
+                              'NONEXISTENT.*')
+        expected = [
+          'X:   NONEXISTENT\n',
+          ]
+        out, err = self.run_bzr('status --short NONEXISTENT', retcode=3)
+        self.assertContainsRe(err,
+                              r'.*ERROR: Path\(s\) do not exist: '
+                              'NONEXISTENT.*')
+
+    def test_status_nonexistent_file_with_others(self):
+        # bzr st [--short] NONEXISTENT ...others..
+        wt = self._prepare_nonexistent()
+        expected = [
+          'removed:\n',
+          '  FILE_E\n',
+          'modified:\n',
+          '  FILE_B\n',
+          '  FILE_C\n',
+          'nonexistent:\n',
+          '  NONEXISTENT\n',
+          ]
+        out, err = self.run_bzr('status NONEXISTENT '
+                                'FILE_A FILE_B FILE_C FILE_D FILE_E',
+                                retcode=3)
+        self.assertEqual(expected, out.splitlines(True))
+        self.assertContainsRe(err,
+                              r'.*ERROR: Path\(s\) do not exist: '
+                              'NONEXISTENT.*')
+        expected = [
+          ' D  FILE_E\n',
+          ' M  FILE_C\n',
+          ' M  FILE_B\n',
+          'X   NONEXISTENT\n',
+          ]
+        out, err = self.run_bzr('status --short NONEXISTENT '
+                                'FILE_A FILE_B FILE_C FILE_D FILE_E',
+                                retcode=3)
+        self.assertEqual(expected, out.splitlines(True))
+        self.assertContainsRe(err,
+                              r'.*ERROR: Path\(s\) do not exist: '
+                              'NONEXISTENT.*')
+
+    def test_status_multiple_nonexistent_files(self):
+        # bzr st [--short] NONEXISTENT ... ANOTHER_NONEXISTENT ...
+        wt = self._prepare_nonexistent()
+        expected = [
+          'removed:\n',
+          '  FILE_E\n',
+          'modified:\n',
+          '  FILE_B\n',
+          '  FILE_C\n',
+          'nonexistent:\n',
+          '  ANOTHER_NONEXISTENT\n',
+          '  NONEXISTENT\n',
+          ]
+        out, err = self.run_bzr('status NONEXISTENT '
+                                'FILE_A FILE_B ANOTHER_NONEXISTENT '
+                                'FILE_C FILE_D FILE_E', retcode=3)
+        self.assertEqual(expected, out.splitlines(True))
+        self.assertContainsRe(err,
+                              r'.*ERROR: Path\(s\) do not exist: '
+                              'ANOTHER_NONEXISTENT NONEXISTENT.*')
+        expected = [
+          ' D  FILE_E\n',
+          ' M  FILE_C\n',
+          ' M  FILE_B\n',
+          'X   ANOTHER_NONEXISTENT\n',
+          'X   NONEXISTENT\n',
+          ]
+        out, err = self.run_bzr('status --short NONEXISTENT '
+                                'FILE_A FILE_B ANOTHER_NONEXISTENT '
+                                'FILE_C FILE_D FILE_E', retcode=3)
+        self.assertEqual(expected, out.splitlines(True))
+        self.assertContainsRe(err,
+                              r'.*ERROR: Path\(s\) do not exist: '
+                              'ANOTHER_NONEXISTENT NONEXISTENT.*')
+
+    def test_status_nonexistent_file_with_unversioned(self):
+        # bzr st [--short] NONEXISTENT A B UNVERSIONED_BUT_EXISTING C D E Q
+        wt = self._prepare_nonexistent()
+        expected = [
+          'removed:\n',
+          '  FILE_E\n',
+          'added:\n',
+          '  FILE_Q\n',
+          'modified:\n',
+          '  FILE_B\n',
+          '  FILE_C\n',
+          'unknown:\n',
+          '  UNVERSIONED_BUT_EXISTING\n',
+          'nonexistent:\n',
+          '  NONEXISTENT\n',
+          ]
+        out, err = self.run_bzr('status NONEXISTENT '
+                                'FILE_A FILE_B UNVERSIONED_BUT_EXISTING '
+                                'FILE_C FILE_D FILE_E FILE_Q', retcode=3)
+        self.assertEqual(expected, out.splitlines(True))
+        self.assertContainsRe(err,
+                              r'.*ERROR: Path\(s\) do not exist: '
+                              'NONEXISTENT.*')
+        expected = [
+          '+N  FILE_Q\n',
+          '?   UNVERSIONED_BUT_EXISTING\n',
+          ' D  FILE_E\n',
+          ' M  FILE_C\n',
+          ' M  FILE_B\n',
+          'X   NONEXISTENT\n',
+          ]
+        out, err = self.run_bzr('status --short NONEXISTENT '
+                                'FILE_A FILE_B UNVERSIONED_BUT_EXISTING '
+                                'FILE_C FILE_D FILE_E FILE_Q', retcode=3)
+        self.assertEqual(expected, out.splitlines(True))
+        self.assertContainsRe(err,
+                              r'.*ERROR: Path\(s\) do not exist: '
+                              'NONEXISTENT.*')
 
     def test_status_out_of_date(self):
         """Simulate status of out-of-date tree after remote push"""
@@ -298,6 +472,21 @@ class BranchStatus(TestCaseWithTransport):
         self.assertEqual("working tree is out of date, run 'bzr update'\n",
                          err)
 
+    def test_status_write_lock(self):
+        """Test that status works without fetching history and
+        having a write lock.
+
+        See https://bugs.launchpad.net/bzr/+bug/149270
+        """
+        mkdir('branch1')
+        wt = self.make_branch_and_tree('branch1')
+        b = wt.branch
+        wt.commit('Empty commit 1')
+        wt2 = b.bzrdir.sprout('branch2').open_workingtree()
+        wt2.commit('Empty commit 2')
+        out, err = self.run_bzr('status branch1 -rbranch:branch2')
+        self.assertEqual('', out)
+
 
 class CheckoutStatus(BranchStatus):
 
@@ -305,7 +494,7 @@ class CheckoutStatus(BranchStatus):
         super(CheckoutStatus, self).setUp()
         mkdir('codir')
         chdir('codir')
-        
+
     def make_branch_and_tree(self, relpath):
         source = self.make_branch(pathjoin('..', relpath))
         checkout = bzrdir.BzrDirMetaFormat1().initialize(relpath)
@@ -442,12 +631,28 @@ class TestStatus(TestCaseWithTransport):
         out, err = self.run_bzr('status --no-pending', working_dir='a')
         self.assertEquals(out, "added:\n  b\n")
 
+    def test_pending_specific_files(self):
+        """With a specific file list, pending merges are not shown."""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/a', 'content of a\n')])
+        tree.add('a')
+        r1_id = tree.commit('one')
+        alt = tree.bzrdir.sprout('alt').open_workingtree()
+        self.build_tree_contents([('alt/a', 'content of a\nfrom alt\n')])
+        alt_id = alt.commit('alt')
+        tree.merge_from_branch(alt.branch)
+        output = self.make_utf8_encoded_stringio()
+        show_tree_status(tree, to_file=output)
+        self.assertContainsRe(output.getvalue(), 'pending merge')
+        out, err = self.run_bzr('status tree/a')
+        self.assertNotContainsRe(out, 'pending merge')
+
 
 class TestStatusEncodings(TestCaseWithTransport):
-    
+
     def setUp(self):
         TestCaseWithTransport.setUp(self)
-        self.user_encoding = bzrlib.user_encoding
+        self.user_encoding = osutils._cached_user_encoding
         self.stdout = sys.stdout
 
     def tearDown(self):
@@ -469,7 +674,7 @@ class TestStatusEncodings(TestCaseWithTransport):
 
     def test_stdout_ascii(self):
         sys.stdout = StringIO()
-        bzrlib.user_encoding = 'ascii'
+        osutils._cached_user_encoding = 'ascii'
         working_tree = self.make_uncommitted_tree()
         stdout, stderr = self.run_bzr("status")
 
@@ -480,7 +685,7 @@ added:
 
     def test_stdout_latin1(self):
         sys.stdout = StringIO()
-        bzrlib.user_encoding = 'latin-1'
+        osutils._cached_user_encoding = 'latin-1'
         working_tree = self.make_uncommitted_tree()
         stdout, stderr = self.run_bzr('status')
 
