@@ -24,7 +24,7 @@ from bzrlib.tests import TestCaseWithTransport
 class TestVersionInfo(TestCaseWithTransport):
 
     def test_invalid_format(self):
-        self.run_bzr('version-info', '--format', 'quijibo', retcode=3)
+        self.run_bzr('version-info --format quijibo', retcode=3)
 
     def create_tree(self):
         wt = self.make_branch_and_tree('branch')
@@ -43,7 +43,7 @@ class TestVersionInfo(TestCaseWithTransport):
     def test_basic(self):
         self.create_tree()
 
-        txt = self.run_bzr('version-info', 'branch')[0]
+        txt = self.run_bzr('version-info branch')[0]
         self.assertContainsRe(txt, 'date:')
         self.assertContainsRe(txt, 'build-date:')
         self.assertContainsRe(txt, 'revno: 2')
@@ -52,8 +52,7 @@ class TestVersionInfo(TestCaseWithTransport):
     def test_all(self):
         """'--all' includes clean, revision history, and file revisions"""
         self.create_tree()
-        txt = self.run_bzr('version-info', 'branch',
-                           '--all')[0]
+        txt = self.run_bzr('version-info branch --all')[0]
         self.assertContainsRe(txt, 'date:')
         self.assertContainsRe(txt, 'revno: 2')
         self.assertContainsRe(txt, 'revision-id: ' + self.revisions[-1])
@@ -71,17 +70,15 @@ class TestVersionInfo(TestCaseWithTransport):
         """Test that --check-clean includes the right info"""
         self.create_tree()
 
-        txt = self.run_bzr('version-info', 'branch',
-                           '--check-clean')[0]
+        txt = self.run_bzr('version-info branch --check-clean')[0]
         self.assertContainsRe(txt, 'clean: True')
 
         self.build_tree_contents([('branch/c', 'now unclean\n')])
-        txt = self.run_bzr('version-info', 'branch',
-                           '--check-clean')[0]
+        txt = self.run_bzr('version-info branch --check-clean')[0]
         self.assertContainsRe(txt, 'clean: False')
 
-        txt = self.run_bzr('version-info', 'branch',
-                           '--check-clean', '--include-file-revisions')[0]
+        txt = self.run_bzr('version-info branch --check-clean'
+                           ' --include-file-revisions')[0]
         self.assertContainsRe(txt, 'revision: unversioned')
 
         os.remove('branch/c')
@@ -91,7 +88,7 @@ class TestVersionInfo(TestCaseWithTransport):
         branch = self.make_branch('just_branch')
         branch.pull(tree.branch)
 
-        txt = self.run_bzr('version-info', 'just_branch')[0]
+        txt = self.run_bzr('version-info just_branch')[0]
         self.assertStartsWith(txt, 'revision-id: r2\n')
 
     def assertEqualNoBuildDate(self, text1, text2):
@@ -114,7 +111,7 @@ class TestVersionInfo(TestCaseWithTransport):
         """Test that bzr defaults to the local working directory"""
         self.create_tree()
 
-        txt1 = self.run_bzr('version-info', 'branch')[0]
+        txt1 = self.run_bzr('version-info branch')[0]
 
         os.chdir('branch')
         txt2 = self.run_bzr('version-info')[0]
@@ -124,9 +121,9 @@ class TestVersionInfo(TestCaseWithTransport):
         """Test that we can pass --format=rio"""
         self.create_tree()
 
-        txt = self.run_bzr('version-info', 'branch')[0]
-        txt1 = self.run_bzr('version-info', '--format', 'rio', 'branch')[0]
-        txt2 = self.run_bzr('version-info', '--format=rio', 'branch')[0]
+        txt = self.run_bzr('version-info branch')[0]
+        txt1 = self.run_bzr('version-info --format rio branch')[0]
+        txt2 = self.run_bzr('version-info --format=rio branch')[0]
         self.assertEqualNoBuildDate(txt, txt1)
         self.assertEqualNoBuildDate(txt, txt2)
 
@@ -134,6 +131,23 @@ class TestVersionInfo(TestCaseWithTransport):
         """Test that we can do --format=python"""
         self.create_tree()
 
-        txt = self.run_bzr('version-info', '--format', 'python', 'branch')[0]
+        txt = self.run_bzr('version-info --format python branch')[0]
 
         self.assertContainsRe(txt, 'version_info = {')
+
+    def test_custom_without_template(self):
+        wt = self.make_branch_and_tree('branch')
+        out, err = self.run_bzr('version-info --custom', retcode=3)
+        self.assertContainsRe(err, r'ERROR: No template specified\.')
+
+    def test_custom_implies_all(self):
+        self.create_tree()
+        out, err = self.run_bzr('version-info --custom --template='
+            '"{revno} {branch_nick} {clean}\n" branch')
+        self.assertEqual("2 branch 1\n", out)
+        self.assertEqual("", err)
+        self.build_tree_contents([('branch/c', 'now unclean\n')])
+        out, err = self.run_bzr('version-info --custom --template='
+            '"{revno} {branch_nick} {clean}\n" branch')
+        self.assertEqual("2 branch 0\n", out)
+        self.assertEqual("", err)
