@@ -50,7 +50,12 @@ class TestBreakLock(TestCaseWithBranch):
     def test_unlocked_repo_locked(self):
         # break lock on the branch should try on the repository even
         # if the branch isn't locked
-        self.branch.repository.lock_write()
+        token = self.branch.repository.lock_write()
+        if token is None:
+            self.branch.repository.unlock()
+            raise TestNotApplicable('Repository does not use physical locks.')
+        self.branch.repository.leave_lock_in_place()
+        self.branch.repository.unlock()
         other_instance = self.branch.repository.bzrdir.open_repository()
         if not other_instance.get_physical_lock_status():
             raise TestNotApplicable("Repository does not lock persistently.")
