@@ -28,11 +28,13 @@ from bzrlib import (
     bzrdir,
     errors,
     lockdir,
+    osutils,
     repository,
     revision as _mod_revision,
     transactions,
     transport,
     ui,
+    urlutils,
     workingtree,
     )
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
@@ -56,7 +58,7 @@ from bzrlib.trace import mutter
 from bzrlib.transport import get_transport
 from bzrlib.transport.local import LocalTransport
 from bzrlib.upgrade import upgrade
-from bzrlib.remote import RemoteBzrDir
+from bzrlib.remote import RemoteBzrDir, RemoteRepository
 from bzrlib.repofmt import weaverepo
 
 
@@ -73,10 +75,10 @@ class TestBzrDir(TestCaseWithBzrDir):
         """Assert that the content of source and target are identical.
 
         paths in ignore list will be completely ignored.
-        
+
         We ignore paths that represent data which is allowed to change during
         a clone or sprout: for instance, inventory.knit contains gzip fragements
-        which have timestamps in them, and as we have read the inventory from 
+        which have timestamps in them, and as we have read the inventory from
         the source knit, the already-read data is recompressed rather than
         reading it again, which leads to changed timestamps. This is ok though,
         because the inventory.kndx file is not ignored, and the integrity of
@@ -124,21 +126,14 @@ class TestBzrDir(TestCaseWithBzrDir):
                 for rev_id in left_repo.all_revision_ids():
                     self.assertEqual(left_repo.get_revision(rev_id),
                         right_repo.get_revision(rev_id))
-                # inventories
-                left_inv_weave = left_repo.inventories
-                right_inv_weave = right_repo.inventories
-                self.assertEqual(set(left_inv_weave.keys()),
-                    set(right_inv_weave.keys()))
-                # XXX: currently this does not handle indirectly referenced
-                # inventories (e.g. where the inventory is a delta basis for
-                # one that is fully present but that the revid for that
-                # inventory is not yet present.)
-                self.assertEqual(set(left_inv_weave.keys()),
-                    set(left_repo.revisions.keys()))
-                left_trees = left_repo.revision_trees(all_revs)
-                right_trees = right_repo.revision_trees(all_revs)
-                for left_tree, right_tree in izip(left_trees, right_trees):
-                    self.assertEqual(left_tree.inventory, right_tree.inventory)
+                # Assert the revision trees (and thus the inventories) are equal
+                sort_key = lambda rev_tree: rev_tree.get_revision_id()
+                rev_trees_a = sorted(
+                    left_repo.revision_trees(all_revs), key=sort_key)
+                rev_trees_b = sorted(
+                    right_repo.revision_trees(all_revs), key=sort_key)
+                for tree_a, tree_b in zip(rev_trees_a, rev_trees_b):
+                    self.assertEqual([], list(tree_a.iter_changes(tree_b)))
                 # texts
                 text_index = left_repo._generate_text_key_index()
                 self.assertEqual(text_index,
@@ -167,7 +162,7 @@ class TestBzrDir(TestCaseWithBzrDir):
 
     def skipIfNoWorkingTree(self, a_bzrdir):
         """Raises TestSkipped if a_bzrdir doesn't have a working tree.
-        
+
         If the bzrdir does have a workingtree, this is a no-op.
         """
         try:
@@ -176,9 +171,17 @@ class TestBzrDir(TestCaseWithBzrDir):
             raise TestSkipped("bzrdir on transport %r has no working tree"
                               % a_bzrdir.transport)
 
+    def openWorkingTreeIfLocal(self, a_bzrdir):
+        """If a_bzrdir is on a local transport, call open_workingtree() on it.
+        """
+        if not isinstance(a_bzrdir.root_transport, LocalTransport):
+            # it's not local, but that's ok
+            return
+        a_bzrdir.open_workingtree()
+
     def createWorkingTreeOrSkip(self, a_bzrdir):
         """Create a working tree on a_bzrdir, or raise TestSkipped.
-        
+
         A simple wrapper for create_workingtree that translates NotLocalUrl into
         TestSkipped.  Returns the newly created working tree.
         """
@@ -189,9 +192,10 @@ class TestBzrDir(TestCaseWithBzrDir):
                               % a_bzrdir.transport)
 
     def sproutOrSkip(self, from_bzrdir, to_url, revision_id=None,
-                     force_new_repo=False, accelerator_tree=None):
+                     force_new_repo=False, accelerator_tree=None,
+                     create_tree_if_local=True):
         """Sprout from_bzrdir into to_url, or raise TestSkipped.
-        
+
         A simple wrapper for from_bzrdir.sprout that translates NotLocalUrl into
         TestSkipped.  Returns the newly sprouted bzrdir.
         """
@@ -201,7 +205,8 @@ class TestBzrDir(TestCaseWithBzrDir):
         target = from_bzrdir.sprout(to_url, revision_id=revision_id,
                                     force_new_repo=force_new_repo,
                                     possible_transports=[to_transport],
-                                    accelerator_tree=accelerator_tree)
+                                    accelerator_tree=accelerator_tree,
+                                    create_tree_if_local=create_tree_if_local)
         return target
 
     def test_create_null_workingtree(self):
@@ -282,7 +287,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/merge-hashes'])
-    
+
     def test_clone_bzrdir_empty_force_new_ignored(self):
         # the force_new_repo parameter should have no effect on an empty
         # bzrdir's clone logic
@@ -291,7 +296,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/merge-hashes'])
-    
+
     def test_clone_bzrdir_repository(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
@@ -385,7 +390,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         self.assertTrue(branch.repository.has_revision('1'))
         self.assertFalse(branch.repository.make_working_trees())
         self.assertTrue(branch.repository.is_shared())
-        
+
     def test_clone_bzrdir_repository_under_shared_force_new_repo(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
@@ -410,7 +415,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a repository with some revisions,
         # and clone it with a revision limit.
-        # 
+        #
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -506,7 +511,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a branch with some revisions,
         # and clone it with a revision limit.
-        # 
+        #
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -518,7 +523,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         dir = source.bzrdir
         target = dir.clone(self.get_url('target'), revision_id='1')
         self.assertEqual('1', target.open_branch().last_revision())
-        
+
     def test_clone_bzrdir_tree_branch_repo(self):
         tree = self.make_branch_and_tree('source')
         self.build_tree(['source/foo'])
@@ -549,11 +554,14 @@ class TestBzrDir(TestCaseWithBzrDir):
         # Ensure no format data is cached
         a_dir = bzrlib.branch.Branch.open_from_transport(
             self.get_transport('source')).bzrdir
-        target_transport = a_dir.root_transport.clone('..').clone('target')
+        target_transport = self.get_transport('target')
         target_bzrdir = a_dir.clone_on_transport(target_transport)
         target_repo = target_bzrdir.open_repository()
         source_branch = bzrlib.branch.Branch.open(
             self.get_vfs_only_url('source'))
+        if isinstance(target_repo, RemoteRepository):
+            target_repo._ensure_real()
+            target_repo = target_repo._real_repository
         self.assertEqual(target_repo._format, source_branch.repository._format)
 
     def test_revert_inventory(self):
@@ -588,7 +596,7 @@ class TestBzrDir(TestCaseWithBzrDir):
             target.open_repository())
 
     def test_clone_bzrdir_tree_branch_reference(self):
-        # a tree with a branch reference (aka a checkout) 
+        # a tree with a branch reference (aka a checkout)
         # should stay a checkout on clone.
         referenced_branch = self.make_branch('referencced')
         dir = self.make_bzrdir('source')
@@ -646,6 +654,13 @@ class TestBzrDir(TestCaseWithBzrDir):
         a_dir.open_branch()
         self.assertRaises(errors.NoWorkingTree, a_dir.open_workingtree)
 
+    def test_clone_respects_stacked(self):
+        branch = self.make_branch('parent')
+        child_transport = self.get_transport('child')
+        child = branch.bzrdir.clone_on_transport(child_transport,
+                                                 stacked_on=branch.base)
+        self.assertEqual(child.open_branch().get_stacked_on_url(), branch.base)
+
     def test_get_branch_reference_on_reference(self):
         """get_branch_reference should return the right url."""
         referenced_branch = self.make_branch('referenced')
@@ -674,12 +689,12 @@ class TestBzrDir(TestCaseWithBzrDir):
 
     def test_sprout_bzrdir_empty(self):
         dir = self.make_bzrdir('source')
-        target = self.sproutOrSkip(dir, self.get_url('target'))
+        target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # creates a new repository branch and tree
         target.open_repository()
         target.open_branch()
-        target.open_workingtree()
+        self.openWorkingTreeIfLocal(target)
 
     def test_sprout_bzrdir_empty_under_shared_repo(self):
         # sprouting an empty dir into a repo uses the repo
@@ -688,7 +703,7 @@ class TestBzrDir(TestCaseWithBzrDir):
             self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = self.sproutOrSkip(dir, self.get_url('target/child'))
+        target = dir.sprout(self.get_url('target/child'))
         self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
         target.open_branch()
         try:
@@ -706,12 +721,11 @@ class TestBzrDir(TestCaseWithBzrDir):
             self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = self.sproutOrSkip(dir, self.get_url('target/child'),
-                                   force_new_repo=True)
+        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
         target.open_repository()
         target.open_branch()
-        target.open_workingtree()
-    
+        self.openWorkingTreeIfLocal(target)
+
     def test_sprout_bzrdir_repository(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
@@ -727,7 +741,7 @@ class TestBzrDir(TestCaseWithBzrDir):
                 dir.open_branch().last_revision())))
         except errors.NotBranchError:
             pass
-        target = self.sproutOrSkip(dir, self.get_url('target'))
+        target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # testing inventory isn't reasonable for repositories
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
@@ -742,7 +756,7 @@ class TestBzrDir(TestCaseWithBzrDir):
             # If we happen to have a tree, we'll guarantee everything
             # except for the tree root is the same.
             inventory_f = file(dir.transport.base+'inventory', 'rb')
-            self.assertContainsRe(inventory_f.read(), 
+            self.assertContainsRe(inventory_f.read(),
                                   '<inventory file_id="TREE_ROOT[^"]*"'
                                   ' format="5">\n</inventory>\n')
             inventory_f.close()
@@ -765,7 +779,7 @@ class TestBzrDir(TestCaseWithBzrDir):
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = self.sproutOrSkip(dir, self.get_url('target/child'))
+        target = dir.sprout(self.get_url('target/child'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertTrue(shared_repo.has_revision('1'))
 
@@ -784,7 +798,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         tree.branch.repository.copy_content_into(shared_repo)
         dir = self.make_bzrdir('shared/source')
         dir.create_branch()
-        target = self.sproutOrSkip(dir, self.get_url('shared/target'))
+        target = dir.sprout(self.get_url('shared/target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
         self.assertTrue(shared_repo.has_revision('1'))
@@ -808,7 +822,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         self.assertTrue(shared_repo.has_revision('1'))
         dir = self.make_bzrdir('shared/source')
         dir.create_branch()
-        target = self.sproutOrSkip(dir, self.get_url('target'))
+        target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
         branch = target.open_branch()
@@ -832,8 +846,7 @@ class TestBzrDir(TestCaseWithBzrDir):
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = self.sproutOrSkip(dir, self.get_url('target/child'),
-                                   force_new_repo=True)
+        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertFalse(shared_repo.has_revision('1'))
 
@@ -841,7 +854,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a repository with some revisions,
         # and sprout it with a revision limit.
-        # 
+        #
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -864,8 +877,10 @@ class TestBzrDir(TestCaseWithBzrDir):
         tree.branch.repository.copy_content_into(source.repository)
         tree.bzrdir.open_branch().copy_content_into(source)
         dir = source.bzrdir
-        target = self.sproutOrSkip(dir, self.get_url('target'))
+        target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
+        target_repo = target.open_repository()
+        self.assertRepositoryHasSameItems(source.repository, target_repo)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     [
                                      './.bzr/basis-inventory-cache',
@@ -876,7 +891,7 @@ class TestBzrDir(TestCaseWithBzrDir):
                                      './.bzr/checkout/stat-cache',
                                      './.bzr/inventory',
                                      './.bzr/parent',
-                                     './.bzr/repository/inventory.knit',
+                                     './.bzr/repository',
                                      './.bzr/stat-cache',
                                      './foo',
                                      ])
@@ -896,7 +911,7 @@ class TestBzrDir(TestCaseWithBzrDir):
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = self.sproutOrSkip(dir, self.get_url('target/child'))
+        target = dir.sprout(self.get_url('target/child'))
         self.assertTrue(shared_repo.has_revision('1'))
 
     def test_sprout_bzrdir_branch_and_repo_shared_force_new_repo(self):
@@ -914,14 +929,13 @@ class TestBzrDir(TestCaseWithBzrDir):
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = self.sproutOrSkip(dir, self.get_url('target/child'),
-                                   force_new_repo=True)
+        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertFalse(shared_repo.has_revision('1'))
 
     def test_sprout_bzrdir_branch_reference(self):
         # sprouting should create a repository if needed and a sprouted branch.
-        referenced_branch = self.make_branch('referencced')
+        referenced_branch = self.make_branch('referenced')
         dir = self.make_bzrdir('source')
         try:
             reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
@@ -930,11 +944,11 @@ class TestBzrDir(TestCaseWithBzrDir):
             # this is ok too, not all formats have to support references.
             return
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
-        target = self.sproutOrSkip(dir, self.get_url('target'))
+        target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
         self.assertEqual(target, target.open_branch().bzrdir)
-        # and as we dont support repositories being detached yet, a repo in 
+        # and as we dont support repositories being detached yet, a repo in
         # place
         target.open_repository()
 
@@ -954,12 +968,12 @@ class TestBzrDir(TestCaseWithBzrDir):
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = self.sproutOrSkip(dir, self.get_url('target/child'))
+        target = dir.sprout(self.get_url('target/child'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
         self.assertEqual(target, target.open_branch().bzrdir)
         # and we want no repository as the target is shared
-        self.assertRaises(errors.NoRepositoryPresent, 
+        self.assertRaises(errors.NoRepositoryPresent,
                           target.open_repository)
         # and we want revision '1' in the shared repo
         self.assertTrue(shared_repo.has_revision('1'))
@@ -980,8 +994,7 @@ class TestBzrDir(TestCaseWithBzrDir):
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
             return
-        target = self.sproutOrSkip(dir, self.get_url('target/child'),
-                                   force_new_repo=True)
+        target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
         self.assertEqual(target, target.open_branch().bzrdir)
@@ -994,7 +1007,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a repository with some revisions,
         # and sprout it with a revision limit.
-        # 
+        #
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -1004,9 +1017,9 @@ class TestBzrDir(TestCaseWithBzrDir):
         tree.branch.repository.copy_content_into(source.repository)
         tree.bzrdir.open_branch().copy_content_into(source)
         dir = source.bzrdir
-        target = self.sproutOrSkip(dir, self.get_url('target'), revision_id='1')
+        target = dir.sprout(self.get_url('target'), revision_id='1')
         self.assertEqual('1', target.open_branch().last_revision())
-        
+
     def test_sprout_bzrdir_tree_branch_repo(self):
         tree = self.make_branch_and_tree('source')
         self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
@@ -1045,11 +1058,11 @@ class TestBzrDir(TestCaseWithBzrDir):
         tree = self.createWorkingTreeOrSkip(dir)
         self.build_tree(['source/subdir/'])
         tree.add('subdir')
-        target = self.sproutOrSkip(dir, self.get_url('target'))
+        target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
         self.assertEqual(target, target.open_branch().bzrdir)
-        # and as we dont support repositories being detached yet, a repo in 
+        # and as we dont support repositories being detached yet, a repo in
         # place
         target.open_repository()
         result_tree = target.open_workingtree()
@@ -1078,7 +1091,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
         self.assertEqual(target, target.open_branch().bzrdir)
-        # and as we dont support repositories being detached yet, a repo in 
+        # and as we dont support repositories being detached yet, a repo in
         # place
         target.open_repository()
         # we trust that the working tree sprouting works via the other tests.
@@ -1111,6 +1124,23 @@ class TestBzrDir(TestCaseWithBzrDir):
                                    accelerator_tree=tree)
         self.assertEqual(['2'], target.open_workingtree().get_parent_ids())
 
+    def test_sprout_branch_no_tree(self):
+        tree = self.make_branch_and_tree('source')
+        self.build_tree(['source/foo'])
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        tree.commit('revision 2', rev_id='2', allow_pointless=True)
+        dir = tree.bzrdir
+        if isinstance(dir, (bzrdir.BzrDirPreSplitOut,)):
+            self.assertRaises(errors.MustHaveWorkingTree, dir.sprout,
+                              self.get_url('target'),
+                              create_tree_if_local=False)
+            return
+        target = dir.sprout(self.get_url('target'), create_tree_if_local=False)
+        self.failIfExists('target/foo')
+        self.assertEqual(tree.branch.last_revision(),
+                         target.open_branch().last_revision())
+
     def test_format_initialize_find_open(self):
         # loopback test to check the current format initializes to itself.
         if not self.bzrdir_format.is_supported():
@@ -1133,6 +1163,28 @@ class TestBzrDir(TestCaseWithBzrDir):
                          opened_dir._format)
         self.failUnless(isinstance(opened_dir, bzrdir.BzrDir))
 
+    def test_format_network_name(self):
+        # All control formats must have a network name.
+        dir = self.make_bzrdir('.')
+        format = dir._format
+        # We want to test that the network_name matches the actual format on
+        # disk. For local control dirsthat means that using network_name as a
+        # key in the registry gives back the same format. For remote obects
+        # we check that the network_name of the RemoteBzrDirFormat we have
+        # locally matches the actual format present on disk.
+        if isinstance(format, bzrdir.RemoteBzrDirFormat):
+            dir._ensure_real()
+            real_dir = dir._real_bzrdir
+            network_name = format.network_name()
+            self.assertEqual(real_dir._format.network_name(), network_name)
+        else:
+            registry = bzrdir.network_format_registry
+            network_name = format.network_name()
+            looked_up_format = registry.get(network_name)
+            self.assertEqual(format.__class__, looked_up_format.__class__)
+        # The network name must be a byte string.
+        self.assertIsInstance(network_name, str)
+
     def test_open_not_bzrdir(self):
         # test the formats specific behaviour for no-content or similar dirs.
         self.assertRaises(NotBranchError,
@@ -1152,7 +1204,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         made_branch = made_control.create_branch()
         self.failUnless(isinstance(made_branch, bzrlib.branch.Branch))
         self.assertEqual(made_control, made_branch.bzrdir)
-        
+
     def test_open_branch(self):
         if not self.bzrdir_format.is_supported():
             # unsupported formats are not loopback testable
@@ -1183,7 +1235,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         self.assertEqual(made_control, made_repo.bzrdir)
 
     def test_create_repository_shared(self):
-        # a bzrdir can create a shared repository or 
+        # a bzrdir can create a shared repository or
         # fail appropriately
         if not self.bzrdir_format.is_supported():
             # unsupported formats are not loopback testable
@@ -1201,7 +1253,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         self.assertTrue(made_repo.is_shared())
 
     def test_create_repository_nonshared(self):
-        # a bzrdir can create a non-shared repository 
+        # a bzrdir can create a non-shared repository
         if not self.bzrdir_format.is_supported():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
@@ -1211,7 +1263,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository(shared=False)
         self.assertFalse(made_repo.is_shared())
-        
+
     def test_open_repository(self):
         if not self.bzrdir_format.is_supported():
             # unsupported formats are not loopback testable
@@ -1240,7 +1292,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         made_tree = self.createWorkingTreeOrSkip(made_control)
         self.failUnless(isinstance(made_tree, workingtree.WorkingTree))
         self.assertEqual(made_control, made_tree.bzrdir)
-        
+
     def test_create_workingtree_revision(self):
         # a bzrdir can construct a working tree for itself @ a specific revision.
         t = self.get_transport()
@@ -1257,7 +1309,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         except errors.NotLocalUrl:
             raise TestSkipped("Can't make working tree on transport %r" % t)
         self.assertEqual(['a'], made_tree.get_parent_ids())
-        
+
     def test_open_workingtree(self):
         if not self.bzrdir_format.is_supported():
             # unsupported formats are not loopback testable
@@ -1365,7 +1417,7 @@ class TestBzrDir(TestCaseWithBzrDir):
             self.get_url('intermediate/child'))
         try:
             child_repo = innermost_control.open_repository()
-            # if there is a repository, then the format cannot ever hit this 
+            # if there is a repository, then the format cannot ever hit this
             # code path.
             return
         except errors.NoRepositoryPresent:
@@ -1386,7 +1438,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         made_control = self.bzrdir_format.initialize(url)
         try:
             child_repo = made_control.open_repository()
-            # if there is a repository, then the format cannot ever hit this 
+            # if there is a repository, then the format cannot ever hit this
             # code path.
             return
         except errors.NoRepositoryPresent:
@@ -1394,7 +1446,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         found_repo = made_control.find_repository()
         self.assertEqual(repo.bzrdir.root_transport.base,
                          found_repo.bzrdir.root_transport.base)
-        
+
     def test_find_repository_standalone_with_containing_shared_repository(self):
         # find repo inside a standalone repo inside a shared repo finds the standalone repo
         try:
@@ -1427,7 +1479,7 @@ class TestBzrDir(TestCaseWithBzrDir):
                             containing_repo.bzrdir.root_transport.base)
 
     def test_find_repository_with_nested_dirs_works(self):
-        # find repo inside a bzrdir inside a bzrdir inside a shared repo 
+        # find repo inside a bzrdir inside a bzrdir inside a shared repo
         # finds the outer shared repo.
         try:
             repo = self.make_repository('.', shared=True)
@@ -1440,7 +1492,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         made_control = self.bzrdir_format.initialize(url)
         try:
             child_repo = made_control.open_repository()
-            # if there is a repository, then the format cannot ever hit this 
+            # if there is a repository, then the format cannot ever hit this
             # code path.
             return
         except errors.NoRepositoryPresent:
@@ -1449,7 +1501,7 @@ class TestBzrDir(TestCaseWithBzrDir):
             self.get_url('intermediate/child'))
         try:
             child_repo = innermost_control.open_repository()
-            # if there is a repository, then the format cannot ever hit this 
+            # if there is a repository, then the format cannot ever hit this
             # code path.
             return
         except errors.NoRepositoryPresent:
@@ -1457,17 +1509,39 @@ class TestBzrDir(TestCaseWithBzrDir):
         found_repo = innermost_control.find_repository()
         self.assertEqual(repo.bzrdir.root_transport.base,
                          found_repo.bzrdir.root_transport.base)
-        
+
     def test_can_and_needs_format_conversion(self):
         # check that we can ask an instance if its upgradable
         dir = self.make_bzrdir('.')
         if dir.can_convert_format():
-            # if its default updatable there must be an updater 
+            # if its default updatable there must be an updater
             # (we force the latest known format as downgrades may not be
             # available
             self.assertTrue(isinstance(dir._format.get_converter(
                 format=dir._format), bzrdir.Converter))
-        dir.needs_format_conversion(None)
+        dir.needs_format_conversion(
+            bzrdir.BzrDirFormat.get_default_format())
+
+    def test_backup_copies_existing(self):
+        tree = self.make_branch_and_tree('test')
+        self.build_tree(['test/a'])
+        tree.add(['a'], ['a-id'])
+        tree.commit('some data to be copied.')
+        old_url, new_url = tree.bzrdir.backup_bzrdir()
+        old_path = urlutils.local_path_from_url(old_url)
+        new_path = urlutils.local_path_from_url(new_url)
+        self.failUnlessExists(old_path)
+        self.failUnlessExists(new_path)
+        for (((dir_relpath1, _), entries1),
+             ((dir_relpath2, _), entries2)) in izip(
+                osutils.walkdirs(old_path),
+                osutils.walkdirs(new_path)):
+            self.assertEquals(dir_relpath1, dir_relpath2)
+            for f1, f2 in zip(entries1, entries2):
+                self.assertEquals(f1[0], f2[0])
+                self.assertEquals(f1[2], f2[2])
+                if f1[2] == "file":
+                    osutils.compare_files(open(f1[4]), open(f2[4]))
 
     def test_upgrade_new_instance(self):
         """Does an available updater work?"""
@@ -1477,7 +1551,7 @@ class TestBzrDir(TestCaseWithBzrDir):
         dir.create_branch()
         self.createWorkingTreeOrSkip(dir)
         if dir.can_convert_format():
-            # if its default updatable there must be an updater 
+            # if its default updatable there must be an updater
             # (we force the latest known format as downgrades may not be
             # available
             pb = ui.ui_factory.nested_progress_bar()
@@ -1513,7 +1587,7 @@ class TestBzrDir(TestCaseWithBzrDir):
             transport=transport)
         self.failUnless(transport.has('.bzr'))
         self.assertRaises((errors.FileExists, errors.DirectoryNotEmpty),
-            bd.retire_bzrdir, limit=0) 
+            bd.retire_bzrdir, limit=0)
 
 
 class TestBreakLock(TestCaseWithBzrDir):
@@ -1608,9 +1682,9 @@ class TestBreakLock(TestCaseWithBzrDir):
         self.assertRaises(errors.LockBroken, master.unlock)
 
     def test_break_lock_tree(self):
-        # break lock with a tree should unlock the tree but not try the 
-        # branch explicitly. However this is very hard to test for as we 
-        # dont have a tree reference class, nor is one needed; 
+        # break lock with a tree should unlock the tree but not try the
+        # branch explicitly. However this is very hard to test for as we
+        # dont have a tree reference class, nor is one needed;
         # the worst case if this code unlocks twice is an extra question
         # being asked.
         tree = self.make_branch_and_tree('.')
@@ -1654,7 +1728,7 @@ class TestTransportConfig(TestCaseWithBzrDir):
 class ChrootedBzrDirTests(ChrootedTestCase):
 
     def test_find_repository_no_repository(self):
-        # loopback test to check the current format fails to find a 
+        # loopback test to check the current format fails to find a
         # share repository correctly.
         if not self.bzrdir_format.is_supported():
             # unsupported formats are not loopback testable
@@ -1669,7 +1743,7 @@ class ChrootedBzrDirTests(ChrootedTestCase):
         made_control = self.bzrdir_format.initialize(self.get_url('subdir'))
         try:
             repo = made_control.open_repository()
-            # if there is a repository, then the format cannot ever hit this 
+            # if there is a repository, then the format cannot ever hit this
             # code path.
             return
         except errors.NoRepositoryPresent:
