@@ -58,3 +58,15 @@ class TestCommitWithBoundBranch(
                           message=u'empty commit', unchanged=True)
         self.assertEquals(1, len(self.connections))
 
+    def test_commit_local(self):
+        """Commits with --local should not connect to the master!"""
+        self.start_logging_connections()
+
+        commit = builtins.cmd_commit()
+        # commit do not provide a directory parameter, we have to change dir
+        # manually
+        os.chdir('local')
+        commit.run(message=u'empty commit', unchanged=True, local=True)
+
+        #it shouldn't open any connections
+        self.assertEquals(0, len(self.connections))
