@@ -32,3 +32,15 @@ class TestPush(TestCaseWithConnectionHookedTransport):
         cmd.outf = tests.StringIOWrapper()
         cmd.run(self.get_url('remote'), directory='branch')
         self.assertEquals(1, len(self.connections))
+
+    def test_push_onto_stacked(self):
+        self.make_branch_and_tree('base', format='1.9')
+        self.make_branch_and_tree('source', format='1.9')
+
+        self.start_logging_connections()
+
+        cmd = cmd_push()
+        cmd.outf = tests.StringIOWrapper()
+        cmd.run(self.get_url('remote'), directory='source',
+                stacked_on=self.get_url('base'))
+        self.assertEqual(1, len(self.connections))
