@@ -20,6 +20,7 @@ import os
 import posixpath
 import random
 import re
+import select
 import SimpleHTTPServer
 import socket
 import SocketServer
@@ -33,10 +34,6 @@ from bzrlib import transport
 from bzrlib.transport import local
 
 
-class WebserverNotAvailable(Exception):
-    pass
-
-
 class BadWebserverPath(ValueError):
     def __str__(self):
         return 'path %s is not in %s' % self.args
@@ -140,7 +137,7 @@ class TestingHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
             # common)
             self.send_response(301)
             self.send_header("Location", self.path + "/")
-            # Indicates that the body is empty for HTTP/1.1 clients 
+            # Indicates that the body is empty for HTTP/1.1 clients
             self.send_header('Content-Length', '0')
             self.end_headers()
             return None
@@ -180,7 +177,7 @@ class TestingHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
             content_length += self._header_line_length(
                 'Content-Range', 'bytes %d-%d/%d' % (start, end, file_size))
             content_length += len('\r\n') # end headers
-            content_length += end - start # + 1
+            content_length += end - start + 1
         content_length += len(boundary_line)
         self.send_header('Content-length', content_length)
         self.end_headers()
@@ -323,7 +320,7 @@ class TestingHTTPServerMixin:
 
     def tearDown(self):
          """Called to clean-up the server.
- 
+
          Since the server may be (surely is, even) in a blocking listen, we
          shutdown its socket before closing it.
          """
@@ -346,7 +343,12 @@ class TestingHTTPServerMixin:
              # WSAENOTCONN (10057) 'Socket is not connected' is harmless on
              # windows (occurs before the first connection attempt
              # vila--20071230)
-             if not len(e.args) or e.args[0] != 10057:
+
+             # 'Socket is not connected' can also occur on OSX, with a
+             # "regular" ENOTCONN (when something went wrong during test case
+             # setup leading to self.setUp() *not* being called but
+             # self.tearDown() still being called -- vila20081106
+             if not len(e.args) or e.args[0] not in (errno.ENOTCONN, 10057):
                  raise
          # Let the server properly close the socket
          self.server_close()
@@ -418,6 +420,13 @@ class HttpServer(transport.Server):
         # Allows tests to verify number of GET requests issued
         self.GET_request_nb = 0
 
+    def create_httpd(self, serv_cls, rhandler_cls):
+        return serv_cls((self.host, self.port), self.request_handler, self)
+
+    def __repr__(self):
+        return "%s(%s:%s)" % \
+            (self.__class__.__name__, self.host, self.port)
+
     def _get_httpd(self):
         if self._httpd is None:
             rhandler = self.request_handler
@@ -435,7 +444,7 @@ class HttpServer(transport.Server):
             if serv_cls is None:
                 raise httplib.UnknownProtocol(proto_vers)
             else:
-                self._httpd = serv_cls((self.host, self.port), rhandler, self)
+                self._httpd = self.create_httpd(serv_cls, rhandler)
             host, self.port = self._httpd.socket.getsockname()
         return self._httpd
 
@@ -467,6 +476,14 @@ class HttpServer(transport.Server):
                 httpd.handle_request()
             except socket.timeout:
                 pass
+            except (socket.error, select.error), e:
+               if e[0] == errno.EBADF:
+                   # Starting with python-2.6, handle_request may raise socket
+                   # or select exceptions when the server is shut down (as we
+                   # do).
+                   pass
+               else:
+                   raise
 
     def _get_remote_url(self, path):
         path_parts = path.split(os.path.sep)
@@ -486,7 +503,7 @@ class HttpServer(transport.Server):
 
     def setUp(self, backing_transport_server=None):
         """See bzrlib.transport.Server.setUp.
-        
+
         :param backing_transport_server: The transport that requests over this
             protocol should be forwarded to. Note that this is currently not
             supported for HTTP.
