@@ -20,8 +20,10 @@ import sys
 import bzrlib
 from bzrlib import (
     errors,
-    repository,
+    inventory,
     osutils,
+    repository,
+    versionedfile,
     )
 from bzrlib.errors import (
     NoSuchRevision,
@@ -36,6 +38,9 @@ from bzrlib.tests import (
 from bzrlib.tests.interrepository_implementations import (
     TestCaseWithInterRepository,
     )
+from bzrlib.tests.interrepository_implementations.test_interrepository import (
+    check_repo_format_for_funky_id_on_win32
+    )
 
 
 class TestInterRepository(TestCaseWithInterRepository):
@@ -66,10 +71,60 @@ class TestInterRepository(TestCaseWithInterRepository):
                 if tree.inventory[file_id].kind == "file":
                     tree.get_file(file_id).read()
 
-        # makes a target version repo 
+        # makes a target version repo
         repo_b = self.make_to_repository('b')
         check_push_rev1(repo_b)
 
+    def test_fetch_inconsistent_last_changed_entries(self):
+        """If an inventory has odd data we should still get what it references.
+
+        This test tests that we do fetch a file text created in a revision not
+        being fetched, but referenced from the revision we are fetching when the
+        adjacent revisions to the one being fetched do not reference that text.
+        """
+        tree = self.make_branch_and_tree('source')
+        revid = tree.commit('old')
+        to_repo = self.make_to_repository('to_repo')
+        to_repo.fetch(tree.branch.repository, revid)
+        # Make a broken revision and fetch it.
+        source = tree.branch.repository
+        source.lock_write()
+        self.addCleanup(source.unlock)
+        source.start_write_group()
+        try:
+            # We need two revisions: OLD and NEW. NEW will claim to need a file
+            # 'FOO' changed in 'OLD'. OLD will not have that file at all.
+            source.texts.insert_record_stream([
+                versionedfile.FulltextContentFactory(('foo', revid), (), None,
+                'contents')])
+            basis = source.revision_tree(revid)
+            parent_id = basis.path2id('')
+            entry = inventory.make_entry('file', 'foo-path', parent_id, 'foo')
+            entry.revision = revid
+            entry.text_size = len('contents')
+            entry.text_sha1 = osutils.sha_string('contents')
+            inv_sha1, _ = source.add_inventory_by_delta(revid, [
+                (None, 'foo-path', 'foo', entry)], 'new', [revid])
+            rev = Revision(timestamp=0,
+                           timezone=None,
+                           committer="Foo Bar <foo@example.com>",
+                           message="Message",
+                           inventory_sha1=inv_sha1,
+                           revision_id='new',
+                           parent_ids=[revid])
+            source.add_revision(rev.revision_id, rev)
+        except:
+            source.abort_write_group()
+            raise
+        else:
+            source.commit_write_group()
+        to_repo.fetch(source, 'new')
+        to_repo.lock_read()
+        self.addCleanup(to_repo.unlock)
+        self.assertEqual('contents',
+            to_repo.texts.get_record_stream([('foo', revid)],
+            'unordered', True).next().get_bytes_as('fulltext'))
+
     def test_fetch_missing_basis_text(self):
         """If fetching a delta, we should die if a basis is not present."""
         tree = self.make_branch_and_tree('tree')
@@ -98,9 +153,12 @@ class TestInterRepository(TestCaseWithInterRepository):
         # generally do).
         try:
             to_repo.fetch(tree.branch.repository, 'rev-two')
-        except errors.RevisionNotPresent, e:
+        except (errors.BzrCheckError, errors.RevisionNotPresent), e:
             # If an exception is raised, the revision should not be in the
             # target.
+            #
+            # Can also just raise a generic check errors; stream insertion
+            # does this to include all the missing data
             self.assertRaises((errors.NoSuchRevision, errors.RevisionNotPresent),
                               to_repo.revision_tree, 'rev-two')
         else:
@@ -134,7 +192,7 @@ class TestInterRepository(TestCaseWithInterRepository):
         source_tree = self.make_branch_and_tree('source')
         source = source_tree.branch.repository
         target = self.make_to_repository('target')
-    
+
         # start by adding a file so the data knit for the file exists in
         # repositories that have specific files for each fileid.
         self.build_tree(['source/id'])
