@@ -16,21 +16,23 @@
 
 from cStringIO import StringIO
 import os
+import socket
 import sys
-import tempfile
+import threading
 
 from bzrlib import (
     bzrdir,
     errors,
     inventory,
+    osutils,
     repository,
     revision as _mod_revision,
     treebuilder,
     )
-from bzrlib.bzrdir import BzrDir
 from bzrlib.bundle import read_mergeable_from_url
 from bzrlib.bundle.apply_bundle import install_bundle, merge_bundle
 from bzrlib.bundle.bundle_data import BundleTree
+from bzrlib.bzrdir import BzrDir
 from bzrlib.directory_service import directories
 from bzrlib.bundle.serializer import write_bundle, read_bundle, v09, v4
 from bzrlib.bundle.serializer.v08 import BundleSerializerV08
@@ -38,8 +40,6 @@ from bzrlib.bundle.serializer.v09 import BundleSerializerV09
 from bzrlib.bundle.serializer.v4 import BundleSerializerV4
 from bzrlib.branch import Branch
 from bzrlib.diff import internal_diff
-from bzrlib.errors import (BzrError, TestamentMismatch, NotABundle, BadBundle, 
-                           NoSuchFile,)
 from bzrlib.merge import Merge3Merger
 from bzrlib.repofmt import knitrepo
 from bzrlib.osutils import sha_file, sha_string
@@ -106,7 +106,7 @@ class MockTree(object):
         elif kind == 'symlink':
             ie = InventoryLink(file_id, name, parent_id)
         else:
-            raise BzrError('unknown kind %r' % kind)
+            raise errors.BzrError('unknown kind %r' % kind)
         ie.text_sha1 = text_sha_1
         ie.text_size = text_size
         return ie
@@ -114,7 +114,7 @@ class MockTree(object):
     def add_dir(self, file_id, path):
         self.paths[file_id] = path
         self.ids[path] = file_id
-    
+
     def add_file(self, file_id, path, contents):
         self.add_dir(file_id, path)
         self.contents[file_id] = contents
@@ -151,12 +151,12 @@ class BTreeTester(TestCase):
         mtree.add_file("c", "grandparent/parent/file", "Hello\n")
         mtree.add_dir("d", "grandparent/alt_parent")
         return BundleTree(mtree, ''), mtree
-        
+
     def test_renames(self):
         """Ensure that file renames have the proper effect on children"""
         btree = self.make_tree_1()[0]
         self.assertEqual(btree.old_path("grandparent"), "grandparent")
-        self.assertEqual(btree.old_path("grandparent/parent"), 
+        self.assertEqual(btree.old_path("grandparent/parent"),
                          "grandparent/parent")
         self.assertEqual(btree.old_path("grandparent/parent/file"),
                          "grandparent/parent/file")
@@ -202,7 +202,7 @@ class BTreeTester(TestCase):
         self.assertTrue(btree.path2id("grandparent2/parent") is None)
         self.assertTrue(btree.path2id("grandparent2/parent/file") is None)
 
-        btree.note_rename("grandparent/parent/file", 
+        btree.note_rename("grandparent/parent/file",
                           "grandparent2/parent2/file2")
         self.assertEqual(btree.id2path("a"), "grandparent2")
         self.assertEqual(btree.id2path("b"), "grandparent2/parent2")
@@ -217,7 +217,7 @@ class BTreeTester(TestCase):
     def test_moves(self):
         """Ensure that file moves have the proper effect on children"""
         btree = self.make_tree_1()[0]
-        btree.note_rename("grandparent/parent/file", 
+        btree.note_rename("grandparent/parent/file",
                           "grandparent/alt_parent/file")
         self.assertEqual(btree.id2path("c"), "grandparent/alt_parent/file")
         self.assertEqual(btree.path2id("grandparent/alt_parent/file"), "c")
@@ -231,7 +231,7 @@ class BTreeTester(TestCase):
 
     def make_tree_2(self):
         btree = self.make_tree_1()[0]
-        btree.note_rename("grandparent/parent/file", 
+        btree.note_rename("grandparent/parent/file",
                           "grandparent/alt_parent/file")
         self.assertTrue(btree.id2path("e") is None)
         self.assertTrue(btree.path2id("grandparent/parent/file") is None)
@@ -266,9 +266,9 @@ class BTreeTester(TestCase):
     def make_tree_3(self):
         btree, mtree = self.make_tree_1()
         mtree.add_file("e", "grandparent/parent/topping", "Anchovies\n")
-        btree.note_rename("grandparent/parent/file", 
+        btree.note_rename("grandparent/parent/file",
                           "grandparent/alt_parent/file")
-        btree.note_rename("grandparent/parent/topping", 
+        btree.note_rename("grandparent/parent/topping",
                           "grandparent/alt_parent/stopping")
         return btree
 
@@ -313,7 +313,7 @@ class BTreeTester(TestCase):
             [inventory.ROOT_ID, 'a', 'b', 'c', 'd'])
         btree.note_deletion("grandparent/parent/file")
         btree.note_id("e", "grandparent/alt_parent/fool", kind="directory")
-        btree.note_last_changed("grandparent/alt_parent/fool", 
+        btree.note_last_changed("grandparent/alt_parent/fool",
                                 "revisionidiguess")
         self.assertEqual(self.sorted_ids(btree),
             [inventory.ROOT_ID, 'a', 'b', 'd', 'e'])
@@ -326,7 +326,7 @@ class BundleTester1(TestCaseWithTransport):
         format.repository_format = knitrepo.RepositoryFormatKnit3()
         serializer = BundleSerializerV08('0.8')
         b = self.make_branch('.', format=format)
-        self.assertRaises(errors.IncompatibleBundleFormat, serializer.write, 
+        self.assertRaises(errors.IncompatibleBundleFormat, serializer.write,
                           b.repository, [], {}, StringIO())
 
     def test_matched_bundle(self):
@@ -352,7 +352,7 @@ class BundleTester1(TestCaseWithTransport):
         format = bzrdir.BzrDirMetaFormat1()
         format.repository_format = knitrepo.RepositoryFormatKnit1()
         target = self.make_branch('target', format=format)
-        self.assertRaises(errors.IncompatibleRevision, install_bundle, 
+        self.assertRaises(errors.IncompatibleRevision, install_bundle,
                           target.repository, read_bundle(text))
 
 
@@ -375,10 +375,10 @@ class BundleTester(object):
 
     def create_bundle_text(self, base_rev_id, rev_id):
         bundle_txt = StringIO()
-        rev_ids = write_bundle(self.b1.repository, rev_id, base_rev_id, 
+        rev_ids = write_bundle(self.b1.repository, rev_id, base_rev_id,
                                bundle_txt, format=self.format)
         bundle_txt.seek(0)
-        self.assertEqual(bundle_txt.readline(), 
+        self.assertEqual(bundle_txt.readline(),
                          '# Bazaar revision bundle v%s\n' % self.format)
         self.assertEqual(bundle_txt.readline(), '#\n')
 
@@ -392,12 +392,12 @@ class BundleTester(object):
         """Create a bundle from base_rev_id -> rev_id in built-in branch.
         Make sure that the text generated is valid, and that it
         can be applied against the base, and generate the same information.
-        
-        :return: The in-memory bundle 
+
+        :return: The in-memory bundle
         """
         bundle_txt, rev_ids = self.create_bundle_text(base_rev_id, rev_id)
 
-        # This should also validate the generated bundle 
+        # This should also validate the generated bundle
         bundle = read_bundle(bundle_txt)
         repository = self.b1.repository
         for bundle_rev in bundle.real_revisions:
@@ -407,13 +407,13 @@ class BundleTester(object):
             # it
             branch_rev = repository.get_revision(bundle_rev.revision_id)
             for a in ('inventory_sha1', 'revision_id', 'parent_ids',
-                      'timestamp', 'timezone', 'message', 'committer', 
+                      'timestamp', 'timezone', 'message', 'committer',
                       'parent_ids', 'properties'):
-                self.assertEqual(getattr(branch_rev, a), 
+                self.assertEqual(getattr(branch_rev, a),
                                  getattr(bundle_rev, a))
-            self.assertEqual(len(branch_rev.parent_ids), 
+            self.assertEqual(len(branch_rev.parent_ids),
                              len(bundle_rev.parent_ids))
-        self.assertEqual(rev_ids, 
+        self.assertEqual(rev_ids,
                          [r.revision_id for r in bundle.real_revisions])
         self.valid_apply_bundle(base_rev_id, bundle,
                                    checkout_dir=checkout_dir)
@@ -423,28 +423,29 @@ class BundleTester(object):
     def get_invalid_bundle(self, base_rev_id, rev_id):
         """Create a bundle from base_rev_id -> rev_id in built-in branch.
         Munge the text so that it's invalid.
-        
+
         :return: The in-memory bundle
         """
         bundle_txt, rev_ids = self.create_bundle_text(base_rev_id, rev_id)
-        new_text = bundle_txt.getvalue().replace('executable:no', 
+        new_text = bundle_txt.getvalue().replace('executable:no',
                                                'executable:yes')
         bundle_txt = StringIO(new_text)
         bundle = read_bundle(bundle_txt)
         self.valid_apply_bundle(base_rev_id, bundle)
-        return bundle 
+        return bundle
 
     def test_non_bundle(self):
-        self.assertRaises(NotABundle, read_bundle, StringIO('#!/bin/sh\n'))
+        self.assertRaises(errors.NotABundle,
+                          read_bundle, StringIO('#!/bin/sh\n'))
 
     def test_malformed(self):
-        self.assertRaises(BadBundle, read_bundle, 
+        self.assertRaises(errors.BadBundle, read_bundle,
                           StringIO('# Bazaar revision bundle v'))
 
     def test_crlf_bundle(self):
         try:
             read_bundle(StringIO('# Bazaar revision bundle v0.8\r\n'))
-        except BadBundle:
+        except errors.BadBundle:
             # It is currently permitted for bundles with crlf line endings to
             # make read_bundle raise a BadBundle, but this should be fixed.
             # Anything else, especially NotABundle, is an error.
@@ -455,7 +456,7 @@ class BundleTester(object):
         """
 
         if checkout_dir is None:
-            checkout_dir = tempfile.mkdtemp(prefix='test-branch-', dir='.')
+            checkout_dir = osutils.mkdtemp(prefix='test-branch-', dir='.')
         else:
             if not os.path.exists(checkout_dir):
                 os.mkdir(checkout_dir)
@@ -482,7 +483,7 @@ class BundleTester(object):
                 for inventory_id in old:
                     try:
                         old_file = old.get_file(inventory_id)
-                    except NoSuchFile:
+                    except errors.NoSuchFile:
                         continue
                     if old_file is None:
                         continue
@@ -518,13 +519,13 @@ class BundleTester(object):
         self.assertIs(repository.has_revision(base_rev_id), True)
         for rev in info.real_revisions:
             self.assert_(not repository.has_revision(rev.revision_id),
-                'Revision {%s} present before applying bundle' 
+                'Revision {%s} present before applying bundle'
                 % rev.revision_id)
         merge_bundle(info, to_tree, True, Merge3Merger, False, False)
 
         for rev in info.real_revisions:
             self.assert_(repository.has_revision(rev.revision_id),
-                'Missing revision {%s} after applying bundle' 
+                'Missing revision {%s} after applying bundle'
                 % rev.revision_id)
 
         self.assert_(to_tree.branch.repository.has_revision(info.target))
@@ -536,7 +537,7 @@ class BundleTester(object):
         rev = info.real_revisions[-1]
         base_tree = self.b1.repository.revision_tree(rev.revision_id)
         to_tree = to_tree.branch.repository.revision_tree(rev.revision_id)
-        
+
         # TODO: make sure the target tree is identical to base tree
         #       we might also check the working tree.
 
@@ -602,7 +603,7 @@ class BundleTester(object):
 
         bundle = self.get_valid_bundle('a@cset-0-1', 'a@cset-0-2')
 
-        # Check a rollup bundle 
+        # Check a rollup bundle
         bundle = self.get_valid_bundle('null:', 'a@cset-0-2')
 
         # Now delete entries
@@ -616,12 +617,12 @@ class BundleTester(object):
         tt.set_executability(False, trans_id)
         tt.apply()
         self.tree1.commit('removed', rev_id='a@cset-0-3')
-        
+
         bundle = self.get_valid_bundle('a@cset-0-2', 'a@cset-0-3')
-        self.assertRaises((TestamentMismatch,
+        self.assertRaises((errors.TestamentMismatch,
             errors.VersionedFileInvalidChecksum), self.get_invalid_bundle,
             'a@cset-0-2', 'a@cset-0-3')
-        # Check a rollup bundle 
+        # Check a rollup bundle
         bundle = self.get_valid_bundle('null:', 'a@cset-0-3')
 
         # Now move the directory
@@ -629,7 +630,7 @@ class BundleTester(object):
         self.tree1.commit('rename dir', rev_id='a@cset-0-4')
 
         bundle = self.get_valid_bundle('a@cset-0-3', 'a@cset-0-4')
-        # Check a rollup bundle 
+        # Check a rollup bundle
         bundle = self.get_valid_bundle('null:', 'a@cset-0-4')
 
         # Modified files
@@ -637,7 +638,7 @@ class BundleTester(object):
         open('b1/sub/dir/ pre space', 'ab').write(
              '\r\nAdding some\r\nDOS format lines\r\n')
         open('b1/sub/dir/nolastnewline.txt', 'ab').write('\n')
-        self.tree1.rename_one('sub/dir/ pre space', 
+        self.tree1.rename_one('sub/dir/ pre space',
                               'sub/ start space')
         self.tree1.commit('Modified files', rev_id='a@cset-0-5')
         bundle = self.get_valid_bundle('a@cset-0-4', 'a@cset-0-5')
@@ -695,7 +696,7 @@ class BundleTester(object):
         self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
         tt = TreeTransform(self.tree1)
-        
+
         # Add
         tt.new_file('file', tt.root, '\x00\n\x00\r\x01\n\x02\r\xff', 'binary-1')
         tt.new_file('file2', tt.root, '\x01\n\x02\r\x03\n\x04\r\xff',
@@ -796,7 +797,7 @@ class BundleTester(object):
         # Handle international characters
         os.mkdir('b1')
         try:
-            f = open(u'b1/with Dod\xe9', 'wb')
+            f = open(u'b1/with Dod\N{Euro Sign}', 'wb')
         except UnicodeEncodeError:
             raise TestSkipped("Filesystem doesn't support unicode")
 
@@ -808,44 +809,30 @@ class BundleTester(object):
             u'William Dod\xe9\n').encode('utf-8'))
         f.close()
 
-        self.tree1.add([u'with Dod\xe9'], ['withdod-id'])
+        self.tree1.add([u'with Dod\N{Euro Sign}'], ['withdod-id'])
         self.tree1.commit(u'i18n commit from William Dod\xe9',
                           rev_id='i18n-1', committer=u'William Dod\xe9')
 
-        if sys.platform == 'darwin':
-            from bzrlib.workingtree import WorkingTree3
-            if type(self.tree1) is WorkingTree3:
-                self.knownFailure("Bug #141438: fails for WorkingTree3 on OSX")
-
-            # On Mac the '\xe9' gets changed to 'e\u0301'
-            self.assertEqual([u'.bzr', u'with Dode\u0301'],
-                             sorted(os.listdir(u'b1')))
-            delta = self.tree1.changes_from(self.tree1.basis_tree())
-            self.assertEqual([(u'with Dod\xe9', 'withdod-id', 'file')],
-                             delta.removed)
-            self.knownFailure("Mac OSX doesn't preserve unicode"
-                              " combining characters.")
-
         # Add
         bundle = self.get_valid_bundle('null:', 'i18n-1')
 
         # Modified
-        f = open(u'b1/with Dod\xe9', 'wb')
+        f = open(u'b1/with Dod\N{Euro Sign}', 'wb')
         f.write(u'Modified \xb5\n'.encode('utf8'))
         f.close()
         self.tree1.commit(u'modified', rev_id='i18n-2')
 
         bundle = self.get_valid_bundle('i18n-1', 'i18n-2')
-        
+
         # Renamed
-        self.tree1.rename_one(u'with Dod\xe9', u'B\xe5gfors')
+        self.tree1.rename_one(u'with Dod\N{Euro Sign}', u'B\N{Euro Sign}gfors')
         self.tree1.commit(u'renamed, the new i18n man', rev_id='i18n-3',
                           committer=u'Erik B\xe5gfors')
 
         bundle = self.get_valid_bundle('i18n-2', 'i18n-3')
 
         # Removed
-        self.tree1.remove([u'B\xe5gfors'])
+        self.tree1.remove([u'B\N{Euro Sign}gfors'])
         self.tree1.commit(u'removed', rev_id='i18n-4')
 
         bundle = self.get_valid_bundle('i18n-3', 'i18n-4')
@@ -888,7 +875,7 @@ class BundleTester(object):
         self.tree1.commit('removed', rev_id='white-4')
 
         bundle = self.get_valid_bundle('white-3', 'white-4')
-        
+
         # Now test a complet roll-up
         bundle = self.get_valid_bundle('null:', 'white-4')
 
@@ -907,7 +894,7 @@ class BundleTester(object):
                           timezone=19800, timestamp=1152544886.0)
 
         bundle = self.get_valid_bundle('null:', 'tz-1')
-        
+
         rev = bundle.revisions[0]
         self.assertEqual('Mon 2006-07-10 20:51:26.000000000 +0530', rev.date)
         self.assertEqual(19800, rev.timezone)
@@ -1273,12 +1260,12 @@ class V4BundleTester(BundleTester, TestCaseWithTransport):
         """Create a bundle from base_rev_id -> rev_id in built-in branch.
         Make sure that the text generated is valid, and that it
         can be applied against the base, and generate the same information.
-        
-        :return: The in-memory bundle 
+
+        :return: The in-memory bundle
         """
         bundle_txt, rev_ids = self.create_bundle_text(base_rev_id, rev_id)
 
-        # This should also validate the generated bundle 
+        # This should also validate the generated bundle
         bundle = read_bundle(bundle_txt)
         repository = self.b1.repository
         for bundle_rev in bundle.real_revisions:
@@ -1288,11 +1275,11 @@ class V4BundleTester(BundleTester, TestCaseWithTransport):
             # it
             branch_rev = repository.get_revision(bundle_rev.revision_id)
             for a in ('inventory_sha1', 'revision_id', 'parent_ids',
-                      'timestamp', 'timezone', 'message', 'committer', 
+                      'timestamp', 'timezone', 'message', 'committer',
                       'parent_ids', 'properties'):
-                self.assertEqual(getattr(branch_rev, a), 
+                self.assertEqual(getattr(branch_rev, a),
                                  getattr(bundle_rev, a))
-            self.assertEqual(len(branch_rev.parent_ids), 
+            self.assertEqual(len(branch_rev.parent_ids),
                              len(bundle_rev.parent_ids))
         self.assertEqual(set(rev_ids),
                          set([r.revision_id for r in bundle.real_revisions]))
@@ -1324,10 +1311,10 @@ class V4BundleTester(BundleTester, TestCaseWithTransport):
 
     def create_bundle_text(self, base_rev_id, rev_id):
         bundle_txt = StringIO()
-        rev_ids = write_bundle(self.b1.repository, rev_id, base_rev_id, 
+        rev_ids = write_bundle(self.b1.repository, rev_id, base_rev_id,
                                bundle_txt, format=self.format)
         bundle_txt.seek(0)
-        self.assertEqual(bundle_txt.readline(), 
+        self.assertEqual(bundle_txt.readline(),
                          '# Bazaar revision bundle v%s\n' % self.format)
         self.assertEqual(bundle_txt.readline(), '#\n')
         rev = self.b1.repository.get_revision(rev_id)
@@ -1583,7 +1570,7 @@ class TestBundleWriterReader(TestCase):
         record = record_iter.next()
         self.assertEqual((None, {'foo': 'bar', 'storage_kind': 'header'},
             'info', None, None), record)
-        self.assertRaises(BadBundle, record_iter.next)
+        self.assertRaises(errors.BadBundle, record_iter.next)
 
 
 class TestReadMergeableFromUrl(TestCaseWithTransport):
@@ -1602,3 +1589,51 @@ class TestReadMergeableFromUrl(TestCaseWithTransport):
         self.build_tree_contents([('./foo:bar', out.getvalue())])
         self.assertRaises(errors.NotABundle, read_mergeable_from_url,
                           'foo:bar')
+
+    def test_smart_server_connection_reset(self):
+        """If a smart server connection fails during the attempt to read a
+        bundle, then the ConnectionReset error should be propagated.
+        """
+        # Instantiate a server that will provoke a ConnectionReset
+        sock_server = _DisconnectingTCPServer()
+        sock_server.setUp()
+        self.addCleanup(sock_server.tearDown)
+        # We don't really care what the url is since the server will close the
+        # connection without interpreting it
+        url = sock_server.get_url()
+        self.assertRaises(errors.ConnectionReset, read_mergeable_from_url, url)
+
+
+class _DisconnectingTCPServer(object):
+    """A TCP server that immediately closes any connection made to it."""
+
+    def setUp(self):
+        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.sock.bind(('127.0.0.1', 0))
+        self.sock.listen(1)
+        self.port = self.sock.getsockname()[1]
+        self.thread = threading.Thread(
+            name='%s (port %d)' % (self.__class__.__name__, self.port),
+            target=self.accept_and_close)
+        self.thread.start()
+
+    def accept_and_close(self):
+        conn, addr = self.sock.accept()
+        conn.shutdown(socket.SHUT_RDWR)
+        conn.close()
+
+    def get_url(self):
+        return 'bzr://127.0.0.1:%d/' % (self.port,)
+
+    def tearDown(self):
+        try:
+            # make sure the thread dies by connecting to the listening socket,
+            # just in case the test failed to do so.
+            conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            conn.connect(self.sock.getsockname())
+            conn.close()
+        except socket.error:
+            pass
+        self.sock.close()
+        self.thread.join()
+
