@@ -1,5 +1,5 @@
 # Copyright (C) 2005, 2006, 2007 Canonical Ltd
-# 
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -30,7 +30,9 @@ from bzrlib import (
     errors,
     help_topics,
     repository,
+    osutils,
     symbol_versioning,
+    remote,
     urlutils,
     win32utils,
     workingtree,
@@ -47,16 +49,17 @@ from bzrlib.tests import (
     TestSkipped,
     test_sftp_transport
     )
-from bzrlib.tests.http_server import HttpServer
-from bzrlib.tests.http_utils import (
-    TestCaseWithTwoWebservers,
-    HTTPServerRedirecting,
+from bzrlib.tests import(
+    http_server,
+    http_utils,
     )
 from bzrlib.tests.test_http import TestWithTransport_pycurl
 from bzrlib.transport import get_transport
 from bzrlib.transport.http._urllib import HttpTransport_urllib
 from bzrlib.transport.memory import MemoryServer
-from bzrlib.repofmt import knitrepo, weaverepo
+from bzrlib.transport.nosmart import NoSmartTransportDecorator
+from bzrlib.transport.readonly import ReadonlyTransportDecorator
+from bzrlib.repofmt import knitrepo, weaverepo, pack_repo
 
 
 class TestDefaultFormat(TestCase):
@@ -82,7 +85,7 @@ class TestFormatRegistry(TestCase):
         my_format_registry.register('weave', bzrdir.BzrDirFormat6,
             'Pre-0.8 format.  Slower and does not support checkouts or shared'
             ' repositories', deprecated=True)
-        my_format_registry.register_lazy('lazy', 'bzrlib.bzrdir', 
+        my_format_registry.register_lazy('lazy', 'bzrlib.bzrdir',
             'BzrDirFormat6', 'Format registered lazily', deprecated=True)
         my_format_registry.register_metadir('knit',
             'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
@@ -115,10 +118,10 @@ class TestFormatRegistry(TestCase):
         my_bzrdir = my_format_registry.make_bzrdir('weave')
         self.assertIsInstance(my_bzrdir, bzrdir.BzrDirFormat6)
         my_bzrdir = my_format_registry.make_bzrdir('default')
-        self.assertIsInstance(my_bzrdir.repository_format, 
+        self.assertIsInstance(my_bzrdir.repository_format,
             knitrepo.RepositoryFormatKnit1)
         my_bzrdir = my_format_registry.make_bzrdir('knit')
-        self.assertIsInstance(my_bzrdir.repository_format, 
+        self.assertIsInstance(my_bzrdir.repository_format,
             knitrepo.RepositoryFormatKnit1)
         my_bzrdir = my_format_registry.make_bzrdir('branch6')
         self.assertIsInstance(my_bzrdir.get_branch_format(),
@@ -128,27 +131,30 @@ class TestFormatRegistry(TestCase):
         my_format_registry = self.make_format_registry()
         self.assertEqual('Format registered lazily',
                          my_format_registry.get_help('lazy'))
-        self.assertEqual('Format using knits', 
+        self.assertEqual('Format using knits',
                          my_format_registry.get_help('knit'))
-        self.assertEqual('Format using knits', 
+        self.assertEqual('Format using knits',
                          my_format_registry.get_help('default'))
         self.assertEqual('Pre-0.8 format.  Slower and does not support'
-                         ' checkouts or shared repositories', 
+                         ' checkouts or shared repositories',
                          my_format_registry.get_help('weave'))
-        
+
     def test_help_topic(self):
         topics = help_topics.HelpTopicRegistry()
-        topics.register('formats', self.make_format_registry().help_topic, 
-                        'Directory formats')
-        topic = topics.get_detail('formats')
-        new, rest = topic.split('Experimental formats')
+        registry = self.make_format_registry()
+        topics.register('current-formats', registry.help_topic,
+                        'Current formats')
+        topics.register('other-formats', registry.help_topic,
+                        'Other formats')
+        new = topics.get_detail('current-formats')
+        rest = topics.get_detail('other-formats')
         experimental, deprecated = rest.split('Deprecated formats')
-        self.assertContainsRe(new, 'These formats can be used')
-        self.assertContainsRe(new, 
+        self.assertContainsRe(new, 'bzr help formats')
+        self.assertContainsRe(new,
                 ':knit:\n    \(native\) \(default\) Format using knits\n')
-        self.assertContainsRe(experimental, 
+        self.assertContainsRe(experimental,
                 ':branch6:\n    \(native\) Experimental successor to knit')
-        self.assertContainsRe(deprecated, 
+        self.assertContainsRe(deprecated,
                 ':lazy:\n    \(native\) Format registered lazily\n')
         self.assertNotContainsRe(new, 'hidden')
 
@@ -175,7 +181,7 @@ class TestFormatRegistry(TestCase):
             'Pre-0.8 format.  Slower and does not support checkouts or shared'
             ' repositories', deprecated=True, alias=True)
         self.assertEqual(frozenset(['weavealias']), a_registry.aliases())
-    
+
 
 class SampleBranch(bzrlib.branch.Branch):
     """A dummy branch for guess what, dummy use."""
@@ -184,6 +190,13 @@ class SampleBranch(bzrlib.branch.Branch):
         self.bzrdir = dir
 
 
+class SampleRepository(bzrlib.repository.Repository):
+    """A dummy repo."""
+
+    def __init__(self, dir):
+        self.bzrdir = dir
+
+
 class SampleBzrDir(bzrdir.BzrDir):
     """A sample BzrDir implementation to allow testing static methods."""
 
@@ -193,7 +206,7 @@ class SampleBzrDir(bzrdir.BzrDir):
 
     def open_repository(self):
         """See BzrDir.open_repository."""
-        return "A repository"
+        return SampleRepository(self)
 
     def create_branch(self):
         """See BzrDir.create_branch."""
@@ -207,7 +220,7 @@ class SampleBzrDir(bzrdir.BzrDir):
 class SampleBzrDirFormat(bzrdir.BzrDirFormat):
     """A sample format
 
-    this format is initializable, unsupported to aid in testing the 
+    this format is initializable, unsupported to aid in testing the
     open and open_downlevel routines.
     """
 
@@ -234,7 +247,7 @@ class TestBzrDirFormat(TestCaseWithTransport):
     def test_find_format(self):
         # is the right format object found for a branch?
         # create a branch with a few known format objects.
-        # this is not quite the same as 
+        # this is not quite the same as
         t = get_transport(self.get_url())
         self.build_tree(["foo/", "bar/"], transport=t)
         def check_format(format, url):
@@ -244,7 +257,7 @@ class TestBzrDirFormat(TestCaseWithTransport):
             self.failUnless(isinstance(found_format, format.__class__))
         check_format(bzrdir.BzrDirFormat5(), "foo")
         check_format(bzrdir.BzrDirFormat6(), "bar")
-        
+
     def test_find_format_nothing_there(self):
         self.assertRaises(NotBranchError,
                           bzrdir.BzrDirFormat.find_format,
@@ -294,7 +307,7 @@ class TestBzrDirFormat(TestCaseWithTransport):
                           branch.bzrdir.open_repository)
 
     def test_create_branch_and_repo_under_shared_force_new(self):
-        # creating a branch and repo in a shared repo can be forced to 
+        # creating a branch and repo in a shared repo can be forced to
         # make a new repo
         format = bzrdir.format_registry.make_bzrdir('knit')
         self.make_repository('.', shared=True, format=format)
@@ -305,12 +318,12 @@ class TestBzrDirFormat(TestCaseWithTransport):
 
     def test_create_standalone_working_tree(self):
         format = SampleBzrDirFormat()
-        # note this is deliberately readonly, as this failure should 
+        # note this is deliberately readonly, as this failure should
         # occur before any writes.
         self.assertRaises(errors.NotLocalUrl,
                           bzrdir.BzrDir.create_standalone_workingtree,
                           self.get_readonly_url(), format=format)
-        tree = bzrdir.BzrDir.create_standalone_workingtree('.', 
+        tree = bzrdir.BzrDir.create_standalone_workingtree('.',
                                                            format=format)
         self.assertEqual('A tree', tree)
 
@@ -318,12 +331,12 @@ class TestBzrDirFormat(TestCaseWithTransport):
         # create standalone working tree always makes a repo.
         format = bzrdir.format_registry.make_bzrdir('knit')
         self.make_repository('.', shared=True, format=format)
-        # note this is deliberately readonly, as this failure should 
+        # note this is deliberately readonly, as this failure should
         # occur before any writes.
         self.assertRaises(errors.NotLocalUrl,
                           bzrdir.BzrDir.create_standalone_workingtree,
                           self.get_readonly_url('child'), format=format)
-        tree = bzrdir.BzrDir.create_standalone_workingtree('child', 
+        tree = bzrdir.BzrDir.create_standalone_workingtree('child',
             format=format)
         tree.bzrdir.open_repository()
 
@@ -348,7 +361,7 @@ class TestBzrDirFormat(TestCaseWithTransport):
         self.vfs_transport_factory = MemoryServer
         # outside a repo the default convenience output is a repo+branch_tree
         format = bzrdir.format_registry.make_bzrdir('knit')
-        branch = bzrdir.BzrDir.create_branch_convenience(self.get_url(), 
+        branch = bzrdir.BzrDir.create_branch_convenience(self.get_url(),
                                                          format=format)
         self.assertRaises(errors.NoWorkingTree,
                           branch.bzrdir.open_workingtree)
@@ -364,7 +377,7 @@ class TestBzrDirFormat(TestCaseWithTransport):
         branch.bzrdir.open_workingtree()
         self.assertRaises(errors.NoRepositoryPresent,
                           branch.bzrdir.open_repository)
-            
+
     def test_create_branch_convenience_under_shared_repo_force_no_tree(self):
         # inside a repo the default convenience output is a branch+ follow the
         # repo tree policy but we can override that
@@ -376,14 +389,14 @@ class TestBzrDirFormat(TestCaseWithTransport):
                           branch.bzrdir.open_workingtree)
         self.assertRaises(errors.NoRepositoryPresent,
                           branch.bzrdir.open_repository)
-            
+
     def test_create_branch_convenience_under_shared_repo_no_tree_policy(self):
         # inside a repo the default convenience output is a branch+ follow the
         # repo tree policy
         format = bzrdir.format_registry.make_bzrdir('knit')
         repo = self.make_repository('.', shared=True, format=format)
         repo.set_make_working_trees(False)
-        branch = bzrdir.BzrDir.create_branch_convenience('child', 
+        branch = bzrdir.BzrDir.create_branch_convenience('child',
                                                          format=format)
         self.assertRaises(errors.NoWorkingTree,
                           branch.bzrdir.open_workingtree)
@@ -419,7 +432,7 @@ class TestRepositoryAcquisitionPolicy(TestCaseWithTransport):
         """The default acquisition policy should create a standalone branch."""
         my_bzrdir = self.make_bzrdir('.')
         repo_policy = my_bzrdir.determine_repository_policy()
-        repo = repo_policy.acquire_repository()
+        repo, is_new = repo_policy.acquire_repository()
         self.assertEqual(repo.bzrdir.root_transport.base,
                          my_bzrdir.root_transport.base)
         self.assertFalse(repo.is_shared())
@@ -429,22 +442,20 @@ class TestRepositoryAcquisitionPolicy(TestCaseWithTransport):
         child_bzrdir = self.make_bzrdir('child')
         parent_bzrdir.get_config().set_default_stack_on('http://example.org')
         repo_policy = child_bzrdir.determine_repository_policy()
-        # XXX: Determining stacking from a containing bzrdir has been
-        #      explicitly disabled.
-        self.assertIs(None, repo_policy._stack_on)
+        self.assertEqual('http://example.org', repo_policy._stack_on)
 
     def test_determine_stacking_policy_relative(self):
         parent_bzrdir = self.make_bzrdir('.')
         child_bzrdir = self.make_bzrdir('child')
         parent_bzrdir.get_config().set_default_stack_on('child2')
         repo_policy = child_bzrdir.determine_repository_policy()
-        # XXX: Determining stacking from a containing bzrdir has been
-        #      explicitly disabled.
-        self.assertIs(None, repo_policy._stack_on)
+        self.assertEqual('child2', repo_policy._stack_on)
+        self.assertEqual(parent_bzrdir.root_transport.base,
+                         repo_policy._stack_on_pwd)
 
-    def prepare_default_stacking(self):
+    def prepare_default_stacking(self, child_format='1.6'):
         parent_bzrdir = self.make_bzrdir('.')
-        child_branch = self.make_branch('child', format='development1')
+        child_branch = self.make_branch('child', format=child_format)
         parent_bzrdir.get_config().set_default_stack_on(child_branch.base)
         new_child_transport = parent_bzrdir.transport.clone('child2')
         return child_branch, new_child_transport
@@ -452,42 +463,95 @@ class TestRepositoryAcquisitionPolicy(TestCaseWithTransport):
     def test_clone_on_transport_obeys_stacking_policy(self):
         child_branch, new_child_transport = self.prepare_default_stacking()
         new_child = child_branch.bzrdir.clone_on_transport(new_child_transport)
-        # XXX: Determining stacking from a containing bzrdir has been
-        #      explicitly disabled.
-        self.assertRaises(errors.NotStacked,
-                          new_child.open_branch().get_stacked_on_url)
+        self.assertEqual(child_branch.base,
+                         new_child.open_branch().get_stacked_on_url())
+
+    def test_default_stacking_with_stackable_branch_unstackable_repo(self):
+        # Make stackable source branch with an unstackable repo format.
+        source_bzrdir = self.make_bzrdir('source')
+        pack_repo.RepositoryFormatKnitPack1().initialize(source_bzrdir)
+        source_branch = bzrlib.branch.BzrBranchFormat7().initialize(source_bzrdir)
+        # Make a directory with a default stacking policy
+        parent_bzrdir = self.make_bzrdir('parent')
+        stacked_on = self.make_branch('parent/stacked-on', format='pack-0.92')
+        parent_bzrdir.get_config().set_default_stack_on(stacked_on.base)
+        # Clone source into directory
+        target = source_bzrdir.clone(self.get_url('parent/target'))
 
     def test_sprout_obeys_stacking_policy(self):
         child_branch, new_child_transport = self.prepare_default_stacking()
         new_child = child_branch.bzrdir.sprout(new_child_transport.base)
-        # XXX: Determining stacking from a containing bzrdir has been
-        #      explicitly disabled.
-        self.assertRaises(errors.NotStacked,
+        self.assertEqual(child_branch.base,
+                         new_child.open_branch().get_stacked_on_url())
+
+    def test_clone_ignores_policy_for_unsupported_formats(self):
+        child_branch, new_child_transport = self.prepare_default_stacking(
+            child_format='pack-0.92')
+        new_child = child_branch.bzrdir.clone_on_transport(new_child_transport)
+        self.assertRaises(errors.UnstackableBranchFormat,
                           new_child.open_branch().get_stacked_on_url)
 
+    def test_sprout_ignores_policy_for_unsupported_formats(self):
+        child_branch, new_child_transport = self.prepare_default_stacking(
+            child_format='pack-0.92')
+        new_child = child_branch.bzrdir.sprout(new_child_transport.base)
+        self.assertRaises(errors.UnstackableBranchFormat,
+                          new_child.open_branch().get_stacked_on_url)
+
+    def test_sprout_upgrades_format_if_stacked_specified(self):
+        child_branch, new_child_transport = self.prepare_default_stacking(
+            child_format='pack-0.92')
+        new_child = child_branch.bzrdir.sprout(new_child_transport.base,
+                                               stacked=True)
+        self.assertEqual(child_branch.bzrdir.root_transport.base,
+                         new_child.open_branch().get_stacked_on_url())
+        repo = new_child.open_repository()
+        self.assertTrue(repo._format.supports_external_lookups)
+        self.assertFalse(repo.supports_rich_root())
+
+    def test_clone_on_transport_upgrades_format_if_stacked_on_specified(self):
+        child_branch, new_child_transport = self.prepare_default_stacking(
+            child_format='pack-0.92')
+        new_child = child_branch.bzrdir.clone_on_transport(new_child_transport,
+            stacked_on=child_branch.bzrdir.root_transport.base)
+        self.assertEqual(child_branch.bzrdir.root_transport.base,
+                         new_child.open_branch().get_stacked_on_url())
+        repo = new_child.open_repository()
+        self.assertTrue(repo._format.supports_external_lookups)
+        self.assertFalse(repo.supports_rich_root())
+
+    def test_sprout_upgrades_to_rich_root_format_if_needed(self):
+        child_branch, new_child_transport = self.prepare_default_stacking(
+            child_format='rich-root-pack')
+        new_child = child_branch.bzrdir.sprout(new_child_transport.base,
+                                               stacked=True)
+        repo = new_child.open_repository()
+        self.assertTrue(repo._format.supports_external_lookups)
+        self.assertTrue(repo.supports_rich_root())
+
     def test_add_fallback_repo_handles_absolute_urls(self):
-        stack_on = self.make_branch('stack_on', format='development1')
-        repo = self.make_repository('repo', format='development1')
+        stack_on = self.make_branch('stack_on', format='1.6')
+        repo = self.make_repository('repo', format='1.6')
         policy = bzrdir.UseExistingRepository(repo, stack_on.base)
         policy._add_fallback(repo)
 
     def test_add_fallback_repo_handles_relative_urls(self):
-        stack_on = self.make_branch('stack_on', format='development1')
-        repo = self.make_repository('repo', format='development1')
+        stack_on = self.make_branch('stack_on', format='1.6')
+        repo = self.make_repository('repo', format='1.6')
         policy = bzrdir.UseExistingRepository(repo, '.', stack_on.base)
         policy._add_fallback(repo)
 
     def test_configure_relative_branch_stacking_url(self):
-        stack_on = self.make_branch('stack_on', format='development1')
-        stacked = self.make_branch('stack_on/stacked', format='development1')
+        stack_on = self.make_branch('stack_on', format='1.6')
+        stacked = self.make_branch('stack_on/stacked', format='1.6')
         policy = bzrdir.UseExistingRepository(stacked.repository,
             '.', stack_on.base)
         policy.configure_branch(stacked)
         self.assertEqual('..', stacked.get_stacked_on_url())
 
     def test_relative_branch_stacking_to_absolute(self):
-        stack_on = self.make_branch('stack_on', format='development1')
-        stacked = self.make_branch('stack_on/stacked', format='development1')
+        stack_on = self.make_branch('stack_on', format='1.6')
+        stacked = self.make_branch('stack_on/stacked', format='1.6')
         policy = bzrdir.UseExistingRepository(stacked.repository,
             '.', self.get_readonly_url('stack_on'))
         policy.configure_branch(stacked)
@@ -506,7 +570,7 @@ class ChrootedTests(TestCaseWithTransport):
     def setUp(self):
         super(ChrootedTests, self).setUp()
         if not self.vfs_transport_factory == MemoryServer:
-            self.transport_readonly_server = HttpServer
+            self.transport_readonly_server = http_server.HttpServer
 
     def local_branch_path(self, branch):
          return os.path.realpath(urlutils.local_path_from_url(branch.base))
@@ -537,7 +601,7 @@ class ChrootedTests(TestCaseWithTransport):
         self.assertEqual(os.path.realpath('topdir'),
                          self.local_branch_path(branch))
         self.assertEqual(
-            os.path.realpath(os.path.join('topdir', '.bzr', 'repository')),
+            osutils.realpath(os.path.join('topdir', '.bzr', 'repository')),
             repo.bzrdir.transport.local_abspath('repository'))
         self.assertEqual(relpath, 'foo')
 
@@ -550,7 +614,7 @@ class ChrootedTests(TestCaseWithTransport):
         self.assertEqual(os.path.realpath('branch'),
                          self.local_branch_path(branch))
         self.assertEqual(
-            os.path.realpath(os.path.join('branch', '.bzr', 'repository')),
+            osutils.realpath(os.path.join('branch', '.bzr', 'repository')),
             repo.bzrdir.transport.local_abspath('repository'))
         self.assertEqual(relpath, 'foo')
 
@@ -562,7 +626,7 @@ class ChrootedTests(TestCaseWithTransport):
         self.assertEqual(tree, None)
         self.assertEqual(branch, None)
         self.assertEqual(
-            os.path.realpath(os.path.join('repo', '.bzr', 'repository')),
+            osutils.realpath(os.path.join('repo', '.bzr', 'repository')),
             repo.bzrdir.transport.local_abspath('repository'))
         self.assertEqual(relpath, '')
 
@@ -577,7 +641,7 @@ class ChrootedTests(TestCaseWithTransport):
         self.assertEqual(os.path.realpath('shared/branch'),
                          self.local_branch_path(branch))
         self.assertEqual(
-            os.path.realpath(os.path.join('shared', '.bzr', 'repository')),
+            osutils.realpath(os.path.join('shared', '.bzr', 'repository')),
             repo.bzrdir.transport.local_abspath('repository'))
         self.assertEqual(relpath, '')
 
@@ -592,7 +656,7 @@ class ChrootedTests(TestCaseWithTransport):
         self.assertEqual(os.path.realpath('foo'),
                          self.local_branch_path(branch))
         self.assertEqual(
-            os.path.realpath(os.path.join('foo', '.bzr', 'repository')),
+            osutils.realpath(os.path.join('foo', '.bzr', 'repository')),
             repo.bzrdir.transport.local_abspath('repository'))
         self.assertEqual(relpath, 'bar')
 
@@ -605,7 +669,7 @@ class ChrootedTests(TestCaseWithTransport):
         self.assertEqual(tree, None)
         self.assertEqual(branch, None)
         self.assertEqual(
-            os.path.realpath(os.path.join('bar', '.bzr', 'repository')),
+            osutils.realpath(os.path.join('bar', '.bzr', 'repository')),
             repo.bzrdir.transport.local_abspath('repository'))
         self.assertEqual(relpath, 'baz')
 
@@ -673,7 +737,7 @@ class ChrootedTests(TestCaseWithTransport):
         opened_bzrdir = bzrdir.BzrDir.open_from_transport(transport)
         self.assertEqual(transport.base, opened_bzrdir.root_transport.base)
         self.assertIsInstance(opened_bzrdir, bzrdir.BzrDir)
-        
+
     def test_open_from_transport_no_bzrdir(self):
         transport = get_transport(self.get_url())
         self.assertRaises(NotBranchError, bzrdir.BzrDir.open_from_transport,
@@ -828,22 +892,33 @@ class TestMeta1DirFormat(TestCaseWithTransport):
 
     def test_needs_conversion_different_working_tree(self):
         # meta1dirs need an conversion if any element is not the default.
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        # test with 
-        new_default = bzrdir.format_registry.make_bzrdir('dirstate')
-        bzrdir.BzrDirFormat._set_default_format(new_default)
-        try:
-            tree = self.make_branch_and_tree('tree', format='knit')
-            self.assertTrue(tree.bzrdir.needs_format_conversion())
-        finally:
-            bzrdir.BzrDirFormat._set_default_format(old_format)
+        new_format = bzrdir.format_registry.make_bzrdir('dirstate')
+        tree = self.make_branch_and_tree('tree', format='knit')
+        self.assertTrue(tree.bzrdir.needs_format_conversion(
+            new_format))
+
+    def test_initialize_on_format_uses_smart_transport(self):
+        self.setup_smart_server_with_call_log()
+        new_format = bzrdir.format_registry.make_bzrdir('dirstate')
+        transport = self.get_transport('target')
+        transport.ensure_base()
+        self.reset_smart_call_log()
+        instance = new_format.initialize_on_transport(transport)
+        self.assertIsInstance(instance, remote.RemoteBzrDir)
+        rpc_count = len(self.hpss_calls)
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertEqual(2, rpc_count)
 
 
 class TestFormat5(TestCaseWithTransport):
     """Tests specific to the version 5 bzrdir format."""
 
     def test_same_lockfiles_between_tree_repo_branch(self):
-        # this checks that only a single lockfiles instance is created 
+        # this checks that only a single lockfiles instance is created
         # for format 5 objects
         dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
         def check_dir_components_use_same_lock(dir):
@@ -856,30 +931,28 @@ class TestFormat5(TestCaseWithTransport):
         # and if we open it normally.
         dir = bzrdir.BzrDir.open(self.get_url())
         check_dir_components_use_same_lock(dir)
-    
+
     def test_can_convert(self):
         # format 5 dirs are convertable
         dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
         self.assertTrue(dir.can_convert_format())
-    
+
     def test_needs_conversion(self):
-        # format 5 dirs need a conversion if they are not the default.
-        # and they start of not the default.
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
-        try:
-            dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
-            self.assertFalse(dir.needs_format_conversion())
-        finally:
-            bzrdir.BzrDirFormat._set_default_format(old_format)
-        self.assertTrue(dir.needs_format_conversion())
+        # format 5 dirs need a conversion if they are not the default,
+        # and they aren't
+        dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
+        # don't need to convert it to itself
+        self.assertFalse(dir.needs_format_conversion(bzrdir.BzrDirFormat5()))
+        # do need to convert it to the current default
+        self.assertTrue(dir.needs_format_conversion(
+            bzrdir.BzrDirFormat.get_default_format()))
 
 
 class TestFormat6(TestCaseWithTransport):
     """Tests specific to the version 6 bzrdir format."""
 
     def test_same_lockfiles_between_tree_repo_branch(self):
-        # this checks that only a single lockfiles instance is created 
+        # this checks that only a single lockfiles instance is created
         # for format 6 objects
         dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
         def check_dir_components_use_same_lock(dir):
@@ -892,21 +965,17 @@ class TestFormat6(TestCaseWithTransport):
         # and if we open it normally.
         dir = bzrdir.BzrDir.open(self.get_url())
         check_dir_components_use_same_lock(dir)
-    
+
     def test_can_convert(self):
         # format 6 dirs are convertable
         dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
         self.assertTrue(dir.can_convert_format())
-    
+
     def test_needs_conversion(self):
         # format 6 dirs need an conversion if they are not the default.
-        old_format = bzrdir.BzrDirFormat.get_default_format()
-        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirMetaFormat1())
-        try:
-            dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
-            self.assertTrue(dir.needs_format_conversion())
-        finally:
-            bzrdir.BzrDirFormat._set_default_format(old_format)
+        dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
+        self.assertTrue(dir.needs_format_conversion(
+            bzrdir.BzrDirFormat.get_default_format()))
 
 
 class NotBzrDir(bzrlib.bzrdir.BzrDir):
@@ -943,13 +1012,13 @@ class NotBzrDirFormat(bzrlib.bzrdir.BzrDirFormat):
 
 class TestNotBzrDir(TestCaseWithTransport):
     """Tests for using the bzrdir api with a non .bzr based disk format.
-    
+
     If/when one of these is in the core, we can let the implementation tests
     verify this works.
     """
 
     def test_create_and_find_format(self):
-        # create a .notbzr dir 
+        # create a .notbzr dir
         format = NotBzrDirFormat()
         dir = format.initialize(self.get_url())
         self.assertIsInstance(dir, NotBzrDir)
@@ -980,7 +1049,7 @@ class NonLocalTests(TestCaseWithTransport):
     def setUp(self):
         super(NonLocalTests, self).setUp()
         self.vfs_transport_factory = MemoryServer
-    
+
     def test_create_branch_convenience(self):
         # outside a repo the default convenience output is a repo+branch_tree
         format = bzrdir.format_registry.make_bzrdir('knit')
@@ -1023,73 +1092,96 @@ class NonLocalTests(TestCaseWithTransport):
                               workingtree.WorkingTreeFormat3)
 
 
-class TestHTTPRedirectionLoop(object):
-    """Test redirection loop between two http servers.
+class TestHTTPRedirections(object):
+    """Test redirection between two http servers.
 
     This MUST be used by daughter classes that also inherit from
     TestCaseWithTwoWebservers.
 
     We can't inherit directly from TestCaseWithTwoWebservers or the
     test framework will try to create an instance which cannot
-    run, its implementation being incomplete. 
+    run, its implementation being incomplete.
     """
 
-    # Should be defined by daughter classes to ensure redirection
-    # still use the same transport implementation (not currently
-    # enforced as it's a bit tricky to get right (see the FIXME
-    # in BzrDir.open_from_transport for the unique use case so
-    # far)
-    _qualifier = None
-
     def create_transport_readonly_server(self):
-        return HTTPServerRedirecting()
+        return http_utils.HTTPServerRedirecting()
 
     def create_transport_secondary_server(self):
-        return HTTPServerRedirecting()
+        return http_utils.HTTPServerRedirecting()
 
     def setUp(self):
-        # Both servers redirect to each server creating a loop
-        super(TestHTTPRedirectionLoop, self).setUp()
+        super(TestHTTPRedirections, self).setUp()
         # The redirections will point to the new server
         self.new_server = self.get_readonly_server()
         # The requests to the old server will be redirected
         self.old_server = self.get_secondary_server()
         # Configure the redirections
         self.old_server.redirect_to(self.new_server.host, self.new_server.port)
-        self.new_server.redirect_to(self.old_server.host, self.old_server.port)
-
-    def _qualified_url(self, host, port):
-        return 'http+%s://%s:%s' % (self._qualifier, host, port)
 
     def test_loop(self):
+        # Both servers redirect to each other creating a loop
+        self.new_server.redirect_to(self.old_server.host, self.old_server.port)
         # Starting from either server should loop
-        old_url = self._qualified_url(self.old_server.host, 
+        old_url = self._qualified_url(self.old_server.host,
                                       self.old_server.port)
         oldt = self._transport(old_url)
         self.assertRaises(errors.NotBranchError,
                           bzrdir.BzrDir.open_from_transport, oldt)
-        new_url = self._qualified_url(self.new_server.host, 
+        new_url = self._qualified_url(self.new_server.host,
                                       self.new_server.port)
         newt = self._transport(new_url)
         self.assertRaises(errors.NotBranchError,
                           bzrdir.BzrDir.open_from_transport, newt)
 
+    def test_qualifier_preserved(self):
+        wt = self.make_branch_and_tree('branch')
+        old_url = self._qualified_url(self.old_server.host,
+                                      self.old_server.port)
+        start = self._transport(old_url).clone('branch')
+        bdir = bzrdir.BzrDir.open_from_transport(start)
+        # Redirection should preserve the qualifier, hence the transport class
+        # itself.
+        self.assertIsInstance(bdir.root_transport, type(start))
 
-class TestHTTPRedirections_urllib(TestHTTPRedirectionLoop,
-                                  TestCaseWithTwoWebservers):
+
+class TestHTTPRedirections_urllib(TestHTTPRedirections,
+                                  http_utils.TestCaseWithTwoWebservers):
     """Tests redirections for urllib implementation"""
 
-    _qualifier = 'urllib'
     _transport = HttpTransport_urllib
 
+    def _qualified_url(self, host, port):
+        return 'http+urllib://%s:%s' % (host, port)
+
 
 
 class TestHTTPRedirections_pycurl(TestWithTransport_pycurl,
-                                  TestHTTPRedirectionLoop,
-                                  TestCaseWithTwoWebservers):
+                                  TestHTTPRedirections,
+                                  http_utils.TestCaseWithTwoWebservers):
     """Tests redirections for pycurl implementation"""
 
-    _qualifier = 'pycurl'
+    def _qualified_url(self, host, port):
+        return 'http+pycurl://%s:%s' % (host, port)
+
+
+class TestHTTPRedirections_nosmart(TestHTTPRedirections,
+                                  http_utils.TestCaseWithTwoWebservers):
+    """Tests redirections for the nosmart decorator"""
+
+    _transport = NoSmartTransportDecorator
+
+    def _qualified_url(self, host, port):
+        return 'nosmart+http://%s:%s' % (host, port)
+
+
+class TestHTTPRedirections_readonly(TestHTTPRedirections,
+                                    http_utils.TestCaseWithTwoWebservers):
+    """Tests redirections for readonly decoratror"""
+
+    _transport = ReadonlyTransportDecorator
+
+    def _qualified_url(self, host, port):
+        return 'readonly+http://%s:%s' % (host, port)
 
 
 class TestDotBzrHidden(TestCaseWithTransport):
@@ -1130,7 +1222,7 @@ class _TestBzrDirFormat(bzrdir.BzrDirMetaFormat1):
 
 class _TestBzrDir(bzrdir.BzrDirMeta1):
     """Test BzrDir implementation for TestBzrDirSprout.
-    
+
     When created a _TestBzrDir already has repository and a branch.  The branch
     is a test double as well.
     """
@@ -1143,19 +1235,35 @@ class _TestBzrDir(bzrdir.BzrDirMeta1):
     def open_branch(self, unsupported=False):
         return self.test_branch
 
-    def cloning_metadir(self):
+    def cloning_metadir(self, require_stacking=False):
         return _TestBzrDirFormat()
 
 
+class _TestBranchFormat(bzrlib.branch.BranchFormat):
+    """Test Branch format for TestBzrDirSprout."""
+
+
 class _TestBranch(bzrlib.branch.Branch):
     """Test Branch implementation for TestBzrDirSprout."""
 
     def __init__(self, *args, **kwargs):
+        self._format = _TestBranchFormat()
         super(_TestBranch, self).__init__(*args, **kwargs)
         self.calls = []
-    
+        self._parent = None
+
     def sprout(self, *args, **kwargs):
         self.calls.append('sprout')
+        return _TestBranch()
+
+    def copy_content_into(self, destination, revision_id=None):
+        self.calls.append('copy_content_into')
+
+    def get_parent(self):
+        return self._parent
+
+    def set_parent(self, parent):
+        self._parent = parent
 
 
 class TestBzrDirSprout(TestCaseWithMemoryTransport):
@@ -1166,7 +1274,7 @@ class TestBzrDirSprout(TestCaseWithMemoryTransport):
         Usually, BzrDir.sprout should delegate to the branch's sprout method
         for part of the work.  This allows the source branch to control the
         choice of format for the new branch.
-        
+
         There are exceptions, but this tests avoids them:
           - if there's no branch in the source bzrdir,
           - or if the stacking has been requested and the format needs to be
@@ -1186,5 +1294,10 @@ class TestBzrDirSprout(TestCaseWithMemoryTransport):
         result = source_bzrdir.sprout(target_url, recurse='no')
 
         # The bzrdir called the branch's sprout method.
-        self.assertEqual(['sprout'], source_bzrdir.test_branch.calls)
-        
+        self.assertSubset(['sprout'], source_bzrdir.test_branch.calls)
+
+    def test_sprout_parent(self):
+        grandparent_tree = self.make_branch('grandparent')
+        parent = grandparent_tree.bzrdir.sprout('parent').open_branch()
+        branch_tree = parent.bzrdir.sprout('branch').open_branch()
+        self.assertContainsRe(branch_tree.get_parent(), '/parent/$')
