@@ -20,6 +20,7 @@ from bzrlib import errors
 from bzrlib.directory_service import DirectoryServiceRegistry, directories
 from bzrlib.tests import TestCase, TestCaseWithTransport
 from bzrlib.transport import get_transport
+from bzrlib import urlutils
 
 
 class FooService(object):
@@ -83,6 +84,11 @@ class TestAliasDirectory(TestCaseWithTransport):
         branch = self.make_branch('.')
         self.assertEqual(branch.base, directories.dereference(':this'))
 
+    def test_extra_path(self):
+        branch = self.make_branch('.')
+        self.assertEqual(urlutils.join(branch.base, 'arg'),
+                         directories.dereference(':this/arg'))
+
     def test_lookup_badname(self):
         branch = self.make_branch('.')
         e = self.assertRaises(errors.InvalidLocationAlias,
