@@ -45,6 +45,13 @@ class TestErrors(TestCaseWithTransport):
             "Error: the reason why",
             str(error))
 
+    def test_dirstate_corrupt(self):
+        error = errors.DirstateCorrupt('.bzr/checkout/dirstate',
+                                       'trailing garbage: "x"')
+        self.assertEqualDiff("The dirstate file (.bzr/checkout/dirstate)"
+            " appears to be corrupt: trailing garbage: \"x\"",
+            str(error))
+
     def test_disabled_method(self):
         error = errors.DisabledMethod("class name")
         self.assertEqualDiff(
@@ -151,7 +158,7 @@ class TestErrors(TestCaseWithTransport):
         error = errors.MediumNotConnected("a medium")
         self.assertEqualDiff(
             "The medium 'a medium' is not connected.", str(error))
- 
+
     def test_no_public_branch(self):
         b = self.make_branch('.')
         error = errors.NoPublicBranch(b)
@@ -164,7 +171,7 @@ class TestErrors(TestCaseWithTransport):
         error = errors.NoRepositoryPresent(dir)
         self.assertNotEqual(-1, str(error).find((dir.transport.clone('..').base)))
         self.assertEqual(-1, str(error).find((dir.transport.base)))
-        
+
     def test_no_smart_medium(self):
         error = errors.NoSmartMedium("a transport")
         self.assertEqualDiff("The transport 'a transport' cannot tunnel the "
@@ -249,8 +256,8 @@ class TestErrors(TestCaseWithTransport):
 
     def test_up_to_date(self):
         error = errors.UpToDateFormat(bzrdir.BzrDirFormat4())
-        self.assertEqualDiff("The branch format Bazaar-NG branch, "
-                             "format 0.0.4 is already at the most "
+        self.assertEqualDiff("The branch format All-in-one "
+                             "format 4 is already at the most "
                              "recent format.",
                              str(error))
 
@@ -454,7 +461,7 @@ class TestErrors(TestCaseWithTransport):
         self.assertEqual(
             "Container has multiple records with the same name: n\xc3\xa5me",
             str(e))
-        
+
     def test_check_error(self):
         # This has a member called 'message', which is problematic in
         # python2.5 because that is a slot on the base Exception class
@@ -541,9 +548,75 @@ class TestErrors(TestCaseWithTransport):
             'Tip change rejected: Unicode message\xe2\x80\xbd',
             str(err))
 
+    def test_error_from_smart_server(self):
+        error_tuple = ('error', 'tuple')
+        err = errors.ErrorFromSmartServer(error_tuple)
+        self.assertEquals(
+            "Error received from smart server: ('error', 'tuple')", str(err))
+
+    def test_untranslateable_error_from_smart_server(self):
+        error_tuple = ('error', 'tuple')
+        orig_err = errors.ErrorFromSmartServer(error_tuple)
+        err = errors.UnknownErrorFromSmartServer(orig_err)
+        self.assertEquals(
+            "Server sent an unexpected error: ('error', 'tuple')", str(err))
+
+    def test_smart_message_handler_error(self):
+        # Make an exc_info tuple.
+        try:
+            raise Exception("example error")
+        except Exception:
+            exc_info = sys.exc_info()
+        err = errors.SmartMessageHandlerError(exc_info)
+        self.assertStartsWith(
+            str(err), "The message handler raised an exception:\n")
+        self.assertEndsWith(str(err), "Exception: example error\n")
+
+    def test_must_have_working_tree(self):
+        err = errors.MustHaveWorkingTree('foo', 'bar')
+        self.assertEqual(str(err), "Branching 'bar'(foo) must create a"
+                                   " working tree.")
+
+    def test_no_such_view(self):
+        err = errors.NoSuchView('foo')
+        self.assertEquals("No such view: foo.", str(err))
+
+    def test_views_not_supported(self):
+        err = errors.ViewsNotSupported('atree')
+        err_str = str(err)
+        self.assertStartsWith(err_str, "Views are not supported by ")
+        self.assertEndsWith(err_str, "; use 'bzr upgrade' to change your "
+            "tree to a later format.")
+
+    def test_file_outside_view(self):
+        err = errors.FileOutsideView('baz', ['foo', 'bar'])
+        self.assertEquals('Specified file "baz" is outside the current view: '
+            'foo, bar', str(err))
+
+    def test_invalid_shelf_id(self):
+        invalid_id = "foo"
+        err = errors.InvalidShelfId(invalid_id)
+        self.assertEqual('"foo" is not a valid shelf id, '
+            'try a number instead.', str(err))
+
+    def test_unresumable_write_group(self):
+        repo = "dummy repo"
+        wg_tokens = ['token']
+        reason = "a reason"
+        err = errors.UnresumableWriteGroup(repo, wg_tokens, reason)
+        self.assertEqual(
+            "Repository dummy repo cannot resume write group "
+            "['token']: a reason", str(err))
+
+    def test_unsuspendable_write_group(self):
+        repo = "dummy repo"
+        err = errors.UnsuspendableWriteGroup(repo)
+        self.assertEqual(
+            'Repository dummy repo cannot suspend a write group.', str(err))
+
 
 class PassThroughError(errors.BzrError):
-    
+
     _fmt = """Pass through %(foo)s and %(bar)s"""
 
     def __init__(self, foo, bar):
@@ -560,7 +633,7 @@ class ErrorWithNoFormat(errors.BzrError):
 
 
 class TestErrorFormatting(TestCase):
-    
+
     def test_always_str(self):
         e = PassThroughError(u'\xb5', 'bar')
         self.assertIsInstance(e.__str__(), str)
@@ -577,7 +650,7 @@ class TestErrorFormatting(TestCase):
                 ['ErrorWithNoFormat uses its docstring as a format, it should use _fmt instead'],
                 lambda x: str(x), e)
         ## s = str(e)
-        self.assertEqual(s, 
+        self.assertEqual(s,
                 "This class has a docstring but no format string.")
 
     def test_mismatched_format_args(self):
