@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2008, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,6 +44,9 @@ from bzrlib import (
     ui,
     urlutils,
     )
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    )
 from bzrlib.tests import (
     http_server,
     http_utils,
@@ -65,99 +68,82 @@ except errors.DependencyNotPresent:
     pycurl_present = False
 
 
-class TransportAdapter(tests.TestScenarioApplier):
-    """Generate the same test for each transport implementation."""
+def load_tests(standard_tests, module, loader):
+    """Multiply tests for http clients and protocol versions."""
+    result = loader.suiteClass()
 
-    def __init__(self):
-        transport_scenarios = [
-            ('urllib', dict(_transport=_urllib.HttpTransport_urllib,
-                            _server=http_server.HttpServer_urllib,
-                            _qualified_prefix='http+urllib',)),
-            ]
-        if pycurl_present:
-            transport_scenarios.append(
-                ('pycurl', dict(_transport=PyCurlTransport,
-                                _server=http_server.HttpServer_PyCurl,
-                                _qualified_prefix='http+pycurl',)))
-        self.scenarios = transport_scenarios
+    # one for each transport implementation
+    t_tests, remaining_tests = tests.split_suite_by_condition(
+        standard_tests, tests.condition_isinstance((
+                TestHttpTransportRegistration,
+                TestHttpTransportUrls,
+                Test_redirected_to,
+                )))
+    transport_scenarios = [
+        ('urllib', dict(_transport=_urllib.HttpTransport_urllib,
+                        _server=http_server.HttpServer_urllib,
+                        _qualified_prefix='http+urllib',)),
+        ]
+    if pycurl_present:
+        transport_scenarios.append(
+            ('pycurl', dict(_transport=PyCurlTransport,
+                            _server=http_server.HttpServer_PyCurl,
+                            _qualified_prefix='http+pycurl',)))
+    tests.multiply_tests(t_tests, transport_scenarios, result)
 
-
-class TransportProtocolAdapter(TransportAdapter):
-    """Generate the same test for each protocol implementation.
-
-    In addition to the transport adaptatation that we inherit from.
-    """
-
-    def __init__(self):
-        super(TransportProtocolAdapter, self).__init__()
-        protocol_scenarios = [
+    # each implementation tested with each HTTP version
+    tp_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance((
+                SmartHTTPTunnellingTest,
+                TestDoCatchRedirections,
+                TestHTTPConnections,
+                TestHTTPRedirections,
+                TestHTTPSilentRedirections,
+                TestLimitedRangeRequestServer,
+                TestPost,
+                TestProxyHttpServer,
+                TestRanges,
+                TestSpecificRequestHandler,
+                )))
+    protocol_scenarios = [
             ('HTTP/1.0',  dict(_protocol_version='HTTP/1.0')),
             ('HTTP/1.1',  dict(_protocol_version='HTTP/1.1')),
             ]
-        self.scenarios = tests.multiply_scenarios(self.scenarios,
-                                                  protocol_scenarios)
+    tp_scenarios = tests.multiply_scenarios(transport_scenarios,
+                                            protocol_scenarios)
+    tests.multiply_tests(tp_tests, tp_scenarios, result)
 
+    # auth: each auth scheme on all http versions on all implementations.
+    tpa_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance((
+                TestAuth,
+                )))
+    auth_scheme_scenarios = [
+        ('basic', dict(_auth_scheme='basic')),
+        ('digest', dict(_auth_scheme='digest')),
+        ]
+    tpa_scenarios = tests.multiply_scenarios(tp_scenarios,
+        auth_scheme_scenarios)
+    tests.multiply_tests(tpa_tests, tpa_scenarios, result)
 
-class TransportProtocolAuthenticationAdapter(TransportProtocolAdapter):
-    """Generate the same test for each authentication scheme implementation.
+    # activity: activity on all http versions on all implementations
+    tpact_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance((
+                TestActivity,
+                )))
+    activity_scenarios = [
+        ('http', dict(_activity_server=ActivityHTTPServer)),
+        ]
+    if tests.HTTPSServerFeature.available():
+        activity_scenarios.append(
+            ('https', dict(_activity_server=ActivityHTTPSServer)))
+    tpact_scenarios = tests.multiply_scenarios(tp_scenarios,
+        activity_scenarios)
+    tests.multiply_tests(tpact_tests, tpact_scenarios, result)
 
-    In addition to the protocol adaptatation that we inherit from.
-    """
+    # No parametrization for the remaining tests
+    result.addTests(remaining_tests)
 
-    def __init__(self):
-        super(TransportProtocolAuthenticationAdapter, self).__init__()
-        auth_scheme_scenarios = [
-            ('basic', dict(_auth_scheme='basic')),
-            ('digest', dict(_auth_scheme='digest')),
-            ]
-
-        self.scenarios = tests.multiply_scenarios(self.scenarios,
-                                                  auth_scheme_scenarios)
-
-def load_tests(standard_tests, module, loader):
-    """Multiply tests for http clients and protocol versions."""
-    # one for each transport
-    t_adapter = TransportAdapter()
-    t_classes= (TestHttpTransportRegistration,
-                TestHttpTransportUrls,
-                )
-    is_testing_for_transports = tests.condition_isinstance(t_classes)
-
-    # multiplied by one for each protocol version
-    tp_adapter = TransportProtocolAdapter()
-    tp_classes= (SmartHTTPTunnellingTest,
-                 TestDoCatchRedirections,
-                 TestHTTPConnections,
-                 TestHTTPRedirections,
-                 TestHTTPSilentRedirections,
-                 TestLimitedRangeRequestServer,
-                 TestPost,
-                 TestProxyHttpServer,
-                 TestRanges,
-                 TestSpecificRequestHandler,
-                 )
-    is_also_testing_for_protocols = tests.condition_isinstance(tp_classes)
-
-    # multiplied by one for each authentication scheme
-    tpa_adapter = TransportProtocolAuthenticationAdapter()
-    tpa_classes = (TestAuth,
-                   )
-    is_also_testing_for_authentication = tests.condition_isinstance(
-        tpa_classes)
-
-    result = loader.suiteClass()
-    for test_class in tests.iter_suite_tests(standard_tests):
-        # Each test class is either standalone or testing for some combination
-        # of transport, protocol version, authentication scheme. Use the right
-        # adpater (or none) depending on the class.
-        if is_testing_for_transports(test_class):
-            result.addTests(t_adapter.adapt(test_class))
-        elif is_also_testing_for_protocols(test_class):
-            result.addTests(tp_adapter.adapt(test_class))
-        elif is_also_testing_for_authentication(test_class):
-            result.addTests(tpa_adapter.adapt(test_class))
-        else:
-            result.addTest(test_class)
     return result
 
 
@@ -172,7 +158,7 @@ class FakeManager(object):
 
 class RecordingServer(object):
     """A fake HTTP server.
-    
+
     It records the bytes sent to it, and replies with a 200.
     """
 
@@ -227,6 +213,35 @@ class RecordingServer(object):
         self.port = None
 
 
+class TestAuthHeader(tests.TestCase):
+
+    def parse_header(self, header):
+        ah =  _urllib2_wrappers.AbstractAuthHandler()
+        return ah._parse_auth_header(header)
+
+    def test_empty_header(self):
+        scheme, remainder = self.parse_header('')
+        self.assertEquals('', scheme)
+        self.assertIs(None, remainder)
+
+    def test_negotiate_header(self):
+        scheme, remainder = self.parse_header('Negotiate')
+        self.assertEquals('negotiate', scheme)
+        self.assertIs(None, remainder)
+
+    def test_basic_header(self):
+        scheme, remainder = self.parse_header(
+            'Basic realm="Thou should not pass"')
+        self.assertEquals('basic', scheme)
+        self.assertEquals('realm="Thou should not pass"', remainder)
+
+    def test_digest_header(self):
+        scheme, remainder = self.parse_header(
+            'Digest realm="Thou should not pass"')
+        self.assertEquals('digest', scheme)
+        self.assertEquals('realm="Thou should not pass"', remainder)
+
+
 class TestHTTPServer(tests.TestCase):
     """Test the HTTP servers implementations."""
 
@@ -449,13 +464,6 @@ class TestHTTPConnections(http_utils.TestCaseWithWebserver):
             '"GET /foo/bar HTTP/1.1" 200 - "-" "bzr/%s'
             % bzrlib.__version__) > -1)
 
-    def test_get_smart_medium(self):
-        # For HTTP, get_smart_medium should return the transport object.
-        server = self.get_readonly_server()
-        http_transport = self._transport(server.get_url())
-        medium = http_transport.get_smart_medium()
-        self.assertIs(medium, http_transport)
-
     def test_has_on_bogus_host(self):
         # Get a free address and don't 'accept' on it, so that we
         # can be sure there is no http handler there, but set a
@@ -818,7 +826,7 @@ class TestRangeRequestServer(TestSpecificRequestHandler):
         # bytes on the socket
         ireadv = iter(t.readv('a', ((0, 1), (1, 1), (2, 4), (6, 4))))
         self.assertEqual((0, '0'), ireadv.next())
-        # The server should have issued one request so far 
+        # The server should have issued one request so far
         self.assertEqual(1, server.GET_request_nb)
         self.assertEqual('0123456789', t.get_bytes('a'))
         # get_bytes issued an additional request, the readv pending ones are
@@ -1145,7 +1153,7 @@ class TestProxyHttpServer(http_utils.TestCaseWithTwoWebservers):
         url = self.server.get_url()
         t = self._transport(url)
         try:
-            self.assertEqual(t.get('foo').read(), 'proxied contents of foo\n')
+            self.assertEqual('proxied contents of foo\n', t.get('foo').read())
         finally:
             self._restore_env()
 
@@ -1154,7 +1162,7 @@ class TestProxyHttpServer(http_utils.TestCaseWithTwoWebservers):
         url = self.server.get_url()
         t = self._transport(url)
         try:
-            self.assertEqual(t.get('foo').read(), 'contents of foo\n')
+            self.assertEqual('contents of foo\n', t.get('foo').read())
         finally:
             self._restore_env()
 
@@ -1270,7 +1278,7 @@ class TestHTTPRedirections(http_utils.TestCaseWithRedirectedWebserver):
                                   ('bundle',
                                   '# Bazaar revision bundle v0.9\n#\n')
                                   ],)
-
+        # The requests to the old server will be redirected to the new server
         self.old_transport = self._transport(self.old_server.get_url())
 
     def test_redirected(self):
@@ -1281,7 +1289,8 @@ class TestHTTPRedirections(http_utils.TestCaseWithRedirectedWebserver):
     def test_read_redirected_bundle_from_url(self):
         from bzrlib.bundle import read_bundle_from_url
         url = self.old_transport.abspath('bundle')
-        bundle = read_bundle_from_url(url)
+        bundle = self.applyDeprecated(deprecated_in((1, 12, 0)),
+                read_bundle_from_url, url)
         # If read_bundle_from_url was successful we get an empty bundle
         self.assertEqual([], bundle.revisions)
 
@@ -1453,7 +1462,7 @@ class TestAuth(http_utils.TestCaseWithWebserver):
     def _testing_pycurl(self):
         return pycurl_present and self._transport == PyCurlTransport
 
-    def get_user_url(self, user=None, password=None):
+    def get_user_url(self, user, password):
         """Build an url embedding user and password"""
         url = '%s://' % self.server._url_protocol
         if user is not None:
@@ -1464,12 +1473,12 @@ class TestAuth(http_utils.TestCaseWithWebserver):
         url += '%s:%s/' % (self.server.host, self.server.port)
         return url
 
-    def get_user_transport(self, user=None, password=None):
+    def get_user_transport(self, user, password):
         return self._transport(self.get_user_url(user, password))
 
     def test_no_user(self):
         self.server.add_user('joe', 'foo')
-        t = self.get_user_transport()
+        t = self.get_user_transport(None, None)
         self.assertRaises(errors.InvalidHttpResponse, t.get, 'a')
         # Only one 'Authentication Required' error should occur
         self.assertEqual(1, self.server.auth_required_errors)
@@ -1563,6 +1572,25 @@ class TestAuth(http_utils.TestCaseWithWebserver):
         # Only one 'Authentication Required' error should occur
         self.assertEqual(1, self.server.auth_required_errors)
 
+    def test_user_from_auth_conf(self):
+        if self._testing_pycurl():
+            raise tests.TestNotApplicable(
+                'pycurl does not support authentication.conf')
+        user = 'joe'
+        password = 'foo'
+        self.server.add_user(user, password)
+        # Create a minimal config file with the right password
+        conf = config.AuthenticationConfig()
+        conf._get_config().update(
+            {'httptest': {'scheme': 'http', 'port': self.server.port,
+                          'user': user, 'password': password}})
+        conf._save()
+        t = self.get_user_transport(None, None)
+        # Issue a request to the server to connect
+        self.assertEqual('contents of a\n', t.get('a').read())
+        # Only one 'Authentication Required' error should occur
+        self.assertEqual(1, self.server.auth_required_errors)
+
     def test_changing_nonce(self):
         if self._auth_scheme != 'digest':
             raise tests.TestNotApplicable('HTTP auth digest only test')
@@ -1614,7 +1642,7 @@ class TestProxyAuth(TestAuth):
                 protocol_version=self._protocol_version)
         return server
 
-    def get_user_transport(self, user=None, password=None):
+    def get_user_transport(self, user, password):
         self._install_env({'all_proxy': self.get_user_url(user, password)})
         return self._transport(self.server.get_url())
 
@@ -1747,5 +1775,258 @@ class SmartClientAgainstNotSmartServer(TestSpecificRequestHandler):
         # No need to build a valid smart request here, the server will not even
         # try to interpret it.
         self.assertRaises(errors.SmartProtocolError,
-                          t.send_http_smart_request, 'whatever')
+                          t.get_smart_medium().send_http_smart_request,
+                          'whatever')
 
+class Test_redirected_to(tests.TestCase):
+
+    def test_redirected_to_subdir(self):
+        t = self._transport('http://www.example.com/foo')
+        r = t._redirected_to('http://www.example.com/foo',
+                             'http://www.example.com/foo/subdir')
+        self.assertIsInstance(r, type(t))
+        # Both transports share the some connection
+        self.assertEquals(t._get_connection(), r._get_connection())
+
+    def test_redirected_to_self_with_slash(self):
+        t = self._transport('http://www.example.com/foo')
+        r = t._redirected_to('http://www.example.com/foo',
+                             'http://www.example.com/foo/')
+        self.assertIsInstance(r, type(t))
+        # Both transports share the some connection (one can argue that we
+        # should return the exact same transport here, but that seems
+        # overkill).
+        self.assertEquals(t._get_connection(), r._get_connection())
+
+    def test_redirected_to_host(self):
+        t = self._transport('http://www.example.com/foo')
+        r = t._redirected_to('http://www.example.com/foo',
+                             'http://foo.example.com/foo/subdir')
+        self.assertIsInstance(r, type(t))
+
+    def test_redirected_to_same_host_sibling_protocol(self):
+        t = self._transport('http://www.example.com/foo')
+        r = t._redirected_to('http://www.example.com/foo',
+                             'https://www.example.com/foo')
+        self.assertIsInstance(r, type(t))
+
+    def test_redirected_to_same_host_different_protocol(self):
+        t = self._transport('http://www.example.com/foo')
+        r = t._redirected_to('http://www.example.com/foo',
+                             'ftp://www.example.com/foo')
+        self.assertNotEquals(type(r), type(t))
+
+    def test_redirected_to_different_host_same_user(self):
+        t = self._transport('http://joe@www.example.com/foo')
+        r = t._redirected_to('http://www.example.com/foo',
+                             'https://foo.example.com/foo')
+        self.assertIsInstance(r, type(t))
+        self.assertEquals(t._user, r._user)
+
+
+class PredefinedRequestHandler(http_server.TestingHTTPRequestHandler):
+    """Request handler for a unique and pre-defined request.
+
+    The only thing we care about here is how many bytes travel on the wire. But
+    since we want to measure it for a real http client, we have to send it
+    correct responses.
+
+    We expect to receive a *single* request nothing more (and we won't even
+    check what request it is, we just measure the bytes read until an empty
+    line.
+    """
+
+    def handle_one_request(self):
+        tcs = self.server.test_case_server
+        requestline = self.rfile.readline()
+        headers = self.MessageClass(self.rfile, 0)
+        # We just read: the request, the headers, an empty line indicating the
+        # end of the headers.
+        bytes_read = len(requestline)
+        for line in headers.headers:
+            bytes_read += len(line)
+        bytes_read += len('\r\n')
+        if requestline.startswith('POST'):
+            # The body should be a single line (or we don't know where it ends
+            # and we don't want to issue a blocking read)
+            body = self.rfile.readline()
+            bytes_read += len(body)
+        tcs.bytes_read = bytes_read
+
+        # We set the bytes written *before* issuing the write, the client is
+        # supposed to consume every produced byte *before* checking that value.
+
+        # Doing the oppposite may lead to test failure: we may be interrupted
+        # after the write but before updating the value. The client can then
+        # continue and read the value *before* we can update it. And yes,
+        # this has been observed -- vila 20090129
+        tcs.bytes_written = len(tcs.canned_response)
+        self.wfile.write(tcs.canned_response)
+
+
+class ActivityServerMixin(object):
+
+    def __init__(self, protocol_version):
+        super(ActivityServerMixin, self).__init__(
+            request_handler=PredefinedRequestHandler,
+            protocol_version=protocol_version)
+        # Bytes read and written by the server
+        self.bytes_read = 0
+        self.bytes_written = 0
+        self.canned_response = None
+
+
+class ActivityHTTPServer(ActivityServerMixin, http_server.HttpServer):
+    pass
+
+
+if tests.HTTPSServerFeature.available():
+    from bzrlib.tests import https_server
+    class ActivityHTTPSServer(ActivityServerMixin, https_server.HTTPSServer):
+        pass
+
+
+class TestActivity(tests.TestCase):
+    """Test socket activity reporting.
+
+    We use a special purpose server to control the bytes sent and received and
+    be able to predict the activity on the client socket.
+    """
+
+    def setUp(self):
+        tests.TestCase.setUp(self)
+        self.server = self._activity_server(self._protocol_version)
+        self.server.setUp()
+        self.activities = {}
+        def report_activity(t, bytes, direction):
+            count = self.activities.get(direction, 0)
+            count += bytes
+            self.activities[direction] = count
+
+        # We override at class level because constructors may propagate the
+        # bound method and render instance overriding ineffective (an
+        # alternative would be be to define a specific ui factory instead...)
+        self.orig_report_activity = self._transport._report_activity
+        self._transport._report_activity = report_activity
+
+    def tearDown(self):
+        self._transport._report_activity = self.orig_report_activity
+        self.server.tearDown()
+        tests.TestCase.tearDown(self)
+
+    def get_transport(self):
+        return self._transport(self.server.get_url())
+
+    def assertActivitiesMatch(self):
+        self.assertEqual(self.server.bytes_read,
+                         self.activities.get('write', 0), 'written bytes')
+        self.assertEqual(self.server.bytes_written,
+                         self.activities.get('read', 0), 'read bytes')
+
+    def test_get(self):
+        self.server.canned_response = '''HTTP/1.1 200 OK\r
+Date: Tue, 11 Jul 2006 04:32:56 GMT\r
+Server: Apache/2.0.54 (Fedora)\r
+Last-Modified: Sun, 23 Apr 2006 19:35:20 GMT\r
+ETag: "56691-23-38e9ae00"\r
+Accept-Ranges: bytes\r
+Content-Length: 35\r
+Connection: close\r
+Content-Type: text/plain; charset=UTF-8\r
+\r
+Bazaar-NG meta directory, format 1
+'''
+        t = self.get_transport()
+        self.assertEqual('Bazaar-NG meta directory, format 1\n',
+                         t.get('foo/bar').read())
+        self.assertActivitiesMatch()
+
+    def test_has(self):
+        self.server.canned_response = '''HTTP/1.1 200 OK\r
+Server: SimpleHTTP/0.6 Python/2.5.2\r
+Date: Thu, 29 Jan 2009 20:21:47 GMT\r
+Content-type: application/octet-stream\r
+Content-Length: 20\r
+Last-Modified: Thu, 29 Jan 2009 20:21:47 GMT\r
+\r
+'''
+        t = self.get_transport()
+        self.assertTrue(t.has('foo/bar'))
+        self.assertActivitiesMatch()
+
+    def test_readv(self):
+        self.server.canned_response = '''HTTP/1.1 206 Partial Content\r
+Date: Tue, 11 Jul 2006 04:49:48 GMT\r
+Server: Apache/2.0.54 (Fedora)\r
+Last-Modified: Thu, 06 Jul 2006 20:22:05 GMT\r
+ETag: "238a3c-16ec2-805c5540"\r
+Accept-Ranges: bytes\r
+Content-Length: 1534\r
+Connection: close\r
+Content-Type: multipart/byteranges; boundary=418470f848b63279b\r
+\r
+\r
+--418470f848b63279b\r
+Content-type: text/plain; charset=UTF-8\r
+Content-range: bytes 0-254/93890\r
+\r
+mbp@sourcefrog.net-20050309040815-13242001617e4a06
+mbp@sourcefrog.net-20050309040929-eee0eb3e6d1e7627
+mbp@sourcefrog.net-20050309040957-6cad07f466bb0bb8
+mbp@sourcefrog.net-20050309041501-c840e09071de3b67
+mbp@sourcefrog.net-20050309044615-c24a3250be83220a
+\r
+--418470f848b63279b\r
+Content-type: text/plain; charset=UTF-8\r
+Content-range: bytes 1000-2049/93890\r
+\r
+40-fd4ec249b6b139ab
+mbp@sourcefrog.net-20050311063625-07858525021f270b
+mbp@sourcefrog.net-20050311231934-aa3776aff5200bb9
+mbp@sourcefrog.net-20050311231953-73aeb3a131c3699a
+mbp@sourcefrog.net-20050311232353-f5e33da490872c6a
+mbp@sourcefrog.net-20050312071639-0a8f59a34a024ff0
+mbp@sourcefrog.net-20050312073432-b2c16a55e0d6e9fb
+mbp@sourcefrog.net-20050312073831-a47c3335ece1920f
+mbp@sourcefrog.net-20050312085412-13373aa129ccbad3
+mbp@sourcefrog.net-20050313052251-2bf004cb96b39933
+mbp@sourcefrog.net-20050313052856-3edd84094687cb11
+mbp@sourcefrog.net-20050313053233-e30a4f28aef48f9d
+mbp@sourcefrog.net-20050313053853-7c64085594ff3072
+mbp@sourcefrog.net-20050313054757-a86c3f5871069e22
+mbp@sourcefrog.net-20050313061422-418f1f73b94879b9
+mbp@sourcefrog.net-20050313120651-497bd231b19df600
+mbp@sourcefrog.net-20050314024931-eae0170ef25a5d1a
+mbp@sourcefrog.net-20050314025438-d52099f915fe65fc
+mbp@sourcefrog.net-20050314025539-637a636692c055cf
+mbp@sourcefrog.net-20050314025737-55eb441f430ab4ba
+mbp@sourcefrog.net-20050314025901-d74aa93bb7ee8f62
+mbp@source\r
+--418470f848b63279b--\r
+'''
+        t = self.get_transport()
+        # Remember that the request is ignored and that the ranges below
+        # doesn't have to match the canned response.
+        l = list(t.readv('/foo/bar', ((0, 255), (1000, 1050))))
+        self.assertEqual(2, len(l))
+        self.assertActivitiesMatch()
+
+    def test_post(self):
+        self.server.canned_response = '''HTTP/1.1 200 OK\r
+Date: Tue, 11 Jul 2006 04:32:56 GMT\r
+Server: Apache/2.0.54 (Fedora)\r
+Last-Modified: Sun, 23 Apr 2006 19:35:20 GMT\r
+ETag: "56691-23-38e9ae00"\r
+Accept-Ranges: bytes\r
+Content-Length: 35\r
+Connection: close\r
+Content-Type: text/plain; charset=UTF-8\r
+\r
+lalala whatever as long as itsssss
+'''
+        t = self.get_transport()
+        # We must send a single line of body bytes, see
+        # PredefinedRequestHandler.handle_one_request
+        code, f = t._post('abc def end-of-body\n')
+        self.assertEqual('lalala whatever as long as itsssss\n', f.read())
+        self.assertActivitiesMatch()
