@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ import bzrlib
 from bzrlib import (
     config,
     errors,
+    lock,
     osutils,
     tests,
     transport,
@@ -125,7 +126,7 @@ class TestLockDir(TestCaseWithTransport):
         lf1.attempt_lock()
         lf2 = LockDir(t, 'test_lock')
         try:
-            # locking is between LockDir instances; aliases within 
+            # locking is between LockDir instances; aliases within
             # a single process are not detected
             lf2.attempt_lock()
             self.fail('Failed to detect lock collision')
@@ -166,7 +167,7 @@ class TestLockDir(TestCaseWithTransport):
 
     def test_30_lock_wait_fail(self):
         """Wait on a lock, then fail
-        
+
         We ask to wait up to 400ms; this should fail within at most one
         second.  (Longer times are more realistic but we don't want the test
         suite to take too long, and this should do for now.)
@@ -184,7 +185,7 @@ class TestLockDir(TestCaseWithTransport):
             after = time.time()
             # it should only take about 0.4 seconds, but we allow more time in
             # case the machine is heavily loaded
-            self.assertTrue(after - before <= 8.0, 
+            self.assertTrue(after - before <= 8.0,
                     "took %f seconds to detect lock contention" % (after - before))
         finally:
             lf1.unlock()
@@ -225,7 +226,7 @@ class TestLockDir(TestCaseWithTransport):
     def test_32_lock_wait_succeed(self):
         """Succeed when trying to acquire a lock that gets released
 
-        One thread holds on a lock and then releases it; another 
+        One thread holds on a lock and then releases it; another
         tries to lock it.
         """
         # This test sometimes fails like this:
@@ -276,7 +277,7 @@ class TestLockDir(TestCaseWithTransport):
         self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
 
     def test_34_lock_write_waits(self):
-        """LockDir.lock_write() will wait for the lock.""" 
+        """LockDir.lock_write() will wait for the lock."""
         # the test suite sets the default to 0 to make deadlocks fail fast.
         # change it for this test, as we want to try a manual deadlock.
         raise tests.TestSkipped('Timing-sensitive test')
@@ -319,19 +320,19 @@ class TestLockDir(TestCaseWithTransport):
 
     def test_35_wait_lock_changing(self):
         """LockDir.wait_lock() will report if the lock changes underneath.
-        
+
         This is the stages we want to happen:
 
         0) Synchronization locks are created and locked.
         1) Lock1 obtains the lockdir, and releases the 'check' lock.
         2) Lock2 grabs the 'check' lock, and checks the lockdir.
-           It sees the lockdir is already acquired, reports the fact, 
+           It sees the lockdir is already acquired, reports the fact,
            and unsets the 'checked' lock.
         3) Thread1 blocks on acquiring the 'checked' lock, and then tells
            Lock1 to release and acquire the lockdir. This resets the 'check'
            lock.
         4) Lock2 acquires the 'check' lock, and checks again. It notices
-           that the holder of the lock has changed, and so reports a new 
+           that the holder of the lock has changed, and so reports a new
            lock holder.
         5) Thread1 blocks on the 'checked' lock, this time, it completely
            unlocks the lockdir, allowing Lock2 to acquire the lock.
@@ -531,7 +532,7 @@ class TestLockDir(TestCaseWithTransport):
         """Check the on-disk representation of LockDirs is as expected.
 
         There should always be a top-level directory named by the lock.
-        When the lock is held, there should be a lockname/held directory 
+        When the lock is held, there should be a lockname/held directory
         containing an info file.
         """
         t = self.get_transport()
@@ -658,3 +659,74 @@ class TestLockDir(TestCaseWithTransport):
         self.assertRaises(errors.LockContention, ld2.attempt_lock)
         # no kibble
         check_dir(['held'])
+
+    def record_hook(self, result):
+        self._calls.append(result)
+
+    def reset_hooks(self):
+        self._old_hooks = lock.Lock.hooks
+        self.addCleanup(self.restore_hooks)
+        lock.Lock.hooks = lock.LockHooks()
+
+    def restore_hooks(self):
+        lock.Lock.hooks = self._old_hooks
+
+    def test_LockDir_acquired_success(self):
+        # the LockDir.lock_acquired hook fires when a lock is acquired.
+        self._calls = []
+        self.reset_hooks()
+        LockDir.hooks.install_named_hook('lock_acquired',
+            self.record_hook, 'record_hook')
+        ld = self.get_lock()
+        ld.create()
+        self.assertEqual([], self._calls)
+        result = ld.attempt_lock()
+        lock_path = ld.transport.abspath(ld.path)
+        self.assertEqual([lock.LockResult(lock_path, result)], self._calls)
+        ld.unlock()
+        self.assertEqual([lock.LockResult(lock_path, result)], self._calls)
+
+    def test_LockDir_acquired_fail(self):
+        # the LockDir.lock_acquired hook does not fire on failure.
+        self._calls = []
+        self.reset_hooks()
+        ld = self.get_lock()
+        ld.create()
+        ld2 = self.get_lock()
+        ld2.attempt_lock()
+        # install a lock hook now, when the disk lock is locked
+        LockDir.hooks.install_named_hook('lock_acquired',
+            self.record_hook, 'record_hook')
+        self.assertRaises(errors.LockContention, ld.attempt_lock)
+        self.assertEqual([], self._calls)
+        ld2.unlock()
+        self.assertEqual([], self._calls)
+
+    def test_LockDir_released_success(self):
+        # the LockDir.lock_released hook fires when a lock is acquired.
+        self._calls = []
+        self.reset_hooks()
+        LockDir.hooks.install_named_hook('lock_released',
+            self.record_hook, 'record_hook')
+        ld = self.get_lock()
+        ld.create()
+        self.assertEqual([], self._calls)
+        result = ld.attempt_lock()
+        self.assertEqual([], self._calls)
+        ld.unlock()
+        lock_path = ld.transport.abspath(ld.path)
+        self.assertEqual([lock.LockResult(lock_path, result)], self._calls)
+
+    def test_LockDir_released_fail(self):
+        # the LockDir.lock_released hook does not fire on failure.
+        self._calls = []
+        self.reset_hooks()
+        ld = self.get_lock()
+        ld.create()
+        ld2 = self.get_lock()
+        ld.attempt_lock()
+        ld2.force_break(ld2.peek())
+        LockDir.hooks.install_named_hook('lock_released',
+            self.record_hook, 'record_hook')
+        self.assertRaises(LockBroken, ld.unlock)
+        self.assertEqual([], self._calls)
