@@ -28,7 +28,7 @@ def split_lines(t):
 # common base
 TZU = split_lines("""     The Nameless is the origin of Heaven and Earth;
      The named is the mother of all things.
-     
+
      Therefore let there always be non-being,
        so we may see their subtlety,
      And let there always be being,
@@ -59,7 +59,7 @@ TAO = split_lines("""     The Way that can be told of is not the eternal Way;
      The name that can be named is not the eternal name.
      The Nameless is the origin of Heaven and Earth;
      The named is the mother of all things.
-     
+
      Therefore let there always be non-being,
        so we may see their subtlety,
      And let there always be being,
@@ -67,7 +67,7 @@ TAO = split_lines("""     The Way that can be told of is not the eternal Way;
      The two are the same,
      But after they are produced,
        they have different names.
-     
+
        -- The Way of Lao-Tzu, tr. Wing-tsit Chan
 
 """)
@@ -85,7 +85,7 @@ MERGED_RESULT = split_lines("""     The Way that can be told of is not the etern
        they have different names.
 <<<<<<< LAO
 =======
-     
+
        -- The Way of Lao-Tzu, tr. Wing-tsit Chan
 
 >>>>>>> TAO
@@ -132,7 +132,7 @@ class TestMerge3(TestCase):
         self.assertEquals(list(m3.merge_groups()),
                           [('a', ['aaa', 'bbb']),
                            ('unchanged', ['zz'])])
-        
+
     def test_null_insert(self):
         m3 = Merge3([],
                     ['aaa', 'bbb'],
@@ -223,7 +223,7 @@ bbb
                             mid_marker='--',
                             end_marker='>>')
         self.assertEquals(''.join(ml), 'aaa\n222\nbbb\n')
-        
+
 
     def test_insert_clash(self):
         """Both try to insert lines in the same place."""
@@ -357,9 +357,68 @@ bbb
         this_text = ("a\n"*10+"b\n" * 10).splitlines(True)
         other_text = ("a\n"*10+"c\n"+"b\n" * 8 + "c\n").splitlines(True)
         m3 = Merge3(base_text, other_text, this_text)
-        m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True, 
+        m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True,
                                  base_marker='|||||||')
         self.assertRaises(CantReprocessAndShowBase, list, m_lines)
 
     def test_binary(self):
         self.assertRaises(BinaryFile, Merge3, ['\x00'], ['a'], ['b'])
+
+    def test_dos_text(self):
+        base_text = 'a\r\n'
+        this_text = 'b\r\n'
+        other_text = 'c\r\n'
+        m3 = Merge3(base_text.splitlines(True), other_text.splitlines(True),
+                    this_text.splitlines(True))
+        m_lines = m3.merge_lines('OTHER', 'THIS')
+        self.assertEqual('<<<<<<< OTHER\r\nc\r\n=======\r\nb\r\n'
+            '>>>>>>> THIS\r\n'.splitlines(True), list(m_lines))
+
+    def test_mac_text(self):
+        base_text = 'a\r'
+        this_text = 'b\r'
+        other_text = 'c\r'
+        m3 = Merge3(base_text.splitlines(True), other_text.splitlines(True),
+                    this_text.splitlines(True))
+        m_lines = m3.merge_lines('OTHER', 'THIS')
+        self.assertEqual('<<<<<<< OTHER\rc\r=======\rb\r'
+            '>>>>>>> THIS\r'.splitlines(True), list(m_lines))
+
+    def test_merge3_cherrypick(self):
+        base_text = "a\nb\n"
+        this_text = "a\n"
+        other_text = "a\nb\nc\n"
+        # When cherrypicking, lines in base are not part of the conflict
+        m3 = Merge3(base_text.splitlines(True), this_text.splitlines(True),
+                    other_text.splitlines(True), is_cherrypick=True)
+        m_lines = m3.merge_lines()
+        self.assertEqualDiff('a\n<<<<<<<\n=======\nc\n>>>>>>>\n',
+                             ''.join(m_lines))
+
+        # This is not symmetric
+        m3 = Merge3(base_text.splitlines(True), other_text.splitlines(True),
+                    this_text.splitlines(True), is_cherrypick=True)
+        m_lines = m3.merge_lines()
+        self.assertEqualDiff('a\n<<<<<<<\nb\nc\n=======\n>>>>>>>\n',
+                             ''.join(m_lines))
+
+    def test_merge3_cherrypick_w_mixed(self):
+        base_text = 'a\nb\nc\nd\ne\n'
+        this_text = 'a\nb\nq\n'
+        other_text = 'a\nb\nc\nd\nf\ne\ng\n'
+        # When cherrypicking, lines in base are not part of the conflict
+        m3 = Merge3(base_text.splitlines(True), this_text.splitlines(True),
+                    other_text.splitlines(True), is_cherrypick=True)
+        m_lines = m3.merge_lines()
+        self.assertEqualDiff('a\n'
+                             'b\n'
+                             '<<<<<<<\n'
+                             'q\n'
+                             '=======\n'
+                             'f\n'
+                             '>>>>>>>\n'
+                             '<<<<<<<\n'
+                             '=======\n'
+                             'g\n'
+                             '>>>>>>>\n',
+                             ''.join(m_lines))
