@@ -20,12 +20,13 @@
 import os
 import sys
 
-import bzrlib
 from bzrlib import (
+    commit,
     errors,
     msgeditor,
     osutils,
     tests,
+    trace,
     )
 from bzrlib.branch import Branch
 from bzrlib.config import ensure_config_dir_exists, config_filename
@@ -34,14 +35,14 @@ from bzrlib.msgeditor import (
     edit_commit_message_encoded
 )
 from bzrlib.tests import (
-    iter_suite_tests,
-    probe_bad_non_ascii,
-    split_suite_by_re,
     TestCaseWithTransport,
     TestNotApplicable,
     TestSkipped,
+    multiply_tests,
+    probe_bad_non_ascii,
+    split_suite_by_re,
     )
-from bzrlib.tests.EncodingAdapter import EncodingTestAdapter
+from bzrlib.tests.EncodingAdapter import encoding_scenarios
 from bzrlib.trace import mutter
 
 
@@ -49,9 +50,7 @@ def load_tests(standard_tests, module, loader):
     """Parameterize the test for tempfile creation with different encodings."""
     to_adapt, result = split_suite_by_re(standard_tests,
         "test__create_temp_file_with_commit_template_in_unicode_dir")
-    for test in iter_suite_tests(to_adapt):
-        result.addTests(EncodingTestAdapter().adapt(test))
-    return result
+    return multiply_tests(to_adapt, encoding_scenarios, result)
 
 
 class MsgEditorTest(TestCaseWithTransport):
@@ -68,7 +67,7 @@ class MsgEditorTest(TestCaseWithTransport):
                 "filesystem encoding %s" % sys.getfilesystemencoding())
         working_tree.add(filename)
         return working_tree
-    
+
     def test_commit_template(self):
         """Test building a commit message template"""
         working_tree = self.make_uncommitted_tree()
@@ -80,6 +79,38 @@ added:
   hell\u00d8
 """)
 
+    def make_multiple_pending_tree(self):
+        from bzrlib import config
+        config.GlobalConfig().set_user_option('email',
+                                              'Bilbo Baggins <bb@hobbit.net>')
+        tree = self.make_branch_and_tree('a')
+        tree.commit('Initial checkin.', timestamp=1230912900, timezone=0)
+        tree2 = tree.bzrdir.clone('b').open_workingtree()
+        tree.commit('Minor tweak.', timestamp=1231977840, timezone=0)
+        tree2.commit('Feature X work.', timestamp=1233186240, timezone=0)
+        tree3 = tree2.bzrdir.clone('c').open_workingtree()
+        tree2.commit('Feature X finished.', timestamp=1233187680, timezone=0)
+        tree3.commit('Feature Y, based on initial X work.',
+                     timestamp=1233285960, timezone=0)
+        tree.merge_from_branch(tree2.branch)
+        tree.merge_from_branch(tree3.branch)
+        return tree
+
+    def test_commit_template_pending_merges(self):
+        """Test building a commit message template when there are pending
+        merges.  The commit message should show all pending merge revisions,
+        as does 'status -v', not only the merge tips.
+        """
+        working_tree = self.make_multiple_pending_tree()
+        template = msgeditor.make_commit_message_template(working_tree, None)
+        self.assertEqualDiff(template,
+u"""\
+pending merges:
+  Bilbo Baggins 2009-01-29 Feature X finished.
+    Bilbo Baggins 2009-01-28 Feature X work.
+  Bilbo Baggins 2009-01-30 Feature Y, based on initial X work.
+""")
+
     def test_commit_template_encoded(self):
         """Test building a commit message template"""
         working_tree = self.make_uncommitted_tree()
@@ -110,19 +141,21 @@ added:
   hell\u00d8
 """.encode('utf8') in template)
 
-    def test_run_editor(self):
+    def make_do_nothing_editor(self):
         if sys.platform == "win32":
             f = file('fed.bat', 'w')
             f.write('@rem dummy fed')
             f.close()
-            os.environ['BZR_EDITOR'] = 'fed.bat'
+            return 'fed.bat'
         else:
             f = file('fed.sh', 'wb')
             f.write('#!/bin/sh\n')
             f.close()
             os.chmod('fed.sh', 0755)
-            os.environ['BZR_EDITOR'] = './fed.sh'
+            return './fed.sh'
 
+    def test_run_editor(self):
+        os.environ['BZR_EDITOR'] = self.make_do_nothing_editor()
         self.assertEqual(True, msgeditor._run_editor(''),
                          'Unable to run dummy fake editor')
 
@@ -217,6 +250,7 @@ if len(sys.argv) == 2:
             f.close()
 
             editors = list(msgeditor._get_editor())
+            editors = [editor for (editor, cfg_src) in editors]
 
             self.assertEqual(['bzr_editor', 'config_editor', 'visual',
                               'editor'], editors[:4])
@@ -242,6 +276,29 @@ if len(sys.argv) == 2:
             else:
                 os.environ['EDITOR'] = editor
 
+    def test__run_editor_EACCES(self):
+        """If running a configured editor raises EACESS, the user is warned."""
+        os.environ['BZR_EDITOR'] = 'eacces.py'
+        f = file('eacces.py', 'wb')
+        f.write('# Not a real editor')
+        f.close()
+        # Make the fake editor unreadable (and unexecutable)
+        os.chmod('eacces.py', 0)
+        # Set $EDITOR so that _run_editor will terminate before trying real
+        # editors.
+        os.environ['EDITOR'] = self.make_do_nothing_editor()
+        # Call _run_editor, capturing mutter.warning calls.
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        _warning = trace.warning
+        trace.warning = warning
+        try:
+            msgeditor._run_editor('')
+        finally:
+            trace.warning = _warning
+        self.assertStartsWith(warnings[0], 'Could not start editor "eacces.py"')
+
     def test__create_temp_file_with_commit_template(self):
         # check that commit template written properly
         # and has platform native line-endings (CRLF on win32)
@@ -280,10 +337,10 @@ if len(sys.argv) == 2:
             # LANG env variable has no effect on Windows
             # but some characters anyway cannot be represented
             # in default user encoding
-            char = probe_bad_non_ascii(bzrlib.user_encoding)
+            char = probe_bad_non_ascii(osutils.get_user_encoding())
             if char is None:
                 raise TestSkipped('Cannot find suitable non-ascii character '
-                    'for user_encoding (%s)' % bzrlib.user_encoding)
+                    'for user_encoding (%s)' % osutils.get_user_encoding())
 
             self.make_fake_editor(message=char)
 
@@ -292,3 +349,18 @@ if len(sys.argv) == 2:
                               msgeditor.edit_commit_message, '')
         finally:
             osutils.set_or_unset_env('LANG', old_env)
+
+    def test_generate_commit_message_template_no_hooks(self):
+        commit_obj = commit.Commit()
+        self.assertIs(None,
+            msgeditor.generate_commit_message_template(commit_obj))
+
+    def test_generate_commit_message_template_hook(self):
+        def restoreDefaults():
+            msgeditor.hooks['commit_message_template'] = []
+        self.addCleanup(restoreDefaults)
+        msgeditor.hooks.install_named_hook("commit_message_template",
+                lambda commit_obj, msg: "save me some typing\n", None)
+        commit_obj = commit.Commit()
+        self.assertEquals("save me some typing\n",
+            msgeditor.generate_commit_message_template(commit_obj))
