@@ -26,10 +26,10 @@ permissions should be inherited individually, rather than all be the same.
 """
 
 # TODO: jam 20051215 There are no tests for ftp yet, because we have no ftp server
-# TODO: jam 20051215 Currently the default behavior for 'bzr branch' is just 
+# TODO: jam 20051215 Currently the default behavior for 'bzr branch' is just
 #                    defined by the local umask. This isn't terrible, is it
 #                    the truly desired behavior?
- 
+
 import os
 import sys
 import stat
@@ -38,7 +38,6 @@ import urllib
 
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.tests import TestCaseWithTransport, TestSkipped
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.transport import get_transport
@@ -97,7 +96,7 @@ class TestPermissions(TestCaseWithTransport):
 
         # although we are modifying the filesystem
         # underneath the objects, they are not locked, and thus it must
-        # be safe for most operations. But here we want to observe a 
+        # be safe for most operations. But here we want to observe a
         # mode change in the control bits, which current do not refresh
         # when a new lock is taken out.
         t = WorkingTree.open('.')
@@ -138,6 +137,18 @@ class TestPermissions(TestCaseWithTransport):
         t.commit('new c')
         check_mode_r(self, '.bzr', 0664, 0775)
 
+    def test_new_files_group_sticky_bit(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+        elif sys.platform == 'darwin':
+            # OS X creates temp dirs with the 'wheel' group, which users are
+            # not likely to be in, and this prevents us from setting the sgid
+            # bit
+            os.chown(self.test_dir, os.getuid(), os.getgid())
+
+        t = self.make_branch_and_tree('.')
+        b = t.branch
+
         # Test the group sticky bit
         # Recursively update the modes of all files
         chmod_r('.bzr', 0664, 02775)
