@@ -38,7 +38,8 @@ from bzrlib import (
     )
 from bzrlib.branch import Branch, BranchReferenceFormat
 import bzrlib.smart.branch
-import bzrlib.smart.bzrdir
+import bzrlib.smart.bzrdir, bzrlib.smart.bzrdir as smart_dir
+import bzrlib.smart.packrepository
 import bzrlib.smart.repository
 from bzrlib.smart.request import (
     FailedSmartServerResponse,
@@ -47,9 +48,7 @@ from bzrlib.smart.request import (
     SuccessfulSmartServerResponse,
     )
 from bzrlib.tests import (
-    iter_suite_tests,
     split_suite_by_re,
-    TestScenarioApplier,
     )
 from bzrlib.transport import chroot, get_transport
 from bzrlib.util import bencode
@@ -59,22 +58,22 @@ def load_tests(standard_tests, module, loader):
     """Multiply tests version and protocol consistency."""
     # FindRepository tests.
     bzrdir_mod = bzrlib.smart.bzrdir
-    applier = TestScenarioApplier()
-    applier.scenarios = [
+    scenarios = [
         ("find_repository", {
             "_request_class":bzrdir_mod.SmartServerRequestFindRepositoryV1}),
         ("find_repositoryV2", {
             "_request_class":bzrdir_mod.SmartServerRequestFindRepositoryV2}),
+        ("find_repositoryV3", {
+            "_request_class":bzrdir_mod.SmartServerRequestFindRepositoryV3}),
         ]
     to_adapt, result = split_suite_by_re(standard_tests,
         "TestSmartServerRequestFindRepository")
     v2_only, v1_and_2 = split_suite_by_re(to_adapt,
         "_v2")
-    for test in iter_suite_tests(v1_and_2):
-        result.addTests(applier.adapt(test))
-    del applier.scenarios[0]
-    for test in iter_suite_tests(v2_only):
-        result.addTests(applier.adapt(test))
+    tests.multiply_tests(v1_and_2, scenarios, result)
+    # The first scenario is only applicable to v1 protocols, it is deleted
+    # since.
+    tests.multiply_tests(v2_only, scenarios[1:], result)
     return result
 
 
@@ -131,10 +130,10 @@ class TestSmartServerResponse(tests.TestCase):
     def test__str__(self):
         """SmartServerResponses can be stringified."""
         self.assertEqual(
-            "<SmartServerResponse status=OK args=('args',) body='body'>",
+            "<SuccessfulSmartServerResponse args=('args',) body='body'>",
             str(SuccessfulSmartServerResponse(('args',), 'body')))
         self.assertEqual(
-            "<SmartServerResponse status=ERR args=('args',) body='body'>",
+            "<FailedSmartServerResponse args=('args',) body='body'>",
             str(FailedSmartServerResponse(('args',), 'body')))
 
 
@@ -160,6 +159,60 @@ class TestSmartServerRequest(tests.TestCaseWithMemoryTransport):
             request.transport_from_client_path('foo/').base)
 
 
+class TestSmartServerBzrDirRequestCloningMetaDir(
+    tests.TestCaseWithMemoryTransport):
+    """Tests for BzrDir.cloning_metadir."""
+
+    def test_cloning_metadir(self):
+        """When there is a bzrdir present, the call succeeds."""
+        backing = self.get_transport()
+        dir = self.make_bzrdir('.')
+        local_result = dir.cloning_metadir()
+        request_class = smart_dir.SmartServerBzrDirRequestCloningMetaDir
+        request = request_class(backing)
+        expected = SuccessfulSmartServerResponse(
+            (local_result.network_name(),
+            local_result.repository_format.network_name(),
+            ('branch', local_result.get_branch_format().network_name())))
+        self.assertEqual(expected, request.execute('', 'False'))
+
+    def test_cloning_metadir_reference(self):
+        """The request works when bzrdir contains a branch reference."""
+        backing = self.get_transport()
+        referenced_branch = self.make_branch('referenced')
+        dir = self.make_bzrdir('.')
+        local_result = dir.cloning_metadir()
+        reference = BranchReferenceFormat().initialize(dir, referenced_branch)
+        reference_url = BranchReferenceFormat().get_reference(dir)
+        # The server shouldn't try to follow the branch reference, so it's fine
+        # if the referenced branch isn't reachable.
+        backing.rename('referenced', 'moved')
+        request_class = smart_dir.SmartServerBzrDirRequestCloningMetaDir
+        request = request_class(backing)
+        expected = SuccessfulSmartServerResponse(
+            (local_result.network_name(),
+            local_result.repository_format.network_name(),
+            ('ref', reference_url)))
+        self.assertEqual(expected, request.execute('', 'False'))
+
+
+class TestSmartServerRequestCreateRepository(tests.TestCaseWithMemoryTransport):
+    """Tests for BzrDir.create_repository."""
+
+    def test_makes_repository(self):
+        """When there is a bzrdir present, the call succeeds."""
+        backing = self.get_transport()
+        self.make_bzrdir('.')
+        request_class = bzrlib.smart.bzrdir.SmartServerRequestCreateRepository
+        request = request_class(backing)
+        reference_bzrdir_format = bzrdir.format_registry.get('default')()
+        reference_format = reference_bzrdir_format.repository_format
+        network_name = reference_format.network_name()
+        expected = SuccessfulSmartServerResponse(
+            ('ok', 'no', 'no', 'no', network_name))
+        self.assertEqual(expected, request.execute('', network_name, 'True'))
+
+
 class TestSmartServerRequestFindRepository(tests.TestCaseWithMemoryTransport):
     """Tests for BzrDir.find_repository."""
 
@@ -172,8 +225,8 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithMemoryTransport):
             request.execute(''))
 
     def test_nonshared_repository(self):
-        # nonshared repositorys only allow 'find' to return a handle when the 
-        # path the repository is being searched on is the same as that that 
+        # nonshared repositorys only allow 'find' to return a handle when the
+        # path the repository is being searched on is the same as that that
         # the repository is at.
         backing = self.get_transport()
         request = self._request_class(backing)
@@ -197,7 +250,12 @@ class TestSmartServerRequestFindRepository(tests.TestCaseWithMemoryTransport):
             subtrees = 'yes'
         else:
             subtrees = 'no'
-        if (smart.bzrdir.SmartServerRequestFindRepositoryV2 ==
+        if (smart.bzrdir.SmartServerRequestFindRepositoryV3 ==
+            self._request_class):
+            return SuccessfulSmartServerResponse(
+                ('ok', '', rich_root, subtrees, 'no',
+                 repo._format.network_name()))
+        elif (smart.bzrdir.SmartServerRequestFindRepositoryV2 ==
             self._request_class):
             # All tests so far are on formats, and for non-external
             # repositories.
@@ -250,7 +308,7 @@ class TestSmartServerRequestInitializeBzrDir(tests.TestCaseWithMemoryTransport):
         self.assertEqual(SmartServerResponse(('ok', )),
             request.execute(''))
         made_dir = bzrdir.BzrDir.open_from_transport(backing)
-        # no branch, tree or repository is expected with the current 
+        # no branch, tree or repository is expected with the current
         # default formart.
         self.assertRaises(errors.NoWorkingTree, made_dir.open_workingtree)
         self.assertRaises(errors.NotBranchError, made_dir.open_branch)
@@ -302,6 +360,36 @@ class TestSmartServerRequestOpenBranch(TestCaseWithChrootedTransport):
             request.execute('reference'))
 
 
+class TestSmartServerRequestOpenBranchV2(TestCaseWithChrootedTransport):
+
+    def test_no_branch(self):
+        """When there is no branch, ('nobranch', ) is returned."""
+        backing = self.get_transport()
+        self.make_bzrdir('.')
+        request = smart.bzrdir.SmartServerRequestOpenBranchV2(backing)
+        self.assertEqual(SmartServerResponse(('nobranch', )),
+            request.execute(''))
+
+    def test_branch(self):
+        """When there is a branch, 'ok' is returned."""
+        backing = self.get_transport()
+        expected = self.make_branch('.')._format.network_name()
+        request = smart.bzrdir.SmartServerRequestOpenBranchV2(backing)
+        self.assertEqual(SuccessfulSmartServerResponse(('branch', expected)),
+            request.execute(''))
+
+    def test_branch_reference(self):
+        """When there is a branch reference, the reference URL is returned."""
+        backing = self.get_transport()
+        request = smart.bzrdir.SmartServerRequestOpenBranchV2(backing)
+        branch = self.make_branch('branch')
+        checkout = branch.create_checkout('reference',lightweight=True)
+        reference_url = BranchReferenceFormat().get_reference(checkout.bzrdir)
+        self.assertFileEqual(reference_url, 'reference/.bzr/branch/location')
+        self.assertEqual(SuccessfulSmartServerResponse(('ref', reference_url)),
+            request.execute('reference'))
+
+
 class TestSmartServerRequestRevisionHistory(tests.TestCaseWithMemoryTransport):
 
     def test_empty(self):
@@ -388,7 +476,7 @@ class TestSmartServerBranchRequestGetConfigFile(tests.TestCaseWithMemoryTranspor
     def test_with_content(self):
         # SmartServerBranchGetConfigFile should return the content from
         # branch.control_files.get('branch.conf') for now - in the future it may
-        # perform more complex processing. 
+        # perform more complex processing.
         backing = self.get_transport()
         request = smart.branch.SmartServerBranchGetConfigFile(backing)
         branch = self.make_branch('.')
@@ -415,7 +503,7 @@ class SetLastRevisionTestBase(tests.TestCaseWithMemoryTransport):
 
     def unlock_branch(self):
         self.tree.branch.unlock()
-        
+
     def set_last_revision(self, revision_id, revno):
         branch_token, repo_token = self.lock_branch()
         response = self._set_last_revision(
@@ -427,7 +515,7 @@ class SetLastRevisionTestBase(tests.TestCaseWithMemoryTransport):
         response = self.set_last_revision(revision_id, revno)
         self.assertEqual(SuccessfulSmartServerResponse(('ok',)), response)
 
-        
+
 class TestSetLastRevisionVerbMixin(object):
     """Mixin test case for verbs that implement set_last_revision."""
 
@@ -538,7 +626,7 @@ class TestSmartServerBranchRequestSetLastRevisionEx(
         self.assertEqual(
             SuccessfulSmartServerResponse(('ok', revno, revision_id)),
             response)
-        
+
     def test_branch_last_revision_info_rewind(self):
         """A branch's tip can be set to a revision that is an ancestor of the
         current tip, but only if allow_overwrite_descendant is passed.
@@ -587,7 +675,7 @@ class TestSmartServerBranchRequestSetLastRevisionEx(
         # child-1.
         new_r2 = self.tree.commit('2nd commit', rev_id='child-2')
         self.tree.unlock()
-        
+
     def test_not_allow_diverged(self):
         """If allow_diverged is not passed, then setting a divergent history
         returns a Diverged error.
@@ -615,6 +703,53 @@ class TestSmartServerBranchRequestSetLastRevisionEx(
         self.assertEqual('child-1', self.tree.branch.last_revision())
 
 
+class TestSmartServerBranchRequestGetParent(tests.TestCaseWithMemoryTransport):
+
+    def test_get_parent_none(self):
+        base_branch = self.make_branch('base')
+        request = smart.branch.SmartServerBranchGetParent(self.get_transport())
+        response = request.execute('base')
+        self.assertEquals(
+            SuccessfulSmartServerResponse(('',)), response)
+
+    def test_get_parent_something(self):
+        base_branch = self.make_branch('base')
+        base_branch.set_parent(self.get_url('foo'))
+        request = smart.branch.SmartServerBranchGetParent(self.get_transport())
+        response = request.execute('base')
+        self.assertEquals(
+            SuccessfulSmartServerResponse(("../foo",)),
+            response)
+
+
+class TestSmartServerBranchRequestGetTagsBytes(tests.TestCaseWithMemoryTransport):
+# Only called when the branch format and tags match [yay factory
+# methods] so only need to test straight forward cases.
+
+    def test_get_bytes(self):
+        base_branch = self.make_branch('base')
+        request = smart.branch.SmartServerBranchGetTagsBytes(
+            self.get_transport())
+        response = request.execute('base')
+        self.assertEquals(
+            SuccessfulSmartServerResponse(('',)), response)
+
+
+class TestSmartServerBranchRequestGetStackedOnURL(tests.TestCaseWithMemoryTransport):
+
+    def test_get_stacked_on_url(self):
+        base_branch = self.make_branch('base', format='1.6')
+        stacked_branch = self.make_branch('stacked', format='1.6')
+        # typically should be relative
+        stacked_branch.set_stacked_on_url('../base')
+        request = smart.branch.SmartServerBranchRequestGetStackedOnURL(
+            self.get_transport())
+        response = request.execute('stacked')
+        self.assertEquals(
+            SmartServerResponse(('ok', '../base')),
+            response)
+
+
 class TestSmartServerBranchRequestLockWrite(tests.TestCaseWithMemoryTransport):
 
     def setUp(self):
@@ -819,7 +954,7 @@ class TestSmartServerRepositoryGetRevisionGraph(tests.TestCaseWithMemoryTranspor
 
         self.assertEqual(SmartServerResponse(('ok', ), rev_id_utf8),
             request.execute('', rev_id_utf8))
-    
+
     def test_no_such_revision(self):
         backing = self.get_transport()
         request = smart.repository.SmartServerRepositoryGetRevisionGraph(backing)
@@ -835,6 +970,45 @@ class TestSmartServerRepositoryGetRevisionGraph(tests.TestCaseWithMemoryTranspor
             request.execute('', 'missingrevision'))
 
 
+class TestSmartServerRepositoryGetStream(tests.TestCaseWithMemoryTransport):
+
+    def make_two_commit_repo(self):
+        tree = self.make_branch_and_memory_tree('.')
+        tree.lock_write()
+        tree.add('')
+        r1 = tree.commit('1st commit')
+        r2 = tree.commit('2nd commit', rev_id=u'\xc8'.encode('utf-8'))
+        tree.unlock()
+        repo = tree.branch.repository
+        return repo, r1, r2
+
+    def test_ancestry_of(self):
+        """The search argument may be a 'ancestry-of' some heads'."""
+        backing = self.get_transport()
+        request = smart.repository.SmartServerRepositoryGetStream(backing)
+        repo, r1, r2 = self.make_two_commit_repo()
+        fetch_spec = ['ancestry-of', r2]
+        lines = '\n'.join(fetch_spec)
+        request.execute('', repo._format.network_name())
+        response = request.do_body(lines)
+        self.assertEqual(('ok',), response.args)
+        stream_bytes = ''.join(response.body_stream)
+        self.assertStartsWith(stream_bytes, 'Bazaar pack format 1')
+
+    def test_search(self):
+        """The search argument may be a 'search' of some explicit keys."""
+        backing = self.get_transport()
+        request = smart.repository.SmartServerRepositoryGetStream(backing)
+        repo, r1, r2 = self.make_two_commit_repo()
+        fetch_spec = ['search', '%s %s' % (r1, r2), 'null:', '2']
+        lines = '\n'.join(fetch_spec)
+        request.execute('', repo._format.network_name())
+        response = request.do_body(lines)
+        self.assertEqual(('ok',), response.args)
+        stream_bytes = ''.join(response.body_stream)
+        self.assertStartsWith(stream_bytes, 'Bazaar pack format 1')
+
+
 class TestSmartServerRequestHasRevision(tests.TestCaseWithMemoryTransport):
 
     def test_missing_revision(self):
@@ -1022,6 +1196,78 @@ class TestSmartServerIsReadonly(tests.TestCaseWithMemoryTransport):
             SmartServerResponse(('yes',)), response)
 
 
+class TestSmartServerRepositorySetMakeWorkingTrees(tests.TestCaseWithMemoryTransport):
+
+    def test_set_false(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.', shared=True)
+        repo.set_make_working_trees(True)
+        request_class = smart.repository.SmartServerRepositorySetMakeWorkingTrees
+        request = request_class(backing)
+        self.assertEqual(SuccessfulSmartServerResponse(('ok',)),
+            request.execute('', 'False'))
+        repo = repo.bzrdir.open_repository()
+        self.assertFalse(repo.make_working_trees())
+
+    def test_set_true(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.', shared=True)
+        repo.set_make_working_trees(False)
+        request_class = smart.repository.SmartServerRepositorySetMakeWorkingTrees
+        request = request_class(backing)
+        self.assertEqual(SuccessfulSmartServerResponse(('ok',)),
+            request.execute('', 'True'))
+        repo = repo.bzrdir.open_repository()
+        self.assertTrue(repo.make_working_trees())
+
+
+class TestSmartServerPackRepositoryAutopack(tests.TestCaseWithTransport):
+
+    def make_repo_needing_autopacking(self, path='.'):
+        # Make a repo in need of autopacking.
+        tree = self.make_branch_and_tree('.', format='pack-0.92')
+        repo = tree.branch.repository
+        # monkey-patch the pack collection to disable autopacking
+        repo._pack_collection._max_pack_count = lambda count: count
+        for x in range(10):
+            tree.commit('commit %s' % x)
+        self.assertEqual(10, len(repo._pack_collection.names()))
+        del repo._pack_collection._max_pack_count
+        return repo
+
+    def test_autopack_needed(self):
+        repo = self.make_repo_needing_autopacking()
+        backing = self.get_transport()
+        request = smart.packrepository.SmartServerPackRepositoryAutopack(
+            backing)
+        response = request.execute('')
+        self.assertEqual(SmartServerResponse(('ok',)), response)
+        repo._pack_collection.reload_pack_names()
+        self.assertEqual(1, len(repo._pack_collection.names()))
+
+    def test_autopack_not_needed(self):
+        tree = self.make_branch_and_tree('.', format='pack-0.92')
+        repo = tree.branch.repository
+        for x in range(9):
+            tree.commit('commit %s' % x)
+        backing = self.get_transport()
+        request = smart.packrepository.SmartServerPackRepositoryAutopack(
+            backing)
+        response = request.execute('')
+        self.assertEqual(SmartServerResponse(('ok',)), response)
+        repo._pack_collection.reload_pack_names()
+        self.assertEqual(9, len(repo._pack_collection.names()))
+
+    def test_autopack_on_nonpack_format(self):
+        """A request to autopack a non-pack repo is a no-op."""
+        repo = self.make_repository('.', format='knit')
+        backing = self.get_transport()
+        request = smart.packrepository.SmartServerPackRepositoryAutopack(
+            backing)
+        response = request.execute('')
+        self.assertEqual(SmartServerResponse(('ok',)), response)
+
+
 class TestHandlers(tests.TestCase):
     """Tests for the request.request_handlers object."""
 
@@ -1037,6 +1283,12 @@ class TestHandlers(tests.TestCase):
         self.assertEqual(
             smart.request.request_handlers.get('Branch.get_config_file'),
             smart.branch.SmartServerBranchGetConfigFile)
+        self.assertEqual(
+            smart.request.request_handlers.get('Branch.get_parent'),
+            smart.branch.SmartServerBranchGetParent)
+        self.assertEqual(
+            smart.request.request_handlers.get('Branch.get_tags_bytes'),
+            smart.branch.SmartServerBranchGetTagsBytes)
         self.assertEqual(
             smart.request.request_handlers.get('Branch.lock_write'),
             smart.branch.SmartServerBranchRequestLockWrite)
@@ -1064,9 +1316,18 @@ class TestHandlers(tests.TestCase):
         self.assertEqual(
             smart.request.request_handlers.get('BzrDirFormat.initialize'),
             smart.bzrdir.SmartServerRequestInitializeBzrDir)
+        self.assertEqual(
+            smart.request.request_handlers.get('BzrDir.cloning_metadir'),
+            smart.bzrdir.SmartServerBzrDirRequestCloningMetaDir)
         self.assertEqual(
             smart.request.request_handlers.get('BzrDir.open_branch'),
             smart.bzrdir.SmartServerRequestOpenBranch)
+        self.assertEqual(
+            smart.request.request_handlers.get('BzrDir.open_branchV2'),
+            smart.bzrdir.SmartServerRequestOpenBranchV2)
+        self.assertEqual(
+            smart.request.request_handlers.get('PackRepository.autopack'),
+            smart.packrepository.SmartServerPackRepositoryAutopack)
         self.assertEqual(
             smart.request.request_handlers.get('Repository.gather_stats'),
             smart.repository.SmartServerRepositoryGatherStats)
@@ -1086,6 +1347,9 @@ class TestHandlers(tests.TestCase):
         self.assertEqual(
             smart.request.request_handlers.get('Repository.lock_write'),
             smart.repository.SmartServerRepositoryLockWrite)
+        self.assertEqual(
+            smart.request.request_handlers.get('Repository.get_stream'),
+            smart.repository.SmartServerRepositoryGetStream)
         self.assertEqual(
             smart.request.request_handlers.get('Repository.tarball'),
             smart.repository.SmartServerRepositoryTarball)
