@@ -150,7 +150,7 @@ class TestAddActions(TestCase):
         self.run_action("")
 
     def test__print(self):
-        self.run_action("added path\n")
+        self.run_action("adding path\n")
 
     def run_action(self, output):
         from bzrlib.add import AddAction
