@@ -42,7 +42,7 @@ class TestStatus(TestCaseWithTransport):
             tree2.unlock()
         self.assertContainsRe(output.getvalue(), 'empty commit')
 
-    def test_multiple_pending(self):
+    def make_multiple_pending_tree(self):
         config.GlobalConfig().set_user_option('email', 'Joe Foo <joe@foo.com>')
         tree = self.make_branch_and_tree('a')
         tree.commit('commit 1', timestamp=1196796819, timezone=0)
@@ -54,10 +54,27 @@ class TestStatus(TestCaseWithTransport):
         tree3.commit('commit 3c', timestamp=1196796819, timezone=0)
         tree.merge_from_branch(tree2.branch)
         tree.merge_from_branch(tree3.branch)
+        return tree
+
+    def test_multiple_pending(self):
+        tree = self.make_multiple_pending_tree()
         output = StringIO()
         tree.lock_read()
         self.addCleanup(tree.unlock)
         show_pending_merges(tree, output)
+        # 2b doesn't appear because it's an ancestor of 3b
+        self.assertEqualDiff(
+            'pending merge tips: (use -v to see all merge revisions)\n'
+            '  Joe Foo 2007-12-04 commit 3b\n'
+            '  Joe Foo 2007-12-04 commit 3c\n',
+            output.getvalue())
+
+    def test_multiple_pending_verbose(self):
+        tree = self.make_multiple_pending_tree()
+        output = StringIO()
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        show_pending_merges(tree, output, verbose=True)
         # Even though 2b is in the ancestry of 3c, it should only be displayed
         # under the first merge parent.
         self.assertEqualDiff('pending merges:\n'
@@ -75,9 +92,10 @@ class TestStatus(TestCaseWithTransport):
         self.addCleanup(tree.unlock)
         output = StringIO()
         show_pending_merges(tree, output)
-        self.assertEqualDiff('pending merges:\n'
-                             '  (ghost) a-ghost-revision\n',
-                             output.getvalue())
+        self.assertEqualDiff(
+            'pending merge tips: (use -v to see all merge revisions)\n'
+            '  (ghost) a-ghost-revision\n',
+            output.getvalue())
 
     def test_pending_with_ghosts(self):
         """Test when a pending merge's ancestry includes ghosts."""
@@ -93,10 +111,10 @@ class TestStatus(TestCaseWithTransport):
         tree.lock_read()
         self.addCleanup(tree.unlock)
         output = StringIO()
-        show_pending_merges(tree, output)
+        show_pending_merges(tree, output, verbose=True)
         self.assertEqualDiff('pending merges:\n'
                              '  Joe Foo 2007-12-04 another non-ghost\n'
-                             '    Joe Foo 2007-12-04 commit with ghost\n'
+                             '    Joe Foo 2007-12-04 [merge] commit with ghost\n'
                              '    (ghost) a-ghost-revision\n'
                              '    Joe Foo 2007-12-04 a non-ghost\n',
                              output.getvalue())
@@ -112,20 +130,3 @@ class TestStatus(TestCaseWithTransport):
                      revision=[RevisionSpec.from_string("revid:%s" % r1_id),
                                RevisionSpec.from_string("revid:%s" % r2_id)])
         # return does not matter as long as it did not raise.
-
-    def test_pending_specific_files(self):
-        """With a specific file list, pending merges are not shown."""
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/a', 'content of a\n')])
-        tree.add('a')
-        r1_id = tree.commit('one')
-        alt = tree.bzrdir.sprout('alt').open_workingtree()
-        self.build_tree_contents([('alt/a', 'content of a\nfrom alt\n')])
-        alt_id = alt.commit('alt')
-        tree.merge_from_branch(alt.branch)
-        output = StringIO()
-        show_tree_status(tree, to_file=output)
-        self.assertContainsRe(output.getvalue(), 'pending merges:')
-        output = StringIO()
-        show_tree_status(tree, to_file=output, specific_files=['a'])
-        self.assertNotContainsRe(output.getvalue(), 'pending merges:')
