@@ -107,7 +107,7 @@ class TestTransport(TestCase):
         finally:
             # restore original values
             _set_protocol_handlers(saved_handlers)
-            
+
     def test_transport_fallback(self):
         """Transport with missing dependency causes no error"""
         saved_handlers = _get_protocol_handlers()
@@ -123,6 +123,19 @@ class TestTransport(TestCase):
         finally:
             _set_protocol_handlers(saved_handlers)
 
+    def test_ssh_hints(self):
+        """Transport ssh:// should raise an error pointing out bzr+ssh://"""
+        try:
+            get_transport('ssh://fooserver/foo')
+        except UnsupportedProtocol, e:
+            e_str = str(e)
+            self.assertEquals('Unsupported protocol'
+                              ' for url "ssh://fooserver/foo":'
+                              ' bzr supports bzr+ssh to operate over ssh, use "bzr+ssh://fooserver/foo".',
+                              str(e))
+        else:
+            self.fail('Did not raise UnsupportedProtocol')
+
     def test_LateReadError(self):
         """The LateReadError helper should raise on read()."""
         a_file = LateReadError('a path')
@@ -180,10 +193,10 @@ class TestCoalesceOffsets(TestCase):
         self.check([(0, 20, [(0, 10), (10, 10)])],
                    [(0, 10), (10, 10)])
 
-    # XXX: scary, http.readv() can't handle that --vila20071209
     def test_coalesce_overlapped(self):
-        self.check([(0, 15, [(0, 10), (5, 10)])],
-                   [(0, 10), (5, 10)])
+        self.assertRaises(ValueError,
+            self.check, [(0, 15, [(0, 10), (5, 10)])],
+                        [(0, 10), (5, 10)])
 
     def test_coalesce_limit(self):
         self.check([(10, 50, [(0, 10), (10, 10), (20, 10),
@@ -225,6 +238,16 @@ class TestCoalesceOffsets(TestCase):
                    [(10, 10), (20, 10), (30, 50), (80, 100)],
                   )
 
+    def test_coalesce_default_limit(self):
+        # By default we use a 100MB max size.
+        ten_mb = 10*1024*1024
+        self.check([(0, 10*ten_mb, [(i*ten_mb, ten_mb) for i in range(10)]),
+                    (10*ten_mb, ten_mb, [(0, ten_mb)])],
+                   [(i*ten_mb, ten_mb) for i in range(11)])
+        self.check([(0, 11*ten_mb, [(i*ten_mb, ten_mb) for i in range(11)]),],
+                   [(i*ten_mb, ten_mb) for i in range(11)],
+                   max_size=1*1024*1024*1024)
+
 
 class TestMemoryTransport(TestCase):
 
@@ -358,7 +381,7 @@ class ChrootDecoratorTransportTest(TestCase):
         self.assertEqual(server, relpath_cloned.server)
         self.assertEqual(server, abspath_cloned.server)
         server.tearDown()
-    
+
     def test_chroot_url_preserves_chroot(self):
         """Calling get_transport on a chroot transport's base should produce a
         transport with exactly the same behaviour as the original chroot
@@ -376,7 +399,7 @@ class ChrootDecoratorTransportTest(TestCase):
         self.assertEqual(transport.server, new_transport.server)
         self.assertEqual(transport.base, new_transport.base)
         server.tearDown()
-        
+
     def test_urljoin_preserves_chroot(self):
         """Using urlutils.join(url, '..') on a chroot URL should not produce a
         URL that escapes the intended chroot.
@@ -541,26 +564,26 @@ class BackupTransportHandler(Transport):
 
 class TestTransportImplementation(TestCaseInTempDir):
     """Implementation verification for transports.
-    
+
     To verify a transport we need a server factory, which is a callable
     that accepts no parameters and returns an implementation of
     bzrlib.transport.Server.
-    
+
     That Server is then used to construct transport instances and test
     the transport via loopback activity.
 
-    Currently this assumes that the Transport object is connected to the 
-    current working directory.  So that whatever is done 
-    through the transport, should show up in the working 
+    Currently this assumes that the Transport object is connected to the
+    current working directory.  So that whatever is done
+    through the transport, should show up in the working
     directory, and vice-versa. This is a bug, because its possible to have
-    URL schemes which provide access to something that may not be 
-    result in storage on the local disk, i.e. due to file system limits, or 
+    URL schemes which provide access to something that may not be
+    result in storage on the local disk, i.e. due to file system limits, or
     due to it being a database or some other non-filesystem tool.
 
     This also tests to make sure that the functions work with both
     generators and lists (assuming iter(list) is effectively a generator)
     """
-    
+
     def setUp(self):
         super(TestTransportImplementation, self).setUp()
         self._server = self.transport_server()
