@@ -18,16 +18,15 @@
 """WorkingTree implementation tests for bzr.
 
 These test the conformance of all the workingtre variations to the expected API.
-Specific tests for individual formats are in the tests/test_workingtree file 
+Specific tests for individual formats are in the tests/test_workingtree file
 rather than in tests/workingtree_implementations/*.py.
 """
 
 import bzrlib.errors as errors
 from bzrlib.transport import get_transport
 from bzrlib.tests import (
-                          adapt_modules,
                           default_transport,
-                          TestScenarioApplier,
+                          multiply_tests,
                           )
 from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
 from bzrlib.workingtree import (WorkingTreeFormat,
@@ -35,46 +34,30 @@ from bzrlib.workingtree import (WorkingTreeFormat,
                                 )
 
 
-class WorkingTreeTestProviderAdapter(TestScenarioApplier):
-    """A tool to generate a suite testing multiple workingtree formats at once.
+def make_scenarios(transport_server, transport_readonly_server, formats):
+    """Create a scenario for the specified converter
 
-    This is done by copying the test once for each transport and injecting
-    the transport_server, transport_readonly_server, and workingtree_format
-    classes into each copy. Each copy is also given a new id() to make it
-    easy to identify.
+    :param workingtree_format: The particular workingtree format to test.
+    :param bzrdir_format: The bzrdir format to test.
+    :return: a (name, options) tuple, where options is a dict of values
+        to be used as members of the TestCase.
     """
+    result = []
+    for workingtree_format in formats:
+        result.append((workingtree_format.__class__.__name__,
+            make_scenario(transport_server, transport_readonly_server,
+            workingtree_format)))
+    return result
 
-    def __init__(self, transport_server, transport_readonly_server, formats):
-        self._transport_server = transport_server
-        self._transport_readonly_server = transport_readonly_server
-        self.scenarios = self.formats_to_scenarios(formats)
 
-    def formats_to_scenarios(self, formats):
-        """Transform the input formats to a list of scenarios.
-
-        :param formats: A list [workingtree_format].
-        """
-
-        result = []
-        for workingtree_format in formats:
-            result.append(self.create_scenario(workingtree_format))
-        return result
-
-    def create_scenario(self, workingtree_format):
-        """Create a scenario for the specified converter
-
-        :param workingtree_format: The particular workingtree format to test.
-        :param bzrdir_format: The bzrdir format to test.
-        :return: a (name, options) tuple, where options is a dict of values
-            to be used as members of the TestCase.
-        """
-        scenario_options = {
-            "transport_server": self._transport_server,
-            "transport_readonly_server": self._transport_readonly_server,
-            "bzrdir_format": workingtree_format._matchingbzrdir,
-            "workingtree_format": workingtree_format,
-            }
-        return workingtree_format.__class__.__name__, scenario_options
+def make_scenario(transport_server, transport_readonly_server,
+    workingtree_format):
+    return {
+        "transport_server": transport_server,
+        "transport_readonly_server": transport_readonly_server,
+        "bzrdir_format": workingtree_format._matchingbzrdir,
+        "workingtree_format": workingtree_format,
+        }
 
 
 class TestCaseWithWorkingTree(TestCaseWithBzrDir):
@@ -86,11 +69,7 @@ class TestCaseWithWorkingTree(TestCaseWithBzrDir):
         return self.workingtree_format.initialize(made_control)
 
 
-def load_tests(basic_tests, module, loader):
-    result = loader.suiteClass()
-    # add the tests for this module
-    result.addTests(basic_tests)
-
+def load_tests(standard_tests, module, loader):
     test_workingtree_implementations = [
         'bzrlib.tests.workingtree_implementations.test_add_reference',
         'bzrlib.tests.workingtree_implementations.test_add',
@@ -101,6 +80,7 @@ def load_tests(basic_tests, module, loader):
         'bzrlib.tests.workingtree_implementations.test_commit',
         'bzrlib.tests.workingtree_implementations.test_executable',
         'bzrlib.tests.workingtree_implementations.test_flush',
+        'bzrlib.tests.workingtree_implementations.test_get_file_with_stat',
         'bzrlib.tests.workingtree_implementations.test_get_file_mtime',
         'bzrlib.tests.workingtree_implementations.test_get_parent_ids',
         'bzrlib.tests.workingtree_implementations.test_inv',
@@ -124,11 +104,12 @@ def load_tests(basic_tests, module, loader):
         'bzrlib.tests.workingtree_implementations.test_smart_add',
         'bzrlib.tests.workingtree_implementations.test_uncommit',
         'bzrlib.tests.workingtree_implementations.test_unversion',
+        'bzrlib.tests.workingtree_implementations.test_views',
         'bzrlib.tests.workingtree_implementations.test_walkdirs',
         'bzrlib.tests.workingtree_implementations.test_workingtree',
         ]
 
-    adapter = WorkingTreeTestProviderAdapter(
+    scenarios = make_scenarios(
         default_transport,
         # None here will cause a readonly decorator to be created
         # by the TestCaseWithTransport.get_readonly_transport method.
@@ -136,5 +117,6 @@ def load_tests(basic_tests, module, loader):
         WorkingTreeFormat._formats.values() + _legacy_formats)
 
     # add the tests for the sub modules
-    adapt_modules(test_workingtree_implementations, adapter, loader, result)
-    return result
+    return multiply_tests(
+        loader.loadTestsFromModuleNames(test_workingtree_implementations),
+        scenarios, standard_tests)
