@@ -32,7 +32,12 @@ from bzrlib.inventory import (
     InventoryLink,
     )
 from bzrlib.revision import Revision
-from bzrlib.tests import SymlinkFeature, TestNotApplicable
+from bzrlib.tests import (
+    KnownFailure,
+    SymlinkFeature,
+    TestNotApplicable,
+    UnicodeFilenameFeature,
+    )
 from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
 from bzrlib.uncommit import uncommit
 
@@ -227,6 +232,26 @@ class TestSetParents(TestParents):
                             (rev3, rev_tree3)])
         self.assertConsistentParents([rev2, rev3], t)
 
+    def test_unicode_symlink(self):
+        # this tests bug #272444
+        self.requireFeature(SymlinkFeature)
+        self.requireFeature(UnicodeFilenameFeature)
+
+        tree = self.make_branch_and_tree('tree1')
+
+        # The link points to a file whose name is an omega
+        # U+03A9 GREEK CAPITAL LETTER OMEGA
+        # UTF-8: ce a9  UTF-16BE: 03a9  Decimal: &#937;
+        os.symlink(u'\u03a9','tree1/link_name')
+        tree.add(['link_name'],['link-id'])
+
+        # the actual commit occurs without errors (strangely):
+        revision1 = tree.commit('added a link to a Unicode target')
+        # python 2.4 failed with UnicodeDecodeError on this commit:
+        revision2 = tree.commit('this revision will be discarded')
+        # python 2.5 failed with UnicodeEncodeError on set_parent_ids:
+        tree.set_parent_ids([revision1])
+
 
 class TestAddParent(TestParents):
 
@@ -237,13 +262,13 @@ class TestAddParent(TestParents):
         uncommit(tree.branch, tree=tree)
         tree.add_parent_tree_id(first_revision)
         self.assertConsistentParents([first_revision], tree)
-        
+
     def test_add_first_parent_id_ghost_rejects(self):
         """Test adding the first parent id - as a ghost"""
         tree = self.make_branch_and_tree('.')
         self.assertRaises(errors.GhostRevisionUnusableHere,
             tree.add_parent_tree_id, 'first-revision')
-        
+
     def test_add_first_parent_id_ghost_force(self):
         """Test adding the first parent id - as a ghost"""
         tree = self.make_branch_and_tree('.')
@@ -256,7 +281,7 @@ class TestAddParent(TestParents):
         tree.add_parent_tree_id('first-revision', allow_leftmost_as_ghost=True)
         tree.add_parent_tree_id('second')
         self.assertConsistentParents(['first-revision', 'second'], tree)
-        
+
     def test_add_second_parent_id(self):
         """Test adding the second parent id"""
         tree = self.make_branch_and_tree('.')
@@ -265,14 +290,14 @@ class TestAddParent(TestParents):
         second_revision = tree.commit('second post')
         tree.add_parent_tree_id(first_revision)
         self.assertConsistentParents([second_revision, first_revision], tree)
-        
+
     def test_add_second_parent_id_ghost(self):
         """Test adding the second parent id - as a ghost"""
         tree = self.make_branch_and_tree('.')
         first_revision = tree.commit('first post')
         tree.add_parent_tree_id('second')
         self.assertConsistentParents([first_revision, 'second'], tree)
-        
+
     def test_add_first_parent_tree(self):
         """Test adding the first parent id"""
         tree = self.make_branch_and_tree('.')
@@ -281,20 +306,20 @@ class TestAddParent(TestParents):
         tree.add_parent_tree((first_revision,
             tree.branch.repository.revision_tree(first_revision)))
         self.assertConsistentParents([first_revision], tree)
-        
+
     def test_add_first_parent_tree_ghost_rejects(self):
         """Test adding the first parent id - as a ghost"""
         tree = self.make_branch_and_tree('.')
         self.assertRaises(errors.GhostRevisionUnusableHere,
             tree.add_parent_tree, ('first-revision', None))
-        
+
     def test_add_first_parent_tree_ghost_force(self):
         """Test adding the first parent id - as a ghost"""
         tree = self.make_branch_and_tree('.')
         tree.add_parent_tree(('first-revision', None),
             allow_leftmost_as_ghost=True)
         self.assertConsistentParents(['first-revision'], tree)
-        
+
     def test_add_second_parent_tree(self):
         """Test adding the second parent id"""
         tree = self.make_branch_and_tree('.')
@@ -304,7 +329,7 @@ class TestAddParent(TestParents):
         tree.add_parent_tree((first_revision,
             tree.branch.repository.revision_tree(first_revision)))
         self.assertConsistentParents([second_revision, first_revision], tree)
-        
+
     def test_add_second_parent_tree_ghost(self):
         """Test adding the second parent id - as a ghost"""
         tree = self.make_branch_and_tree('.')
@@ -315,7 +340,7 @@ class TestAddParent(TestParents):
 
 class UpdateToOneParentViaDeltaTests(TestCaseWithWorkingTree):
     """Tests for the update_basis_by_delta call.
-    
+
     This is intuitively defined as 'apply an inventory delta to the basis and
     discard other parents', but for trees that have an inventory that is not
     managed as a tree-by-id, the implementation requires roughly duplicated
@@ -337,8 +362,10 @@ class UpdateToOneParentViaDeltaTests(TestCaseWithWorkingTree):
         # the delta.
         result_basis = tree.basis_tree()
         result_basis.lock_read()
-        self.addCleanup(result_basis.unlock)
-        self.assertEqual(expected_inventory, result_basis.inventory)
+        try:
+            self.assertEqual(expected_inventory, result_basis.inventory)
+        finally:
+            result_basis.unlock()
 
     def make_inv_delta(self, old, new):
         """Make an inventory delta from two inventories."""
