@@ -41,6 +41,7 @@ from bzrlib import (
     errors,
     osutils,
     symbol_versioning,
+    ui,
     urlutils,
     )
 """)
@@ -84,7 +85,7 @@ def _clear_protocol_handlers():
 def _get_transport_modules():
     """Return a list of the modules providing transports."""
     modules = set()
-    for prefix, factory_list in transport_list_registry.iteritems():
+    for prefix, factory_list in transport_list_registry.items():
         for factory in factory_list:
             if hasattr(factory, "_module_name"):
                 modules.add(factory._module_name)
@@ -100,7 +101,7 @@ def _get_transport_modules():
 class TransportListRegistry(registry.Registry):
     """A registry which simplifies tracking available Transports.
 
-    A registration of a new protocol requires two step:
+    A registration of a new protocol requires two steps:
     1) register the prefix with the function register_transport( )
     2) register the protocol provider with the function
     register_transport_provider( ) ( and the "lazy" variant )
@@ -242,7 +243,7 @@ class FileStream(object):
 
 class FileFileStream(FileStream):
     """A file stream object returned by open_write_stream.
-    
+
     This version uses a file like object to perform writes.
     """
 
@@ -254,12 +255,12 @@ class FileFileStream(FileStream):
         self.file_handle.close()
 
     def write(self, bytes):
-        self.file_handle.write(bytes)
+        osutils.pump_string_file(bytes, self.file_handle)
 
 
 class AppendBasedFileStream(FileStream):
     """A file stream object returned by open_write_stream.
-    
+
     This version uses append on a transport to perform writes.
     """
 
@@ -272,7 +273,7 @@ class Transport(object):
     from/to a storage location.
 
     Most functions have a _multi variant, which allows you to queue up
-    multiple requests. They generally have a dumb base implementation 
+    multiple requests. They generally have a dumb base implementation
     which just iterates over the arguments, but smart Transport
     implementations can do pipelining.
     In general implementations should support having a generator or a list
@@ -293,7 +294,7 @@ class Transport(object):
     _bytes_to_read_before_seek = 0
 
     def __init__(self, base):
-        super(Transport, self).__init__(base=base)
+        super(Transport, self).__init__()
         self.base = base
 
     def _translate_error(self, e, path, raise_generic=True):
@@ -323,7 +324,7 @@ class Transport(object):
 
     def clone(self, offset=None):
         """Return a new Transport object, cloned from the current location,
-        using a subdirectory or parent directory. This allows connections 
+        using a subdirectory or parent directory. This allows connections
         to be pooled, rather than a new one needed for each subdir.
         """
         raise NotImplementedError(self.clone)
@@ -367,7 +368,7 @@ class Transport(object):
         raise NotImplementedError(self.external_url)
 
     def _pump(self, from_file, to_file):
-        """Most children will need to copy from one file-like 
+        """Most children will need to copy from one file-like
         object or string to another one.
         This just gives them something easy to call.
         """
@@ -382,6 +383,18 @@ class Transport(object):
         except TypeError: # We can't tell how many, because relpaths is a generator
             return None
 
+    def _report_activity(self, bytes, direction):
+        """Notify that this transport has activity.
+
+        Implementations should call this from all methods that actually do IO.
+        Be careful that it's not called twice, if one method is implemented on
+        top of another.
+
+        :param bytes: Number of bytes read or written.
+        :param direction: 'read' or 'write' or None.
+        """
+        ui.ui_factory.report_transport_activity(self, bytes, direction)
+
     def _update_pb(self, pb, msg, count, total):
         """Update the progress bar based on the current count
         and total available, total may be None if it was
@@ -439,7 +452,7 @@ class Transport(object):
             t._combine_paths('/home/sarah', '/etc')
                 => '/etc'
 
-        :param base_path: urlencoded path for the transport root; typically a 
+        :param base_path: urlencoded path for the transport root; typically a
              URL but need not contain scheme/host/etc.
         :param relpath: relative url string for relative part of remote path.
         :return: urlencoded string for final path.
@@ -472,7 +485,7 @@ class Transport(object):
         """Return the recommended page size for this transport.
 
         This is potentially different for every path in a given namespace.
-        For example, local transports might use an operating system call to 
+        For example, local transports might use an operating system call to
         get the block size for a given path, which can vary due to mount
         points.
 
@@ -484,8 +497,8 @@ class Transport(object):
         """Return the local path portion from a given absolute path.
 
         This default implementation is not suitable for filesystems with
-        aliasing, such as that given by symlinks, where a path may not 
-        start with our base, but still be a relpath once aliasing is 
+        aliasing, such as that given by symlinks, where a path may not
+        start with our base, but still be a relpath once aliasing is
         resolved.
         """
         # TODO: This might want to use bzrlib.osutils.relpath
@@ -506,9 +519,9 @@ class Transport(object):
 
     def has(self, relpath):
         """Does the file relpath exist?
-        
+
         Note that some transports MAY allow querying on directories, but this
-        is not part of the protocol.  In other words, the results of 
+        is not part of the protocol.  In other words, the results of
         t.has("a_directory_name") are undefined.
 
         :rtype: bool
@@ -535,7 +548,7 @@ class Transport(object):
         """Iter the relative paths of files in the transports sub-tree.
 
         *NOTE*: This only lists *files*, not subdirectories!
-        
+
         As with other listing functions, only some transports implement this,.
         you may check via listable() to determine if it will.
         """
@@ -568,7 +581,11 @@ class Transport(object):
 
         :param relpath: The relative path to the file
         """
-        return self.get(relpath).read()
+        f = self.get(relpath)
+        try:
+            return f.read()
+        finally:
+            f.close()
 
     @deprecated_method(one_four)
     def get_smart_client(self):
@@ -673,7 +690,7 @@ class Transport(object):
         # Cache the results, but only until they have been fulfilled
         data_map = {}
         for c_offset in coalesced:
-            # TODO: jam 20060724 it might be faster to not issue seek if 
+            # TODO: jam 20060724 it might be faster to not issue seek if
             #       we are already at the right location. This should be
             #       benchmarked.
             fp.seek(c_offset.start)
@@ -688,8 +705,21 @@ class Transport(object):
             # Now that we've read some data, see if we can yield anything back
             while cur_offset_and_size in data_map:
                 this_data = data_map.pop(cur_offset_and_size)
-                yield cur_offset_and_size[0], this_data
-                cur_offset_and_size = offset_stack.next()
+                this_offset = cur_offset_and_size[0]
+                try:
+                    cur_offset_and_size = offset_stack.next()
+                except StopIteration:
+                    # Close the file handle as there will be no more data
+                    # The handle would normally be cleaned up as this code goes
+                    # out of scope, but as we are a generator, not all code
+                    # will re-enter once we have consumed all the expected
+                    # data. For example:
+                    #   zip(range(len(requests)), readv(foo, requests))
+                    # Will stop because the range is done, and not run the
+                    # cleanup code for the readv().
+                    fp.close()
+                    cur_offset_and_size = None
+                yield this_offset, this_data
 
     def _sort_expand_and_combine(self, offsets, upper_limit):
         """Helper for readv.
@@ -758,29 +788,35 @@ class Transport(object):
         into a single large request, while retaining the original
         offsets.
         Turns  [(15, 10), (25, 10)] => [(15, 20, [(0, 10), (10, 10)])]
+        Note that overlapping requests are not permitted. (So [(15, 10), (20,
+        10)] will raise a ValueError.) This is because the data we access never
+        overlaps, and it allows callers to trust that we only need any byte of
+        data for 1 request (so nothing needs to be buffered to fulfill a second
+        request.)
 
         :param offsets: A list of (start, length) pairs
-
         :param limit: Only combine a maximum of this many pairs Some transports
                 penalize multiple reads more than others, and sometimes it is
                 better to return early.
                 0 means no limit
-
         :param fudge_factor: All transports have some level of 'it is
-                better to read some more data and throw it away rather 
+                better to read some more data and throw it away rather
                 than seek', so collapse if we are 'close enough'
-
         :param max_size: Create coalesced offsets no bigger than this size.
                 When a single offset is bigger than 'max_size', it will keep
                 its size and be alone in the coalesced offset.
                 0 means no maximum size.
-
-        :return: yield _CoalescedOffset objects, which have members for where
-                to start, how much to read, and how to split those 
-                chunks back up
+        :return: return a list of _CoalescedOffset objects, which have members
+            for where to start, how much to read, and how to split those chunks
+            back up
         """
         last_end = None
         cur = _CoalescedOffset(None, None, [])
+        coalesced_offsets = []
+
+        if max_size <= 0:
+            # 'unlimited', but we actually take this to mean 100MB buffer limit
+            max_size = 100*1024*1024
 
         for start, size in offsets:
             end = start + size
@@ -789,18 +825,21 @@ class Transport(object):
                 and start >= cur.start
                 and (limit <= 0 or len(cur.ranges) < limit)
                 and (max_size <= 0 or end - cur.start <= max_size)):
+                if start < last_end:
+                    raise ValueError('Overlapping range not allowed:'
+                        ' last range ended at %s, new one starts at %s'
+                        % (last_end, start))
                 cur.length = end - cur.start
                 cur.ranges.append((start-cur.start, size))
             else:
                 if cur.start is not None:
-                    yield cur
+                    coalesced_offsets.append(cur)
                 cur = _CoalescedOffset(start, size, [(0, size)])
             last_end = end
 
         if cur.start is not None:
-            yield cur
-
-        return
+            coalesced_offsets.append(cur)
+        return coalesced_offsets
 
     def get_multi(self, relpaths, pb=None):
         """Get a list of file-like objects, one for each entry in relpaths.
@@ -838,7 +877,7 @@ class Transport(object):
                              dir_mode=None):
         """Copy the string into the target location.
 
-        This function is not strictly safe to use. See 
+        This function is not strictly safe to use. See
         Transport.put_bytes_non_atomic for more information.
 
         :param relpath: The remote location to put the contents.
@@ -926,7 +965,7 @@ class Transport(object):
         be synchronised with any internal buffering that may be present.
 
         :param relpath: The relative path to the file.
-        :param mode: The mode for the newly created file, 
+        :param mode: The mode for the newly created file,
                      None means just use the default
         :return: A FileStream. FileStream objects have two methods, write() and
             close(). There is no guarantee that data is committed to the file
@@ -981,15 +1020,15 @@ class Transport(object):
 
     def copy(self, rel_from, rel_to):
         """Copy the item at rel_from to the location at rel_to.
-        
-        Override this for efficiency if a specific transport can do it 
+
+        Override this for efficiency if a specific transport can do it
         faster than this default implementation.
         """
         self.put_file(rel_to, self.get(rel_from))
 
     def copy_multi(self, relpaths, pb=None):
         """Copy a bunch of entries.
-        
+
         :param relpaths: A list of tuples of the form [(from, to), (from, to),...]
         """
         # This is the non-pipelined implementation, so that
@@ -1013,26 +1052,35 @@ class Transport(object):
     def copy_tree(self, from_relpath, to_relpath):
         """Copy a subtree from one relpath to another.
 
-        If a faster implementation is available, specific transports should 
+        If a faster implementation is available, specific transports should
         implement it.
         """
         source = self.clone(from_relpath)
-        self.mkdir(to_relpath)
         target = self.clone(to_relpath)
+        target.mkdir('.')
+        source.copy_tree_to_transport(target)
+
+    def copy_tree_to_transport(self, to_transport):
+        """Copy a subtree from one transport to another.
+
+        self.base is used as the source tree root, and to_transport.base
+        is used as the target.  to_transport.base must exist (and be a
+        directory).
+        """
         files = []
         directories = ['.']
         while directories:
             dir = directories.pop()
             if dir != '.':
-                target.mkdir(dir)
-            for path in source.list_dir(dir):
+                to_transport.mkdir(dir)
+            for path in self.list_dir(dir):
                 path = dir + '/' + path
-                stat = source.stat(path)
+                stat = self.stat(path)
                 if S_ISDIR(stat.st_mode):
                     directories.append(path)
                 else:
                     files.append(path)
-        source.copy_to(files, target)
+        self.copy_to(files, to_transport)
 
     def rename(self, rel_from, rel_to):
         """Rename a file or directory.
@@ -1058,7 +1106,7 @@ class Transport(object):
 
         The destination is deleted if possible, even if it's a non-empty
         directory tree.
-        
+
         If a transport can directly implement this it is suggested that
         it do so for efficiency.
         """
@@ -1071,7 +1119,7 @@ class Transport(object):
 
     def move_multi(self, relpaths, pb=None):
         """Move a bunch of entries.
-        
+
         :param relpaths: A list of tuples of the form [(from1, to1), (from2, to2),...]
         """
         return self._iterate_over(relpaths, self.move, pb, 'move', expand=True)
@@ -1130,7 +1178,7 @@ class Transport(object):
         NOTE: This returns an object with fields such as 'st_size'. It MAY
         or MAY NOT return the literal result of an os.stat() call, so all
         access should be via named fields.
-        ALSO NOTE: Stats of directories may not be supported on some 
+        ALSO NOTE: Stats of directories may not be supported on some
         transports.
         """
         raise NotImplementedError(self.stat)
@@ -1173,7 +1221,7 @@ class Transport(object):
         These methods may be removed in the future.
 
         Transports may raise TransportNotPossible if OS-level locks cannot be
-        taken over this transport.  
+        taken over this transport.
 
         :return: A lock object, which should contain an unlock() function.
         """
@@ -1200,12 +1248,12 @@ class Transport(object):
         """Return true if this transport can store and retrieve unix modebits.
 
         (For example, 0700 to make a directory owner-private.)
-        
-        Note: most callers will not want to switch on this, but should rather 
+
+        Note: most callers will not want to switch on this, but should rather
         just try and set permissions and let them be either stored or not.
         This is intended mainly for the use of the test suite.
-        
-        Warning: this is not guaranteed to be accurate as sometimes we can't 
+
+        Warning: this is not guaranteed to be accurate as sometimes we can't
         be sure: for example with vfat mounted on unix, or a windows sftp
         server."""
         # TODO: Perhaps return a e.g. TransportCharacteristics that can answer
@@ -1218,6 +1266,22 @@ class Transport(object):
         # should be asked to ConnectedTransport only.
         return None
 
+    def _redirected_to(self, source, target):
+        """Returns a transport suitable to re-issue a redirected request.
+
+        :param source: The source url as returned by the server.
+        :param target: The target url as returned by the server.
+
+        The redirection can be handled only if the relpath involved is not
+        renamed by the redirection.
+
+        :returns: A transport or None.
+        """
+        # This returns None by default, meaning the transport can't handle the
+        # redirection.
+        return None
+
+
 
 class _SharedConnection(object):
     """A connection shared between several transports."""
@@ -1389,7 +1453,7 @@ class ConnectedTransport(Transport):
 
     def abspath(self, relpath):
         """Return the full url to the given relative path.
-        
+
         :param relpath: the relative path urlencoded
 
         :returns: the Unicode version of the absolute path for relpath.
@@ -1455,7 +1519,7 @@ class ConnectedTransport(Transport):
 
         Some protocols can renegociate the credentials within a connection,
         this method allows daughter classes to share updated credentials.
-        
+
         :param credentials: the updated credentials.
         """
         # We don't want to call _set_connection here as we are only updating
@@ -1529,7 +1593,7 @@ def get_transport(base, possible_transports=None):
     def convert_path_to_url(base, error_str):
         m = _urlRE.match(base)
         if m:
-            # This looks like a URL, but we weren't able to 
+            # This looks like a URL, but we weren't able to
             # instantiate it as such raise an appropriate error
             # FIXME: we have a 'error_str' unused and we use last_err below
             raise errors.UnsupportedProtocol(base, last_err)
@@ -1556,7 +1620,7 @@ def get_transport(base, possible_transports=None):
                     possible_transports.append(t_same_connection)
                 return t_same_connection
 
-    for proto, factory_list in transport_list_registry.iteritems():
+    for proto, factory_list in transport_list_registry.items():
         if proto is not None and base.startswith(proto):
             transport, last_err = _try_transport_factories(base, factory_list)
             if transport:
@@ -1601,7 +1665,7 @@ def do_catching_redirections(action, transport, redirected):
     :param action: A callable, what the caller want to do while catching
                   redirections.
     :param transport: The initial transport used.
-    :param redirected: A callable receiving the redirected transport and the 
+    :param redirected: A callable receiving the redirected transport and the
                   RedirectRequested exception.
 
     :return: Whatever 'action' returns
@@ -1634,12 +1698,12 @@ def do_catching_redirections(action, transport, redirected):
 
 class Server(object):
     """A Transport Server.
-    
+
     The Server interface provides a server for a given transport. We use
     these servers as loopback testing tools. For any given transport the
     Servers it provides must either allow writing, or serve the contents
     of os.getcwdu() at the time setUp is called.
-    
+
     Note that these are real servers - they must implement all the things
     that we want bzr transports to take advantage of.
     """
@@ -1652,20 +1716,20 @@ class Server(object):
 
     def get_url(self):
         """Return a url for this server.
-        
-        If the transport does not represent a disk directory (i.e. it is 
+
+        If the transport does not represent a disk directory (i.e. it is
         a database like svn, or a memory only transport, it should return
         a connection to a newly established resource for this Server.
         Otherwise it should return a url that will provide access to the path
         that was os.getcwdu() when setUp() was called.
-        
+
         Subsequent calls will return the same resource.
         """
         raise NotImplementedError
 
     def get_bogus_url(self):
         """Return a url for this protocol, that will fail to connect.
-        
+
         This may raise NotImplementedError to indicate that this server cannot
         provide bogus urls.
         """
@@ -1708,12 +1772,13 @@ register_transport_proto('http://',
                  help="Read-only access of branches exported on the web.")
 register_transport_proto('https://',
             help="Read-only access of branches exported on the web using SSL.")
+# The default http implementation is urllib, but https is pycurl if available
+register_lazy_transport('http://', 'bzrlib.transport.http._pycurl',
+                        'PyCurlTransport')
 register_lazy_transport('http://', 'bzrlib.transport.http._urllib',
                         'HttpTransport_urllib')
 register_lazy_transport('https://', 'bzrlib.transport.http._urllib',
                         'HttpTransport_urllib')
-register_lazy_transport('http://', 'bzrlib.transport.http._pycurl',
-                        'PyCurlTransport')
 register_lazy_transport('https://', 'bzrlib.transport.http._pycurl',
                         'PyCurlTransport')
 
@@ -1722,6 +1787,25 @@ register_lazy_transport('ftp://', 'bzrlib.transport.ftp', 'FtpTransport')
 register_transport_proto('aftp://', help="Access using active FTP.")
 register_lazy_transport('aftp://', 'bzrlib.transport.ftp', 'FtpTransport')
 
+# Default to trying GSSAPI authentication (if the kerberos module is available)
+register_transport_proto('ftp+gssapi://', register_netloc=True)
+register_lazy_transport('ftp+gssapi://', 'bzrlib.transport.ftp._gssapi',
+                        'GSSAPIFtpTransport')
+register_transport_proto('aftp+gssapi://', register_netloc=True)
+register_lazy_transport('aftp+gssapi://', 'bzrlib.transport.ftp._gssapi',
+                        'GSSAPIFtpTransport')
+register_transport_proto('ftp+nogssapi://', register_netloc=True)
+register_transport_proto('aftp+nogssapi://', register_netloc=True)
+
+register_lazy_transport('ftp://', 'bzrlib.transport.ftp._gssapi',
+                        'GSSAPIFtpTransport')
+register_lazy_transport('aftp://', 'bzrlib.transport.ftp._gssapi',
+                        'GSSAPIFtpTransport')
+register_lazy_transport('ftp+nogssapi://', 'bzrlib.transport.ftp',
+                        'FtpTransport')
+register_lazy_transport('aftp+nogssapi://', 'bzrlib.transport.ftp',
+                        'FtpTransport')
+
 register_transport_proto('memory://')
 register_lazy_transport('memory://', 'bzrlib.transport.memory',
                         'MemoryTransport')
@@ -1739,6 +1823,9 @@ register_transport_proto('fakenfs+')
 register_lazy_transport('fakenfs+', 'bzrlib.transport.fakenfs',
                         'FakeNFSTransportDecorator')
 
+register_transport_proto('log+')
+register_lazy_transport('log+', 'bzrlib.transport.log', 'TransportLogDecorator')
+
 register_transport_proto('trace+')
 register_lazy_transport('trace+', 'bzrlib.transport.trace',
                         'TransportTraceDecorator')
@@ -1792,3 +1879,7 @@ register_transport_proto('bzr+ssh://',
             register_netloc=True)
 register_lazy_transport('bzr+ssh://', 'bzrlib.transport.remote',
                         'RemoteSSHTransport')
+
+register_transport_proto('ssh:')
+register_lazy_transport('ssh:', 'bzrlib.transport.remote',
+                        'HintingSSHTransport')
