@@ -17,9 +17,9 @@
 """Fake transport with some restrictions of Windows VFAT filesystems.
 
 VFAT on Windows has several restrictions that are not present on unix
-filesystems, which are imposed by this transport. 
+filesystems, which are imposed by this transport.
 
-VFAT is strictly 8-bit using codepages to represent non-ascii characters. 
+VFAT is strictly 8-bit using codepages to represent non-ascii characters.
 This implementation currently doesn't model the codepage but just insists
 on only ascii characters being written.
 
@@ -50,7 +50,7 @@ from bzrlib.transport.decorator import TransportDecorator, DecoratorServer
 
 # TODO: Perhaps don't inherit from TransportDecorator so that methods
 # which are not implemented here fail by default?
-    
+
 
 class FakeVFATTransportDecorator(TransportDecorator):
     """A decorator that can convert any transport to be readonly.
@@ -64,6 +64,10 @@ class FakeVFATTransportDecorator(TransportDecorator):
     which actually stored the files.
     """
 
+    def _can_roundtrip_unix_modebits(self):
+        """See Transport._can_roundtrip_unix_modebits()."""
+        return False
+
     @classmethod
     def _get_url_prefix(self):
         """Readonly transport decorators are invoked via 'vfat+'"""
@@ -88,7 +92,7 @@ class FakeVFATTransportDecorator(TransportDecorator):
     def has(self, relpath):
         return self._decorated.has(self._squash_name(relpath))
 
-    def readv(self, relpath, offsets):
+    def _readv(self, relpath, offsets):
         return self._decorated.readv(self._squash_name(relpath), offsets)
 
     def put_file(self, relpath, f, mode=None):
