@@ -25,11 +25,12 @@ from cStringIO import StringIO
 from bzrlib.smart import protocol, medium
 from bzrlib.transport import chroot, get_transport
 from bzrlib.urlutils import local_path_to_url
-    
 
-def make_app(root, prefix, path_var='REQUEST_URI', readonly=True):
+
+def make_app(root, prefix, path_var='REQUEST_URI', readonly=True,
+    load_plugins=True, enable_logging=True):
     """Convenience function to construct a WSGI bzr smart server.
-    
+
     :param root: a local path that requests will be relative to.
     :param prefix: See RelpathSetter.
     :param path_var: See RelpathSetter.
@@ -39,6 +40,12 @@ def make_app(root, prefix, path_var='REQUEST_URI', readonly=True):
         base_transport = get_transport('readonly+' + local_url)
     else:
         base_transport = get_transport(local_url)
+    if load_plugins:
+        from bzrlib.plugin import load_plugins
+        load_plugins()
+    if enable_logging:
+        import bzrlib.trace
+        bzrlib.trace.enable_default_logging()
     app = SmartWSGIApp(base_transport, prefix)
     app = RelpathSetter(app, '', path_var)
     return app
@@ -46,7 +53,7 @@ def make_app(root, prefix, path_var='REQUEST_URI', readonly=True):
 
 class RelpathSetter(object):
     """WSGI middleware to set 'bzrlib.relpath' in the environ.
-    
+
     Different servers can invoke a SmartWSGIApp in different ways.  This
     middleware allows an adminstrator to configure how to the SmartWSGIApp will
     determine what path it should be serving for a given request for many common
@@ -58,7 +65,7 @@ class RelpathSetter(object):
     prefix="/some/prefix/" and path_var="REQUEST_URI" will set that request's
     'bzrlib.relpath' variable to "repo/branch".
     """
-    
+
     def __init__(self, app, prefix='', path_var='REQUEST_URI'):
         """Constructor.
 
@@ -103,8 +110,8 @@ class SmartWSGIApp(object):
         self.backing_transport = get_transport(self.chroot_server.get_url())
         self.root_client_path = root_client_path
         # While the chroot server can technically be torn down at this point,
-        # as all it does is remove the scheme registration from transport's 
-        # protocol dictionary, we don't *just in case* there are parts of 
+        # as all it does is remove the scheme registration from transport's
+        # protocol dictionary, we don't *just in case* there are parts of
         # bzrlib that will invoke 'get_transport' on urls rather than cloning
         # around the existing transport.
         #self.chroot_server.tearDown()
