@@ -33,15 +33,11 @@ class TransportTraceDecorator(TransportDecorator):
     Not all operations are logged at this point, if you need an unlogged
     operation please add a test to the tests of this transport, for the logging
     of the operation you want logged.
-
-    Another future enhancement would be to log to bzrlib.trace.mutter when
-    trace+ is used from the command line (or perhaps as well/instead use
-    -Dtransport), to make tracing operations of the entire program easily.
     """
 
     def __init__(self, url, _decorated=None, _from_transport=None):
         """Set the 'base' path where files will be stored.
-        
+
         _decorated is a private parameter for cloning.
         """
         TransportDecorator.__init__(self, url, _decorated)
@@ -76,7 +72,7 @@ class TransportTraceDecorator(TransportDecorator):
 
     def get(self, relpath):
         """See Transport.get()."""
-        self._activity.append(('get', relpath))
+        self._trace(('get', relpath))
         return self._decorated.get(relpath)
 
     def get_smart_client(self):
@@ -92,6 +88,7 @@ class TransportTraceDecorator(TransportDecorator):
 
     def mkdir(self, relpath, mode=None):
         """See Transport.mkdir()."""
+        self._trace(('mkdir', relpath, mode))
         return self._decorated.mkdir(relpath, mode)
 
     def open_write_stream(self, relpath, mode=None):
@@ -101,12 +98,20 @@ class TransportTraceDecorator(TransportDecorator):
     def put_file(self, relpath, f, mode=None):
         """See Transport.put_file()."""
         return self._decorated.put_file(relpath, f, mode)
-    
+
     def put_bytes(self, relpath, bytes, mode=None):
         """See Transport.put_bytes()."""
-        self._activity.append(('put_bytes', relpath, len(bytes), mode))
+        self._trace(('put_bytes', relpath, len(bytes), mode))
         return self._decorated.put_bytes(relpath, bytes, mode)
 
+    def put_bytes_non_atomic(self, relpath, bytes, mode=None,
+        create_parent_dir=False, dir_mode=None):
+        """See Transport.put_bytes_non_atomic."""
+        self._trace(('put_bytes_non_atomic', relpath, len(bytes), mode,
+            create_parent_dir, dir_mode))
+        return self._decorated.put_bytes_non_atomic(relpath, bytes, mode=mode,
+            create_parent_dir=create_parent_dir, dir_mode=dir_mode)
+
     def listable(self):
         """See Transport.listable."""
         return self._decorated.listable()
@@ -114,7 +119,7 @@ class TransportTraceDecorator(TransportDecorator):
     def iter_files_recursive(self):
         """See Transport.iter_files_recursive()."""
         return self._decorated.iter_files_recursive()
-    
+
     def list_dir(self, relpath):
         """See Transport.list_dir()."""
         return self._decorated.list_dir(relpath)
@@ -122,7 +127,7 @@ class TransportTraceDecorator(TransportDecorator):
     def readv(self, relpath, offsets, adjust_for_latency=False,
         upper_limit=None):
         """See Transport.readv."""
-        self._activity.append(('readv', relpath, offsets, adjust_for_latency,
+        self._trace(('readv', relpath, offsets, adjust_for_latency,
             upper_limit))
         return self._decorated.readv(relpath, offsets, adjust_for_latency,
             upper_limit)
@@ -134,9 +139,10 @@ class TransportTraceDecorator(TransportDecorator):
     def rename(self, rel_from, rel_to):
         self._activity.append(('rename', rel_from, rel_to))
         return self._decorated.rename(rel_from, rel_to)
-    
+
     def rmdir(self, relpath):
         """See Transport.rmdir."""
+        self._trace(('rmdir', relpath))
         return self._decorated.rmdir(relpath)
 
     def stat(self, relpath):
@@ -151,6 +157,13 @@ class TransportTraceDecorator(TransportDecorator):
         """See Transport.lock_write."""
         return self._decorated.lock_write(relpath)
 
+    def _trace(self, operation_tuple):
+        """Record that a transport operation occured.
+
+        :param operation: Tuple of transport call name and arguments.
+        """
+        self._activity.append(operation_tuple)
+
 
 class TraceServer(DecoratorServer):
     """Server for the TransportTraceDecorator for testing with."""
