@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2008, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,9 @@ Set the ui_factory member to define the behaviour.  The default
 displays no output.
 """
 
+import os
 import sys
+import warnings
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -49,8 +51,7 @@ class UIFactory(object):
     """
 
     def __init__(self):
-        super(UIFactory, self).__init__()
-        self._progress_bar_stack = None
+        self._task_stack = []
 
     def get_password(self, prompt='', **kwargs):
         """Prompt the user for a password.
@@ -73,17 +74,47 @@ class UIFactory(object):
         When the bar has been finished with, it should be released by calling
         bar.finished().
         """
-        raise NotImplementedError(self.nested_progress_bar)
+        if self._task_stack:
+            t = progress.ProgressTask(self._task_stack[-1], self)
+        else:
+            t = progress.ProgressTask(None, self)
+        self._task_stack.append(t)
+        return t
+
+    def _progress_finished(self, task):
+        """Called by the ProgressTask when it finishes"""
+        if not self._task_stack:
+            warnings.warn("%r finished but nothing is active"
+                % (task,))
+        elif task != self._task_stack[-1]:
+            warnings.warn("%r is not the active task %r"
+                % (task, self._task_stack[-1]))
+        else:
+            del self._task_stack[-1]
+        if not self._task_stack:
+            self._progress_all_finished()
+
+    def _progress_all_finished(self):
+        """Called when the top-level progress task finished"""
+        pass
+
+    def _progress_updated(self, task):
+        """Called by the ProgressTask when it changes.
+
+        Should be specialized to draw the progress.
+        """
+        pass
 
     def clear_term(self):
         """Prepare the terminal for output.
 
         This will, for example, clear text progress bars, and leave the
-        cursor at the leftmost position."""
-        raise NotImplementedError(self.clear_term)
+        cursor at the leftmost position.
+        """
+        pass
 
     def get_boolean(self, prompt):
-        """Get a boolean question answered from the user. 
+        """Get a boolean question answered from the user.
 
         :param prompt: a message to prompt the user with. Should be a single
         line without terminating \n.
@@ -104,13 +135,26 @@ class UIFactory(object):
             current_format_name,
             basedir)
 
+    def report_transport_activity(self, transport, byte_count, direction):
+        """Called by transports as they do IO.
+
+        This may update a progress bar, spinner, or similar display.
+        By default it does nothing.
+        """
+        pass
+
+
 
 class CLIUIFactory(UIFactory):
-    """Common behaviour for command line UI factories."""
+    """Common behaviour for command line UI factories.
 
-    def __init__(self):
-        super(CLIUIFactory, self).__init__()
-        self.stdin = sys.stdin
+    This is suitable for dumb terminals that can't repaint existing text."""
+
+    def __init__(self, stdin=None, stdout=None, stderr=None):
+        UIFactory.__init__(self)
+        self.stdin = stdin or sys.stdin
+        self.stdout = stdout or sys.stdout
+        self.stderr = stderr or sys.stderr
 
     def get_boolean(self, prompt):
         self.clear_term()
@@ -136,7 +180,7 @@ class CLIUIFactory(UIFactory):
         :param kwargs: Arguments which will be expanded into the prompt.
                        This lets front ends display different things if
                        they so choose.
-        :return: The password string, return None if the user 
+        :return: The password string, return None if the user
                  canceled the request.
         """
         prompt += ': '
@@ -147,6 +191,11 @@ class CLIUIFactory(UIFactory):
 
     def prompt(self, prompt):
         """Emit prompt on the CLI."""
+        self.stdout.write(prompt)
+
+    def note(self, msg):
+        """Write an already-formatted message."""
+        self.stdout.write(msg + '\n')
 
 
 class SilentUIFactory(CLIUIFactory):
@@ -155,19 +204,16 @@ class SilentUIFactory(CLIUIFactory):
     This is the default UI, if another one is never registered.
     """
 
+    def __init__(self):
+        CLIUIFactory.__init__(self)
+
     def get_password(self, prompt='', **kwargs):
         return None
 
-    def nested_progress_bar(self):
-        if self._progress_bar_stack is None:
-            self._progress_bar_stack = progress.ProgressBarStack(
-                klass=progress.DummyProgress)
-        return self._progress_bar_stack.get_nested()
-
-    def clear_term(self):
+    def prompt(self, prompt):
         pass
 
-    def recommend_upgrade(self, *args):
+    def note(self, msg):
         pass
 
 
@@ -178,5 +224,28 @@ def clear_decorator(func, *args, **kwargs):
 
 
 ui_factory = SilentUIFactory()
-"""IMPORTANT: never import this symbol directly. ONLY ever access it as 
+"""IMPORTANT: never import this symbol directly. ONLY ever access it as
 ui.ui_factory."""
+
+
+def make_ui_for_terminal(stdin, stdout, stderr):
+    """Construct and return a suitable UIFactory for a text mode program.
+
+    If stdout is a smart terminal, this gets a smart UIFactory with
+    progress indicators, etc.  If it's a dumb terminal, just plain text output.
+    """
+    cls = None
+    isatty = getattr(stdin, 'isatty', None)
+    if isatty is None:
+        cls = CLIUIFactory
+    elif not isatty():
+        cls = CLIUIFactory
+    elif os.environ.get('TERM') in ('dumb', ''):
+        # e.g. emacs compile window
+        cls = CLIUIFactory
+    # User may know better, otherwise default to TextUIFactory
+    if (   os.environ.get('BZR_USE_TEXT_UI', None) is not None
+        or cls is None):
+        from bzrlib.ui.text import TextUIFactory
+        cls = TextUIFactory
+    return cls(stdin=stdin, stdout=stdout, stderr=stderr)
