@@ -145,7 +145,7 @@ def format_command (params, cmd):
 
 
 man_preamble = """\
-Man page for Bazaar (%(bzrcmd)s)
+.\\\"Man page for Bazaar (%(bzrcmd)s)
 .\\\"
 .\\\" Large parts of this file are autogenerated from the output of
 .\\\"     \"%(bzrcmd)s help commands\"
